#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <string>
#include <vector>
#include <gps.h>


using namespace std;


#define FALSE 0
#define TRUE 1
#define RTCM_ANTFREQ 10

unsigned int rtcm_antfreq;
unsigned int rtcm_info_count;
string rtcm_antfile = "TEST.antenna";
struct rtcm_t rtcm;
int rtcm_refstaid;
vector<unsigned char> rtcm_packet;
vector<unsigned char> rtcm3_packet;

unsigned int   gpsMinute;
unsigned short gpsFrac;

extern "C" {
extern int rtcm_undump(/*@out@*/struct rtcm_t *, char *);
extern int rtcm_dump(/*@out@*/struct rtcm_t *, char *, size_t);
extern bool rtcm_repack(struct rtcm_t *, isgps30bits_t *);
extern int rtcm3_repack(struct rtcm_t *, unsigned char *);
extern int rtcm3_unpack(struct rtcm_t *, unsigned char *);
extern void rtcm_buffer_magnavox(isgps30bits_t * ip, unsigned char *, int *);
}

int MakeRTCMInfoRecords()
{

	enum {rtcm_3, rtcm_22, rtcm_23, rtcm_24, rtcm_1005, rtcm_1006, rtcm_1007, rtcm_1008, rtcm_MAX};
	struct rtcm_t rtcm_info[8]; // space for all record types

	char buf[BUFSIZ];
	unsigned char buf2[BUFSIZ];
	int i;
	bool rtcm22 = FALSE;
	int offset = 0;
	char ch;
	isgps30bits_t isgpsbuf[RTCM_WORDS_MAX];
	// unsigned char ucbuf[RTCM_WORDS_MAX];
	unsigned char ucbuf[1024];
	printf("MakeRTCMInfoRecords: Entering...\n");

	printf("MakeRTCMInfoRecords: rtcm_info_count = %d, rtcm_antfreq = %d\n", rtcm_info_count, rtcm_antfreq);  

	if (rtcm_info_count < rtcm_antfreq) {
		printf("MakeRTCMInfoRecords: Exiting (rtcm_info_count = %d, rtcm_antfreq = %d)\n",rtcm_info_count, rtcm_antfreq);
		rtcm_info_count++;
		return(0);
	}
	rtcm_info_count = 0;

// line 1 = version 2.2 or 2.3
	printf("MakeRTCMInfoRecords: Opening %s\n",rtcm_antfile.c_str());

	FILE *in = fopen(rtcm_antfile.c_str(),"r");
	if (in == NULL) {
		printf("MakeRTCMInfoRecords: Unable to open %s: %s\n",rtcm_antfile.c_str(), strerror(errno));
		return(-1);
	}
	fgets(buf, BUFSIZ, in);
        int fldcount = sscanf(buf, "%c\t%u\t%u\n",&ch,&rtcm_refstaid,&rtcm_antfreq);
	switch (fldcount) {
		case 1:
			rtcm_refstaid = 0;
		case 2:
			rtcm_antfreq = RTCM_ANTFREQ;
		case 3:
			if (ch == '2') rtcm22 = TRUE;
		default:
			break;
	}
	printf("MakeRTCMInfoRecords: read config line(fldcount = %d): RTCM v2.%c, rtcm_refstaid=%d, frequency=%d\n",fldcount,ch,rtcm_refstaid,rtcm_antfreq);

	int haverec = 0;
	rtcm_t tmp;
	memset(&tmp, 0, sizeof(rtcm_t));

	while (fgets(buf, BUFSIZ, in)) {
	        printf("MakeRTCMInfoRecords: buf=%s", buf);
		i = rtcm_undump(&tmp, buf);
//		if (i < 0) break;
		if (i == 0) {
			unsigned short s = rtcm_MAX;
			if (tmp.type == 3) s = rtcm_3;
			if (tmp.type == 22) s = rtcm_22;
			if (tmp.type == 23) s = rtcm_23;
			if (tmp.type == 24) s = rtcm_24;
			if (tmp.type == 1005) s = rtcm_1005;
			if (tmp.type == 1006) s = rtcm_1006;
			if (tmp.type == 1007) s = rtcm_1007;
			if (tmp.type == 1008) {
			  s = rtcm_1008;
			}
			printf("MakeRTCMInfoRecords: type=%d\n\n", tmp.type);
			haverec |= (1 << s);
			memcpy(&rtcm_info[s], &tmp, sizeof(rtcm_t));
			memset(&tmp, 0, sizeof(rtcm_t));
			rtcm_info[s].refstaid = rtcm_refstaid;
		}
	}
	fclose(in);

// RTCM 2.X handling

	printf("MakeRTCMInfoRecords: Creating rtcmoutput\n");
	offset = 0;
	if (rtcm22) {
		if (haverec & (1 << rtcm_3)) {
			rtcm_repack(&rtcm_info[rtcm_3],isgpsbuf);
			rtcm_buffer_magnavox(isgpsbuf, buf2, &offset);
			printf("MakeRTCMInfoRecords: packed rtcm_3 with %d characters\n",offset);
		}
		if (haverec & (1 << rtcm_22)) {
			rtcm_repack(&rtcm_info[rtcm_22],isgpsbuf);
			rtcm_buffer_magnavox(isgpsbuf, buf2, &offset);
			printf("MakeRTCMInfoRecords: packed rtcm_22 with %d characters\n",offset);
		}
	} else {
		if (haverec & (1 << rtcm_23)) {
			rtcm_repack(&rtcm_info[rtcm_23],isgpsbuf);
			rtcm_buffer_magnavox(isgpsbuf, buf2, &offset);
			printf("MakeRTCMInfoRecords: packed rtcm_23 with %d characters\n",offset);
		}
		if (haverec & (1 << rtcm_24)) {
			rtcm_repack(&rtcm_info[rtcm_24],isgpsbuf);
			rtcm_buffer_magnavox(isgpsbuf, buf2, &offset);
			printf("MakeRTCMInfoRecords: packed rtcm_24 with %d characters\n",offset);
		}
	}

#if FOO
	printf("StreamType::MakeRTCMInfoRecords: Copying %d characters into RTCM2 Packet\n",offset);
	for (int i = 0; i < offset; i++) {
		rtcm_packet.push_back(buf2[i]);
	}
#endif

// Handle RTCM 3 packets

	if (haverec & (1 << rtcm_1006)) {
		offset = rtcm3_repack(&rtcm_info[rtcm_1006], ucbuf);
		printf("MakeRTCMInfoRecords: packed rtcm_1006 with %d characters\n",offset);
	} else if ( haverec & (1 << rtcm_1005)) {
	        offset = rtcm3_repack(&rtcm_info[rtcm_1005], ucbuf);
		printf("MakeRTCMInfoRecords: packed rtcm_1005 with %d characters\n",offset);
	}
#if FOO
	printf("MakeRTCMInfoRecords: Copying %d characters into RTCM3 Packet\n",offset);
	for (int i = 0; i < offset; i++) {
	        // printf("MakeRTCMInfoRecords: byte %d = %x\n",i,ucbuf[i]);
		rtcm3_packet.push_back(ucbuf[i]);
	}
#endif

	if (haverec & (1 << rtcm_1008)) {
		offset = rtcm3_repack(&rtcm_info[rtcm_1008], ucbuf);
		printf("MakeRTCMInfoRecords: packed rtcm_1008 with %d characters\n",offset);
	} else if ( haverec & (1 << rtcm_1007)) {
		offset = rtcm3_repack(&rtcm_info[rtcm_1007], ucbuf);
		printf("MakeRTCMInfoRecords: packed rtcm_1007 with %d characters\n",offset);
	}
	printf("MakeRTCMInfoRecords: Copying %d characters into RTCM3 Packet\n",offset);
	for (int i = 0; i < offset; i++) {
	        // printf("MakeRTCMInfoRecords: byte %d = %x\n",i,ucbuf[i]);
		rtcm3_packet.push_back(ucbuf[i]);
	}

	if (haverec & (1 << rtcm_1008)) {
	  struct rtcm_t ttt;
	  rtcm3_unpack(&ttt, ucbuf);
	  rtcm_dump(&ttt, buf, BUFSIZ);
	  printf("MakeRTCMInfoRecords: %s", buf);
	}

	printf("MakeRTCMInfoRecords: Exiting with station ID %d\n\n",rtcm_refstaid);

	return(rtcm_refstaid);
}

void InitRTCMStruct()
{
	unsigned long long receive_time;
	unsigned int gnss;
	unsigned long tow;
	double zcount;
	
	printf("InitRTCMStruct: Entering...\n");

	memset(&rtcm, 0 , sizeof(rtcm_t));

//get hour portion of receive time
	printf("InitRTCMStruct: Minute = %d, Frac = %d\n",gpsMinute, gpsFrac);
        receive_time = (unsigned long long) gpsMinute;
        receive_time *= 60ULL * 1000ULL;
	receive_time += gpsFrac;
	tow = receive_time % (86400 * 7 * 1000);
	printf("InitRTCMStruct: receive_time = %lld, tow = %ld\n", receive_time, tow);
	receive_time = receive_time % (3600 * 1000);

// divide receive_time into zcount and gnss time
	gnss = receive_time % 600;
	zcount = ((double)receive_time - (double)gnss)/1e3;
	gnss *= 1000;

	printf("InitRTCMStruct: receive_time = %f, gnss = %d, zcount = %f\n", receive_time, gnss, zcount);

// set basic header information on all of the types
	rtcm.length = 0;
	rtcm.refstaid = rtcm_refstaid;
	rtcm.stathlth = 0;
	rtcm.msg_data.gpstow = tow;
	rtcm.zcount = zcount;

	rtcm.msg_data.obs_data[RTCM_L1].frequency = 0;
	rtcm.msg_data.obs_data[RTCM_L2].frequency = 2;
	rtcm.msg_data.obs_data[RTCM_C1].frequency = 0;
	rtcm.msg_data.obs_data[RTCM_P1].frequency = 0;
	rtcm.msg_data.obs_data[RTCM_P2].frequency = 2;

	rtcm.msg_data.numobs = 0;
	rtcm.msg_data.gnsstime = gnss;
	for (int j = 0; j < RTCMMAXTYPES; j++) {
		rtcm.msg_data.obs_data[j].obs_avail = false;
	}

	printf("InitRTCMStruct: Exiting...\n");
}

int MakeRTCMRecord()
{

	int i;
	static int seqnum = 0;
	unsigned char buf[BUFSIZ * 4];
	int offset2 = 0;

	isgps30bits_t isgpsbuf[RTCM_WORDS_MAX];
	
	printf("MakeRTCMRecord: Entering...\n");

	for (i = RTCMMAXTYPES - 1; i >= 0; i--) {
		if (rtcm.msg_data.obs_data[i].obs_avail) {
			printf("MakeRTCMRecord: setting multiple for %d to 0\n",i);
			rtcm.msg_data.obs_data[i].obs[rtcm.msg_data.numobs - 1].multiple = 0;
			break;
		}
	}

	for (i = 0; i< RTCMMAXTYPES; i++) {
		if (! rtcm.msg_data.obs_data[i].obs_avail)
			continue;
		rtcm.length = 1 + (rtcm.msg_data.numobs * 2);
		switch (i) {
			case RTCM_L1:
			case RTCM_L2:
				rtcm.msg_data.phase = &(rtcm.msg_data.obs_data[i]);
				rtcm.type = 18;
				break;
			case RTCM_C1:
			case RTCM_P1:
			case RTCM_P2:
				rtcm.msg_data.pseudo = &(rtcm.msg_data.obs_data[i]);
				rtcm.type = 19;
				break;
		}
		rtcm.seqnum = seqnum++;
		seqnum &= 0x7;
		printf("MakeRTCMRecord: packing rtcm for session[%d] of length %d\n",i,rtcm.length);
		(void)rtcm_repack(&rtcm,isgpsbuf);
		printf("MakeRTCMRecord: getting output from rtcm_buffer_magnavox\n");
		rtcm_buffer_magnavox(isgpsbuf, buf, &offset2);
/*
		char buf2[BUFSIZ];
		rtcm_dump(&rtcm, buf2, BUFSIZ);
		printf("MakeRTCMRecord: dump\n%s\n",buf2);
*/
	}

	for (int i = 0; i < offset2; i++ ) {
		rtcm_packet.push_back(buf[i]);
	}

	printf("MakeRTCMRecord: output buffer: length = %d\n",offset2);

	printf("MakeRTCMRecord: Exiting...\n");
	return(offset2);
}

int MakeRTCM3Record() {

        unsigned char buf[BUFSIZ * 4];
	unsigned char offset = 0;

	struct rtcm_t rtcm_test;

	memset(&rtcm_test, '\0', sizeof(struct rtcm_t));
	rtcm.type = 1004;
	offset = rtcm3_repack(&rtcm, buf);

/*
	char buf2[BUFSIZ * 4];
	rtcm_dump(&rtcm, buf2, BUFSIZ * 4);
	buf2[BUFSIZ * 4 - 5] = '\0';
	printf("MakeRTCM3Record: dump\n%s\n",buf2);

	rtcm3_unpack(&rtcm_test, buf);
	rtcm_dump(&rtcm_test, buf2, BUFSIZ *4);
	buf2[BUFSIZ * 4 - 5] = '\0';
	printf("MakeRTCM3Record: dump\n%s\n",buf2);
*/
	for (int i = 0; i < offset; i++ ) {
		rtcm3_packet.push_back(buf[i]);
	}
		
	return(rtcm3_packet.size());
}


int GetRTCMRecords(vector<unsigned char> &rtcm_out, vector<unsigned char> &rtcm3_out)
{

	printf("GetRTCMRecords: Entering...\n");

	printf("GetRTCMRecords: Initializing RTCM Structure\n");
	InitRTCMStruct();

	// printf("GetRTCMRecords: Calling DecomposePacket\n");
	// DecomposePacket();

	printf("GetRTCMRecords: Making RTCM Antenna Info Structure\n");
	MakeRTCMInfoRecords();

	{       // FOO
	        unsigned char ucbuf[RTCM_WORDS_MAX];
	        for (size_t i = 0; i < rtcm3_packet.size(); i++) {
	           ucbuf[i] = rtcm3_packet[i];
	        }
		struct rtcm_t ttt;
	        rtcm3_unpack(&ttt, ucbuf);
		char buf[BUFSIZ];
		rtcm_dump(&ttt, buf, BUFSIZ);
		printf("GetRTCMRecords: FOO: %s", buf);
	} 

	printf("GetRTCMRecords: Making RTCM 2 Structure\n");
	MakeRTCMRecord();

	for (size_t i = 0; i < rtcm_packet.size(); i++) {
		rtcm_out.push_back(rtcm_packet[i]);
	}
	rtcm_packet.clear();

	printf("GetRTCMRecords: Making RTCM 3 Structure\n");
	// FOO MakeRTCM3Record();

	for (size_t i = 0; i < rtcm3_packet.size(); i++) {
		rtcm3_out.push_back(rtcm3_packet[i]);
	}
	rtcm3_packet.clear();

	printf("GetRTCMRecords: Exiting...\n");
        return(0);
}

int main (int argc, char *argv[]) {
  unsigned long long receive_time;
  unsigned char ucbuf[RTCM_WORDS_MAX];
  vector<unsigned char> vrtcm;
  unsigned char *rtcm3;
  vector<unsigned char> vrtcm3;
  unsigned int rtcm3_len;

  GetRTCMRecords(vrtcm, vrtcm3);
  rtcm3_len = vrtcm3.size();
  rtcm3 = new unsigned char[rtcm3_len];

  printf("main: rtcm3_len=%d\n",rtcm3_len );

  for (size_t i = 0; i < rtcm3_len; i++) {
      rtcm3[i] = vrtcm3[i];
  }
  
  vrtcm3.clear();

  struct rtcm_t ttt;
  rtcm3_unpack(&ttt, rtcm3);
  if(ttt.type != 1004) {
    char buf[BUFSIZ];
    rtcm_dump(&ttt, buf, BUFSIZ);
    printf("main: %s", buf);
  }
			
  exit(0);
}
