/*
#    $Id: ArgUtils.h,v 1.5 2018/02/25 06:44:52 gps Exp $
#
#    Copyright (C) 2000 US Geological Survey
#    Written by Keith F. Stark
#    
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifndef _ARGUTILS_H
#define _ARGUTILS_H

#if 0
#include <serial.h>
#include <string>
#include <strings.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>

#endif
#include "gpscommon.h"
#include "sharc.h"

int DecodeArguments(int argc, char **argv, SOps *pOps, SComm *pComm);
int SetValues(int c, char *argument, SOps *pOps, SComm *pComm);
void ReadConfig(SComm *pComm, SOps *pOps);
void ShowUsage();
void InitOpsStruct(SOps *pOps);
void InitFileOpsStruct(SFileOps *pFileOps);
#if USE_STREAM
writerinfo_t * InitWriterInfo(SOps *pOps);
#endif
int DecodeIndexTimeRange(SOps *pOps, char *argument, int indexes[], int *numindexes, gpstime *filestart, gpstime *fileend, char *session);
int GetTimeRange(string token, gpstime *filestart, gpstime *fileend);
int GetIndexList(string token, int indexes[], int *numindexes);
int GetSession(string token, char *session);
int ArgError(const char *error);

#define MAXPARAM 5
struct Arguments {
	const char *param[MAXPARAM];
	int has_arg;
	int val;
	const char *description;
	const char *argumentdesc;
};

typedef struct Arguments SArguments;

enum {
	SWITCH_configfile,
	SWITCH_datadir,
	SWITCH_debug,
	SWITCH_debugfile,
	SWITCH_port,
	SWITCH_flow,
	SWITCH_databits,
	SWITCH_stopbits,
	SWITCH_parity,
	SWITCH_baud,
	SWITCH_login,
	SWITCH_minbps,
	SWITCH_connectvia,
	SWITCH_phonenumber,
	SWITCH_freewaveslave,
	SWITCH_repeater,
	SWITCH_initstring,
	SWITCH_verifycode,
	SWITCH_fatfile,
	SWITCH_sumfile,
	SWITCH_getid,
	SWITCH_download,
	SWITCH_retries,
	SWITCH_joinfiles,
	SWITCH_purge,
	SWITCH_session,
	SWITCH_offset,
	SWITCH_help,
	SWITCH_manpage,
	SWITCH_receivertype,
	SWITCH_filetemplate,
	SWITCH_filetimefrom,
	SWITCH_strings,
	SWITCH_terminal,
	SWITCH_forceterminal,
	SWITCH_multtimeout,
	SWITCH_maxerror,
	SWITCH_resumedir,
	SWITCH_forcesave,
#if USE_STREAM
// these are for realtime output
	SWITCH_stream,
	SWITCH_mount,
	SWITCH_datatype,
	SWITCH_file,
	SWITCH_net,
	SWITCH_ntrip,
	SWITCH_stats,
#ifdef RTCM_ENABLE
	SWITCH_rtcm,
	SWITCH_rtcmv22,
	SWITCH_rtcmv24,
	SWITCH_rtcmv30,
#endif
	SWITCH_forcestream,
#if 0
	SWITCH_pipe,
	SWITCH_exec,
#endif
#endif
	SWITCH_version,
	SWITCH_COUNT
};

static const SArguments myoptions[] =
{
	{{"debug","el",NULL,NULL,NULL},
			1,
			SWITCH_debug,
			"<level>",
			"Change the level of debug information that is output. A higher number means more information. "
	},
	{{"debugfile","ef",NULL,NULL,NULL},
			1,
			SWITCH_debugfile,
			"<filename|- for stdout>",
			"Cause the debug information to be printed to a file rather than to standard out."
	},
	{{"configfile","cf",NULL,NULL,NULL},
			1,
			SWITCH_configfile,
			"<filename>",
			"Read configuration options from different config file."
	},
	{{"port","cp","Cp",NULL,NULL},
			1,
			SWITCH_port,
			"<portname>|<hostname:port>",
			"Specify the port to connect to (ie /dev/ttyS0).\n\
This will also support an IP:port specification and connecto to a serial<->ethernet proxy.\n\
"
	},
	{{"flow",NULL,NULL,NULL,NULL},
			1,
			SWITCH_flow,
			"<hard|soft|both|none>",
			"Type of flow control (HARD is default)"
	},
	{{"databits","ca","Ca",NULL,NULL},
			1,
			SWITCH_databits,
			"<databits>",
			"Number of data bits"
	},
	{{"stopbits","cs","Cs",NULL,NULL},
			1,
			SWITCH_stopbits,
			"<stopbits>",
			"Number of stop bits"
	},
	{{"parity","cy","Cy",NULL,NULL},
			1,
			SWITCH_parity,
			NULL,
			"Use parity"
	},
	{{"baud","cb","Cb",NULL,NULL},
			1,
			SWITCH_baud,
			"<baud rate>",
			"Baud rate to connect to local device"
	},
	{{"login","","",NULL,NULL},
			1,
			SWITCH_login,
			"user,pass",
			"Supply username and password for receiver login"
	},
	{{"minbps","","",NULL,NULL},
			1,
			SWITCH_minbps,
			"<minbps>[,seconds]",
			"Exit if not at minimum baud (default=no minimum) rate for number of seconds (default=120)"
	},
	{{"connectvia","cv","Cv",NULL,NULL},
			1,
			SWITCH_connectvia,
			"<modem|freewave|direct>",
			"Type of connection: modem, freewave, direct"
	},
	{{"phonenumber","cd","Cd",NULL,NULL},
			1,
			SWITCH_phonenumber,
			"<phone number, freewave serial number or IP:Port>",
			"Phone Number or freewave slave serial number.\n\
Freewave Slave serial number should be only digits.\n\
IP:Port will override the port specifier"
	},
	{{"freewave","fs","Cfs",NULL,NULL},
			1,
			SWITCH_freewaveslave,
			"<freewave serial number>",
			"freewave slave serial number. Freewave Slave serial number should be only digits"
	},
	{{"repeater","fr","Cfr",NULL,NULL},
			1,
			SWITCH_repeater,
			"<callbook number of repeater path>",
			"Callbook entry number for repeater path to use"
	},
	{{"initstring","ci","Ci",NULL,NULL},
			1,
			SWITCH_initstring,
			"<modem init string>",
			"Initialization string for the modem or other local device"
	},
	{{"verifycode","vc","Vc",NULL,NULL},
			1,
			SWITCH_verifycode,
			"<Site code to verify>",
			"Verify site code in receiver against this argument to make sure we are connected to correct receiver."
	},
	{{"fatfile","l","L",NULL,NULL},
			1,
			SWITCH_fatfile,
			"<filename>",
			"Filename for output of File Allocation Table"
	},
	{{"sumfile","s","S",NULL,NULL},
			1,
			SWITCH_sumfile,
			"<filename>",
			"Filename for sumfile output. defaults to standard out"
	},
	{{"getid","i","I",NULL,NULL},
			0,
			SWITCH_getid,
			NULL,
			"Get ID from receiver. Used to check serial link to receiver without doing anything else."
	},
	{{"download","di","dis","dt","dts"},
			1,
			SWITCH_download,
			"<index|timerange[ session]> | all",
"The keyword \"all\" will select all but the currently active file(s);\n\
\n\
<index> = any comma separated list of numbers or number ranges. This will\n\
delete the Nth file on the receiver. Sharc always begins counting at 1 and\n\
will adjust the index given to get the correct file on the receiver which \n\
may use a different numbering scheme (ie. start from 0).\n\
	example: 1,3,6-9,12-15\n\
<timerange> = <time>-<time>\n\
	<time> =\n\
		GPSWeek/GPSSec or\n\
		%Y/%m/%d[:%H[:%M[:%S]]] or \n\
		%Y/0/%J[:%H[:%M[:%S]]]\n\
		%Y = 4 digit year\n\
		%m = 2 digit month\n\
		%d = 2 digit day of month\n\
		%H = 2 digit hour (default 00|23);\n\
		%M = 2 digit hour (default 00|59);\n\
		%S = 2 digit hour (default 00|59);\n\
\n\
Specifying both an index list and a time range translates to a logical \"and\". The time range must occur in the list of indexes. All files matching the time range will be downloaded.\n\
\n\
I do not recommend using the session argument unless only a single file is being downloaded.\n\
\n\
example to download data from Julian Day 117 year 2001 if it occurs in the first 10 indexes or the 13th index:\n\
--di \"1-10,13 2001/0/117-2001/0/117\"\n\
"
	},
	{{"retries","","",NULL,NULL},
			1,
			SWITCH_retries,
			"<retries max=5>",
			"Retry N times (disconnect,reconnect) before exiting"
	},
	{{"joinfiles","jf",NULL,NULL,NULL},
			0,
			SWITCH_joinfiles,
			"NOT WORKING",
			"Join all files within a time range into one file."
	},
	{{"purge","pi","Pi","pt","Pt"},
			1,
			SWITCH_purge,
			"<index|timerange[ session]> | all",
			"See explanation of --download"
	},
	{{"session",NULL,NULL,NULL,NULL},
			1,
			SWITCH_session,
			"<start_sec,end_sec,interval,elevation_mask,minsv>|<number_of_hours_per_session,interval[,mask[,minsv]]>",
"Add a session starting at start_sec seconds after the beginning of the day, stopping end_sec seconds after the beginning of the day, with interval, elevation_mask, and minsv as requirements. Interval can be  a floating point number. Enclose the four arguments in quotes. Multiple sessions can be programmed by supplying this switch multiple times.  \nor\nnumber of hours per session.  Valid number for hours are 1,2,3,4,6,8,12,24 minsv and mask will default to 1 and 0 respectively. Interval is supplied as second argument"
	},
	{{"offset",NULL,NULL,NULL,NULL},
			1,
			SWITCH_offset,
			"<seconds>",
"Offset the session programming by given seconds when using short form of session programming"
	},
	{{"help","h","H",NULL,NULL},
			0,
			SWITCH_help,
			NULL,
			"The help page"
	},
	{{"manpage",NULL,NULL,NULL,NULL},
			0,
			SWITCH_manpage,
			NULL,
			"Print out basic format of manpage"
	},
	{{"receivertype","type","rx",NULL,NULL},
			1,
			SWITCH_receivertype,
			"<receiver>",
			"Where <receiver> is one of ashtech, topcon, navcom, leica, trogue, or trimble. The default value is ashtech."
	},
	{{"filetemplate",NULL, NULL, NULL,NULL},
			1,
			SWITCH_filetemplate,
			"<template>",

"This is the filename that will be assigned.\n\
All of the strftime format strings are valid.\n\
+S = Four character site code\n\
+s = session\n\
+G = GPS Week (%04d)\n\
+g = gps second (%06d)\n\
+J = Seconds into day (%05d)\n\
+0-9 = see --strings argument\n"
	},
	{{"strings",NULL, NULL, NULL,NULL},
			1,
			SWITCH_strings,
			"<string0> <string1> .. <string9>",
			"These strings will be used to replace +1 - +9 in the filetemplate argument. This can be used to pass long site codes or any other information. Any whitespace will be a delimiter for the next string."
	},
	{{"datadir",NULL, NULL, NULL,NULL},
			1,
			SWITCH_datadir,
			"<pathname>",
			"ALL output files will be put in this directory"
	},
	{{"resumedir",NULL, NULL, NULL,NULL},
			1,
			SWITCH_resumedir,
			"<pathname>",
			"ALL zmodem resume files will be put in this directory"
	},
	{{"forcesave",NULL, NULL, NULL,NULL},
			0,
			SWITCH_forcesave,
			"",
			"Save all ZMODEM downloads to resumedir directory"
	},
	{{"filetimefrom",NULL, NULL, NULL,NULL},
			1,
			SWITCH_filetimefrom,
			"request|actual[ start|stop]",
			"Get file time for file template from the file start time, file stop time or the requested start time or the requested stop time. actual start time is default."
	},
	{{"terminal",NULL, NULL, NULL,NULL},
			0,
			SWITCH_terminal,
			"",
			"Give a command prompt to communicate with receiver"
	},
	{{"forceterminal",NULL, NULL, NULL,NULL},
			0,
			SWITCH_forceterminal,
			"",
			"Give a command prompt to communicate with receiver"
	},
	{{"multtimeout","xto", NULL, NULL,NULL},
			1,
			SWITCH_multtimeout,
			"<multiplier>",
			"This will multiply the timeout length in the serial package"
	},
	{{"maxerror",NULL, NULL, NULL,NULL},
			1,
			SWITCH_maxerror,
			"<errors>",
			"Set the maximum number of X/Z modem errors before exiting. This is the maximum errors across all downloads, not just a single file download."
	},
#if USE_STREAM
	{{"stream",NULL, NULL, NULL,NULL},
			1,
			SWITCH_stream,
			"<samplerate,receiverport>",
			"specifies that sharc should go into streaming mode with the specified sample rate and from the receiver serial port given (ashtech=a,b,c,d). A value of 0 for samplerate will turn off streaming on the specified port."
	},
	{{"mount",NULL, NULL, NULL,NULL},
			1,
			SWITCH_mount,
			"<mount:[user:]password>",
			"If connecting to an IP based stream, use the ntrip protocol to login to the port. This automatically sets --forcestream since no configuration should be done via an ntrip interface. If user is excluded, the mountpoint will be used as the username"
	},
	{{"datatype",NULL, NULL, NULL,NULL},
			1,
			SWITCH_datatype,
			"<dben|binex|raw>",
			"Real time data stream type default=binex, raw data format will support the Navcom"
	},
	{{"file",NULL, NULL, NULL,NULL},
			1,
			SWITCH_file,
			"<file_template,seconds[,samplerate]",
			"This will create a new file specified by file_template based on the time of the record. The file_template is run through chartime for meta replacements (see strftime manual).\nWhen time is divisible by seconds it will create a new file. Supplyng a sample rate can be used to decimate the data to different files. This switch can be used more than once."
	},
	{{"net",NULL, NULL, NULL,NULL},
			1,
			SWITCH_net,
			"<host:port[,sample]>",
			"Connect to given host and port and begin streaming data to it. If sample rate is supplied only send data divisible by sample (decimate) "
	},
	{{"ntrip",NULL, NULL, NULL,NULL},
			1,
			SWITCH_ntrip,
			"<mount:password@host:port[,sample]>",
			"Connect to given Ntrip caster at specified port using mount and passwourd and begin streaming data to it. If sample rate is supplied only send data divisible by sample (decimate) "
	},
	{{"stats",NULL, NULL, NULL,NULL},
			1,
			SWITCH_stats,
			"<frequency>,<leapseconds>",
			"Output stream statistics every N seconds with an offset of leapseconds. It is very important to be running NTP on the local machine for the statistics to make any sense at all. The leapseconds value will affect the latency calculations to correct for the differenc in GPS time and local time."
	},
#ifdef RTCM_ENABLE
	{{"rtcm",NULL, NULL, NULL,NULL},
			1,
			SWITCH_rtcm,
			"<antennfilename>",
			"All of the remaining streams will output translated RTCM format if the stream format is BINEX. The input file includes all of the necessary information for the rtcm stream. A sample data file can be found in the sharc distribution.",
	},
	{{"rtcmv22",NULL, NULL, NULL,NULL},
			0,
			SWITCH_rtcmv22,
			"",
			"Next stream outputs should be in RTCM V2.2 format. Need to have rtcm antenna file specified with --rtcm <file>.",
	},
	{{"rtcmv24",NULL, NULL, NULL,NULL},
			0,
			SWITCH_rtcmv24,
			"",
			"Next stream outputs should be in RTCM V2.4 format. Need to have rtcm antenna file specified with --rtcm <file>.",
	},
	{{"rtcmv30",NULL, NULL, NULL,NULL},
			0,
			SWITCH_rtcmv30,
			"",
			"Next stream outputs should be in RTCM V3.0 format. Need to have rtcm antenna file specified with --rtcm <file>.",
	},
#endif
	{{"forcestream",NULL, NULL, NULL,NULL},
			0,
			SWITCH_forcestream,
			"",
			"Connect to currently running stream without any configuration\n"
	},
#if 0
	{{"exec",NULL, NULL, NULL,NULL},
			1,
			SWITCH_exec,
			"",
			""
	},
	{{"pipe",NULL, NULL, NULL,NULL},
			1,
			SWITCH_pipe,
			"",
			""
	},
#endif
#endif
	{{"version",NULL,NULL,NULL,NULL},
			0,
			SWITCH_version,
			NULL,
			"Print version string"
	},
	{{NULL,NULL,NULL,NULL,NULL},
			0,
			0,
			NULL,
			NULL}
};

#endif /*ARGUTILS_H*/
