/* 
#    $Id: Ashtech.h,v 1.3 2012/02/10 00:48:57 gps Exp $
#
#    Ashtech.h -- Include file for the Ashtech Receiver
#
#    Copyright (C) 2000 US Geological Survey
#    Written by Keith F. Stark
#    
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifndef _ASHTECH_H
#define _ASHTECH_H

#include <fstream>
#include <string>

#include <gpscommon.h>
#include <AshtechDefs.h>
#include <Receiver.h>
#include <gpstime.h>

#define FILELENGTH 12
#define ASHBYTEORDER __BIG_ENDIAN

class Ashtech : public Receiver {
public:
	Ashtech();

	static string DownloadReceiverID(Serial *pSerial);
// Download fat
	static int GetFATEntryAtIndex(SFATEntry *pFAT, int index);
	static int GetReceiverID();
	static int Stream(Serial *pSerial, int format, double rate, char port);
	static int DBENStream(Serial *pSerial, double rate, char port);
	static int BinexStream(Serial *pSerial, double rate, char port);
	static long AutoBaud(Serial *pSerial);
	static int GetPortInfo(Serial *pSerial, char *port, long *speed);
	static int SetPortInfo(Serial *pSerial, long baud);
	static int Login(Serial *pSerial, int direct, string login, string password);
	static int TerminalSession(Serial *pSerial);
	static int CompareSessions(SAshtechSession *tmpsession);


	static int CloseOpenFile(Serial *pSerial);

	VIRTUAL int DownloadFileByTime(Serial *pSerial, gpstime *start, gpstime *stop) { return(1); };
	// Download File routines

	VIRTUAL int DeleteFileByTime(Serial *pSerial, gpstime *start, gpstime *stop);
	VIRTUAL int WriteFiles();
	int setImageFile(unsigned char *myImage, int fileSize);

	void AddImage(Ashtech *newAshtech);
	void AddImage(unsigned char *image, int fileSize, SAshtechFATEntry *myFAT);

	void AddToIndexBuffer(int index, unsigned char *in, size_t size);

	static int SendQuery(Serial *pSerial, string QueryString);
	static int SendCommand(Serial *pSerial, string CommandString, int DoCheckSum = FALSE, int WaitAck = TRUE);
	static int CheckAck(Serial *pSerial);
	
	virtual unsigned char *getImageFile() { return(imageFile); }
	virtual int getImageFileSize() { return(cimageFile); }
	virtual int getIndex() { return(myIndex); }
	
protected:
	// File Data
	SAshtechFATEntry *pFATentry;
	RawImageHeader *pImageHead;	
	
	int cimageFile;
	int myIndex;

	// sitename and times
	string sitename;
	
	// Actual RFile image
	unsigned char *imageFile;
	unsigned char *header;
	unsigned char *fatEntry;
	
	unsigned char *indexData[MAX_RCVR_FILES];
	char session;

};

#endif
