/* 
#    $Id: AshtechDefs.h,v 1.3 2012/02/10 00:48:57 gps Exp $
#
#    AshtechDefs.h -- Data Structures 
#
#    Copyright (C) 2000 US Geological Survey
#    Written by Keith F. Stark
#    
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifndef _ASH_DEFS_H
#define _ASH_DEFS_H

/* Supported receiver types */
#define RCV_Z12 1
#define RCV_UZ  2

/****************************************************************************/
/* these defs are for the new development                                   */
/****************************************************************************/

#define GENERAL_TIMEOUT 10000L
#define DELETION_TIMEOUT 120000L
#define ASHTECH_XMODEM_CLEANUP_TIMEOUT 10000L

#define NUM_EXTRA_ASHTECH_XMODEM_EOT 4
#define PASH_RID_REPLY_LENGTH 35
#define DELETION_COMPLETE_REPLY 'C'
#define MAX_PROG_SESSIONS 24
#define MAX_ASHTECH_FILES 100
#define INTERMEDIATEHEADERLEN 20

/****************************************************************************/
#define SITENAME_LEN 4    /* The number of charactes used by the site name. */
#define OPERATOR_LEN 12   /* Number of characters for operator field.       */
#define PROJECT_LEN  34   /* Number of characters for project field.        */
/****************************************************************************/

// Define records for reading in Ashtech data and navigation records
// These definitions are as printed from the Ashtech filetool program

struct RawImageHeader {
	char receiverType[20];
	char version[20];
	char oneFileMode;
	char res;
	long totalMemory;
	long FATstart;
	long FATend;
	long photoStart;
	long photoEnd;
	long almanacStart;
	long almanacEnd;
	long dataStart;
	long dataEnd;
	char spare[71];
	char imageHeadVers;
};

#define RAWIMAGEHEADERSIZE_F 150

struct AshtechFATEntry {
	long         segbeg;
	long         wordcount;
	char         segname  [SITENAME_LEN]; /* 4 */
	char         ranger_mode;
	char         reserv;
	short        wnbeg;
	long         timebeg;
	char         oper[4];
	char         project  [PROJECT_LEN]; /* 34 */
	short        segwn; /* was int */
	long         segtime;
	unsigned short anthibef;  /* was unsigned int */
	unsigned short anthiaft;  /* was unsigned int */
	char         tdrybef;
	char         tdryaft;
	char         twetbef;
	char         twetaft;
	char         humidbef;
	char         humidaft;
	short        pressurebef;     /* was int */
	short        pressureaft;     /* was int */
	unsigned short checksum;       /* was unsigned int */
};
typedef struct AshtechFATEntry SAshtechFATEntry;

#define ASHTECHFATENTRYSIZE_F 80

#define  NUM_SESS 24
struct AshtechSessionStruct {
	short	refday;
	short	offset;
	short	inuse;
	char	flag[NUM_SESS];
	char	colstat[NUM_SESS];
	long	start[NUM_SESS];
	long	end[NUM_SESS];
	long	span[NUM_SESS];
	float	interval[NUM_SESS];
	char	elevmask[NUM_SESS];
	char	minsv[NUM_SESS];
	char	type[NUM_SESS];
	unsigned short	checksum;
};

typedef struct AshtechSessionStruct SAshtechSession;

#endif /*define ASH_DEFS_H */
