/* 
#    $Id: Leica.h,v 1.1 2011/04/04 17:29:34 gps Exp $
#
#    Leica.h -- Include file for the Leica Receiver
#
#    Copyright (C) 2000 US Geological Survey
#    Written by Keith F. Stark
#    
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifndef _LEICA_H
#define _LEICA_H

#include <gpstime.h>
#include <string>
#include <gpscommon.h>
#include <LeicaDefs.h>
#include <Receiver.h>

#define FILELENGTH 12

class Leica : public Receiver {
public:
	Leica();

	static string DownloadReceiverID(Serial *pSerial);
	static int DownloadReceiverFAT(Serial *pSerial);
	static int GetFATEntryAtIndex(SFATEntry *pFAT, int index);
	static int GetReceiverID();
	static int SetReceiverSession(Serial *pSerial);
	static int ReceiveLB2(Serial *pSerial, U_1_CHAR **pmessage, U_2_INT *length);
	static int SendLB2(Serial *pSerial, U_1_CHAR *msg, unsigned short msgLength);
	static int ReadPortConf(Serial *pSerial);
	static int Login(Serial *pSerial, int direct, string login, string password);
	static int TerminalSession(Serial *pSerial) {return(1);};



	VIRTUAL int DownloadFileByTime(Serial *pSerial, gpstime *start, gpstime *stop);
	VIRTUAL int DownloadFileAtIndex(Serial *pSerial, int index);
	int ReceiveA1File(Serial *pSerial, U_4_INT indexFile);

	VIRTUAL int DeleteFileByTime(Serial *pSerial, gpstime *start, gpstime *stop);
	VIRTUAL int DeleteFileAtIndex(Serial *pSerial, int index);
	VIRTUAL int WriteFiles();
	int setFatEntry(SLeicaFATEntry myFAT);
	int setImageFile(unsigned char *myImage, int fileSize);

	void AddImage(Leica *newLeica);
	void AddImage(unsigned char *image, int fileSize, SLeicaFATEntry *myFAT);

	int AddMessage(U_1_CHAR *message, int length);
	static void ReverseFATEntry(unsigned char *outFAT, unsigned char *inFAT);
	
	unsigned char **getImageFile() { return(imageFile); }
	int getImageFileSize() { return(cimageFile); }
	virtual int getIndex() { return(myIndex); }
	
private:
	SLeicaFATEntry *pFATentry;
	// File Data
	int myIndex;

	// sitename and times
	string sitename;
	
	// Actual RFile image
	unsigned char **imageFile;
	int numMessages;
	int cimageFile;
	ReportStruct reportstruct;
	
	char session;

	static SLeicaFATEntry pFAT[MAX_LEICA_FILES];
	static SLeicaPortConfig receiverPorts;
	static U_1_CHAR logicalPort;
	static U_1_CHAR serialPort;

};
#endif
