/* 
#    $Id: LeicaDefs.h,v 1.1 2011/04/04 17:29:34 gps Exp $
#
#    LeicaDefs.h -- Data Structures 
#
#    Copyright (C) 2000 US Geological Survey
#    Written by Keith F. Stark
#    
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifndef _LEICA_DEFS_H
#define _LEICA_DEFS_H
/****************************************************************************/
/* these defs are for the new development                                   */
/****************************************************************************/

#define MAXLB2SIZE 2048
#define GENERAL_TIMEOUT 10000L
#define DELETION_TIMEOUT 120000L
#define LEICA_XMODEM_CLEANUP_TIMEOUT 10000L

#define NUM_EXTRA_LEICA_XMODEM_EOT 4
#define DELETION_COMPLETE_REPLY 'C'
#define MAX_PROG_SESSIONS 24
#define INTERMEDIATEHEADERLEN 20

#define MAX_TRIES_FOR_ID 10
#define MAX_MESSAGES_BEFORE_RESPONSE 10

#define MAX_RCVR_PAGES 4
#define MAX_RCVR_FILES_PER_PAGE 8
#define MAX_LEICA_FILES 32

#define MAX_RCVR_PORTS 4

#define MAX_CHAN 12

#define CN0_SCALER 25

#define  W_L1 0.1902936727983649

#define  W_L2 0.2442102134245683

#define bAE84 6378137.0
#define bFLAT84 0.00335281066474


typedef char S_1_CHAR;
typedef unsigned char U_1_CHAR;
typedef unsigned char U_1_INT;
typedef int S_4_INT;
typedef unsigned int U_4_INT;
typedef short S_2_INT;
typedef unsigned short U_2_INT;
typedef float S_4_REAL;
typedef double S_8_REAL;
typedef long double S_10_REAL;

struct ChanBlock {
    U_1_INT ant;
    U_2_INT stat;
};
typedef struct ChanBlock SChanBlock;

struct L1Block {
    U_1_CHAR quality;
    S_8_REAL phase;
    S_2_INT code;
    U_1_INT ctc;
};
typedef struct L1Block SL1Block;

struct L2Block {
    U_1_CHAR quality;
    S_8_REAL phase;
    S_2_INT code;
    U_1_INT ctc;
};
typedef struct L2Block SL2Block;

struct MeasurementData {
    U_4_INT gpsSec;
    S_2_INT atrcs;
    U_1_CHAR daf;

    SChanBlock chan[MAX_CHAN];
    SL1Block l1[MAX_CHAN];
    SL2Block l2[MAX_CHAN];

    S_1_CHAR numSats;
};
typedef struct MeasurementData SMeasData;

struct LeicaPortConfig {
    U_1_CHAR availablePort[4];
    U_1_CHAR numPorts;
    unsigned long baudRate[4];
    U_1_CHAR pacingMode[4];
    U_1_CHAR stopBits[4];
    U_1_CHAR parity[4];
    U_1_CHAR hardwareProtocol[4];
    U_1_CHAR outputFormat[4];
    U_1_CHAR inputFormat[4];
};
typedef struct LeicaPortConfig SLeicaPortConfig;

struct LeicaFATEntry {
    U_1_CHAR fileNumber;
    U_4_INT fileLength;

    U_2_INT logContStatus;
    
    U_1_CHAR numLogFileControlBlocks;
    
    U_1_CHAR extent;
    U_1_CHAR extentInterval;
    
	gpstime fileStart;
	gpstime fileEnd;
};
typedef struct LeicaFATEntry SLeicaFATEntry;

#endif
