/* 
#    $Id: MicroZ.cpp,v 1.2 2018/02/23 04:34:13 gps Exp $
#
#	 MicroZ.cpp -- This file will be the major interaction with
#                   an ashtech MicroZ receiver/file
#    Copyright (C) 2000 US Geological Survey
#    Written by Keith F. Stark
#    
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <iostream>
#include <string>
#include <iomanip>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <stdio.h>
#include <time.h>
#include <sys/time.h>
#include <sys/types.h>

#include "gpscommon.h"
#include "MicroZ.h"

string MicroZ::UZ_FileNames[MAX_RCVR_FILES];

//  ===================================================================
//  NAME:		MicroZ::MicroZ
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
MicroZ::MicroZ () {

	debug(D_FLOW,"MicroZ::MicroZ: Entering...\n");
//	usesFAT = TRUE;

	debug(D_FLOW,"MicroZ::MicroZ: Exiting\n");

}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
void MicroZ::AddImage(Ashtech *newAshtech) {
	debug(50,"MicroZ::AddImage(Ashtech): Entering...\n");

	int fileSize = newAshtech->getImageFileSize();
	unsigned char *image = newAshtech->getImageFile();

	unsigned char *tmp = new unsigned char[cimageFile+fileSize];
	memcpy(tmp, imageFile, cimageFile);
	memcpy(&tmp[cimageFile], image, fileSize);
	setImageFile(tmp, cimageFile+fileSize);
	debug(25,"MicroZ::Addimage(Ashtech): fileSize is %d\n",cimageFile);
	delete [] tmp;

	debug(50,"MicroZ::AddImage(Ashtech): Entering...\n");


}

//  ===================================================================
//  NAME:		MicroZ::DownloadFileByTime
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int MicroZ::DownloadFileByTime(Serial *pSerial, gpstime *start, gpstime *stop) {
	int i;
	int numindex, indexes[MAX_RCVR_FILES];
	int status = FAILURE;
	char buffer[100];

	debug(D_FLOW,"MicroZ::DownloadFileByTime: Entering...\n");

	numindex = FindIndexesByTime(start, stop, indexes, FALSE);

	if (numindex > 0) {
		debug(D_STD_V2,"MicroZ::DownloadFileByTime: Downloading file at index %d\n", indexes[0]);
		status = DownloadFileAtIndex(pSerial, indexes[0]);
	} else {
		debug(D_STD_V2,"MicroZ::DownloadFileByTime: No Files found for time range\n");
		status = SUCCESS_NOFILE;
	}
	if ((status == SUCCESS) && (numindex > 1)) {
		for (i = 1; i < numindex; i++) {
			debug(D_STD_V2,"MicroZ::DownloadFileByTime: Downloading file at index %d\n", indexes[i]);
			MicroZ *pTmp = new MicroZ;
			status = pTmp->DownloadFileAtIndex(pSerial, indexes[i]);
			if (status == FAILURE) {
				delete(pTmp);
				break;
			}
			AddImage(pTmp);
			delete(pTmp);
			addToSumFile(" Span: ");
			addToSumFile(start->chartime("%%G/%%g-",buffer,100));
			addToSumFile(stop->chartime("%%G/%%g\n",buffer,100));
		}
	}
	debug(D_FLOW,"DownloadFileByTime: Exiting\n");
	return(status);
}



//  ===================================================================
//  NAME:		MicroZ::DownloadFileAtIndex
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int MicroZ::DownloadFileAtIndex(Serial *pSerial, int index) {
	unsigned char *myfile = NULL;
	char buffer[100];
	time_t t_start, t_used;
	int downLoadOK = FALSE;

	debug(D_FLOW,"MicroZ::DownloadFileAtIndex: Entering...\n");

	debug(D_STD_V2,"MicroZ::DownloadFileAtIndex: Download at index %d\n",index);
	if (index >= numFATEntries) {
		debug(25,"MicroZ::DownloadFileAtIndex: Index %d does not exist\n",index);
		return(FAILURE);
	}

	addToSumFile("\nDownload:\n");
	long fileSize;

	sleep(1);
	pSerial->ClearBuffers();
	long expectSize = ReceiverFAT[index].fileSize;
	sprintf(buffer,"FIL,DOWNLOAD,%s,ZMODEM",UZ_FileNames[index].c_str());

	debug(D_STD_V2,"MicroZ::DownloadFileAtIndex: Running Query %s\n", buffer);

	SendCommand(pSerial, buffer, FALSE, FALSE);
	sleep(1);
	t_start = time(NULL);

	debug(D_STD_V2,"MicroZ::DownloadFileAtIndex: Calling Zmodem Transfer, timeout = %d, Start Time = %d\n",2400,t_start);

	myfile = pSerial->zmodemreceive(&fileSize, 2400);
	t_used = time(NULL) - t_start;

	debug(D_STD_V2,"MicroZ::DownloadFileAtIndex: Returned From Zmodem Transfer, fileSize = %d, Time Used = %d\n", fileSize,t_used);

	if ((myfile != NULL) && (fileSize >= (expectSize-1024))) { //within one block

		debug(D_STD_V,"MicroZ::DownloadFileAtIndex: Download OK, Copying image to MicroZ Class\n");
		setImageFile(myfile, fileSize);
		myIndex = index;
		downLoadOK = TRUE;
		char buffer[100];
		delete [] myfile;

		addToSumFile(" Index: %d\n",index+1);
		addToSumFile(" Start Time: %s\n",ReceiverFAT[index].fileStart.chartime("%%G %%g", buffer, 100));
		addToSumFile(" End Time: %s\n",ReceiverFAT[index].fileEnd.chartime("%%G %%g", buffer, 100));
		addToSumFile(" File: %s\n",GetFileName(myIndex).c_str());
		addToSumFile(" Bytes: %d\n",expectSize);
		addToSumFile(" Time: %d seconds\n",t_used);

	} else {
		downLoadOK = FALSE;

		debug(D_ERR, "MicroZ::DownloadFileAtIndex: Download Failed\n");

		pSerial->ClearBuffers(); sleep(1); pSerial->ClearBuffers();
	}
	addToSumFile("\n");
	debug(D_FLOW,"MicroZ::DownloadFiles: Exiting\n");

	return((downLoadOK) ? SUCCESS : FAILURE);

}

//  ===================================================================
//  NAME:		MicroZ::DeleteFileAtIndex
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int MicroZ::DeleteFileAtIndex(Serial *pSerial, int index) {
	string Check;
	int status;
	char buffer[100];

	debug(D_FLOW,"MicroZ::DeleteFileAtIndex: Entering...\n");

	sprintf(buffer,"FIL,D,%d",index);
	debug(D_STD,"\n\nDeleting file #%d",index+1);
	debug(D_FLOW," (index #%d)", index);
	debug(D_STD," -- ");
	status = SendCommand(pSerial, buffer, FALSE, TRUE);
	if (status == SUCCESS) {
		debug(D_STD," Deleted\n");
	} else {
		debug(D_STD," NOT Deleted\n\n");
	}
	debug(D_FLOW,"MicroZ::DeleteFiles: Exiting\n");
	return(status);
}

int MicroZ::VerifyReceiverSession(Serial *pSerial) {
	debug(D_FLOW,"MicroZ::VerifyReceiverSession: Entering...\n");

	SAshtechSession tmpsess;
	char buffer[1025];

	for (int sess = 0; sess < NUM_SESS; sess++) {
		sprintf(buffer,"SSN,%c",(char)(sess+65));
		string tmp = buffer;
		SendQuery(pSerial,tmp);
		tmp = buffer;
		debug(D_STD_V,buffer);
		string tokens[16];

		try
		{
			int hour = 0;
			int minute = 0;
			int second = 0;
			int numtokens = stringtokenize(string(buffer), ",", tokens, 16);
			if (numtokens != 15)
				throw "Incorrect return from Session request\n";

			if (tokens[0].compare("$PASHR") != 0)
				throw "MicroZ::VerifyReceiverSession: Bad response from query\n";

			if (tokens[1].compare("SSN") != 0)
				throw "MicroZ::VerifyReceiverSession: Bad response from query\n";

			tmpsess.inuse = ((tokens[2].compare("Y") == 0) || (tokens[2].compare("A") == 0)) ? 1 : 0;

			tmpsess.refday = atoi(tokens[3].c_str());

			minute = atoi(tokens[4].substr(0,2).c_str());
			second = atoi(tokens[4].substr(3,2).c_str());
			tmpsess.offset = (minute*60) + second;

			tmpsess.flag[sess] = tokens[8][0];

			hour = atoi(tokens[9].substr(0,2).c_str());
			minute = atoi(tokens[9].substr(3,2).c_str());
			second = atoi(tokens[9].substr(6,2).c_str());
			tmpsess.start[sess] = (hour*3600) + (minute*60) + second;

			hour = atoi(tokens[10].substr(0,2).c_str());
			minute = atoi(tokens[10].substr(3,2).c_str());
			second = atoi(tokens[10].substr(6,2).c_str());
			tmpsess.end[sess] = (hour*3600) + (minute*60) + second;

			tmpsess.interval[sess] = atof(tokens[11].c_str());
			tmpsess.elevmask[sess] = (char) atoi(tokens[12].c_str());
			tmpsess.minsv[sess] = (char) atoi(tokens[13].c_str());
			tmpsess.type[sess] = (char) atoi(tokens[14].c_str());
		}
		catch (const char *pmessage)
		{
			debug(D_ERR, pmessage);
		}
	}

	return(CompareSessions(&tmpsess));
#if 0
	addToSumFile("\nSession Programming:\n RefDay = %d, Offset = %d, inuse = %d\n",
				tmpsess.refday, tmpsess.offset, tmpsess.inuse);
	addToSumFile("inuse start      end  interval mask minsv type\n");
	for (int i = 0; i < NUM_SESS; i++) {
		addToSumFile("%2d %c      %02d:%02d    %02d:%02d %03.1f      %03d  %03d    %03d\n",
				i,
				tmpsess.flag[i],
				tmpsess.start[i]/3600, (tmpsess.start[i]%3600)/60,
				tmpsess.end[i]/3600, (tmpsess.end[i]%3600)/60,
				tmpsess.interval[i], (int)tmpsess.elevmask[i],
				(int)tmpsess.minsv[i], (int)tmpsess.type[i]);
	}

	int found;
	for (int i = 0; i < numSessions; i++) {
		long start    = GetSessionStartTime(i);
		long end      = GetSessionEndTime(i);
		float interval = GetSessionInterval(i);
		int elevmask = GetSessionElevationMask(i);
		int minsv    = GetSessionMinimumSV(i);
		found = FALSE;
		for (int j = 0; (j < NUM_SESS) && (!found); j++) {
			if (tmpsess.flag[j] == 'N') continue;
			if (start != tmpsess.start[j]) continue;
			if (end != tmpsess.end[j]) continue;
			if (interval != tmpsess.interval[j]) continue;
			if (elevmask != tmpsess.elevmask[j]) continue;
			if (minsv != tmpsess.minsv[j]) continue;
			debug(D_STD_V,"MicroZ::VerifyReceiverSession: Found Match for Session %d\n",j);
			found = TRUE;
		}
		if (! found) break;
	}
	debug(D_FLOW,"MicroZ::VerifyReceiverSession: Exiting...\n");
	if (found) return(SUCCESS);
	return(FAILURE);
#endif
}

//  ===================================================================
//  NAME:		MicroZ::SetReceiverSession
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int MicroZ::SetReceiverSession(Serial *pSerial) {
	int refday = 1;
	int offsec = 0;
	int rectype = 0;

	float interval = 30;
	int elevmask = 0;
	int start = 0;
	int end = 0;
	int minsv = 1;
	char inuse = 'N';
	char buffer[200];

	debug(D_FLOW,"MicroZ::SetReceiverSession: Entering...\n");

	int status = VerifyReceiverSession(pSerial);
	if (status == SUCCESS) {
		debug(D_WARN,"MicroZ::SetReceiverSessions: Sessions already match, doing nothing\n");
		return(SUCCESS);
	}
	pSerial->ClearBuffers();
	sprintf(buffer, "SCRN,70,Y%03d%04d",refday, offsec);
	SendCommand(pSerial, "SES,DEL");
	if (pSerial->WaitFor(3000, 6) == 0) {
		debug(D_STD_V3,"MicroZ::SetReceiverSession: Sessions deletion ACK'ed\n");
	}

	sprintf(buffer, "SES,PAR,Y,%03d,%04d",refday, offsec);
	SendCommand(pSerial, buffer, TRUE, FALSE);
	if (pSerial->WaitFor(3000, 6) == 0) {
		debug(D_STD_V3,"MicroZ::SetReceiverSession: Reference day ACK'ed\n");
	}
	debug(D_FLOW,"MicroZ::SetReceiverSession: Adding new sessions\n");
	for (int i = 0; i <= 23; i++) {
		if (i < numSessions) {
			start    = GetSessionStartTime(i);
			end      = GetSessionEndTime(i);
			interval = GetSessionInterval(i);
			elevmask = GetSessionElevationMask(i);
			minsv    = GetSessionMinimumSV(i);
			inuse    = 'Y';
		} else {
			start    = 0;
			end      = 0;
			interval = 30;
			elevmask = 0;
			minsv    = 1;
			inuse    = 'N';
		}
		sprintf(buffer, "SES,SET,%c,%c,%02d%02d%02d,%02d%02d%02d,%d,%d,%d,%d", 
			65+i, inuse,
			start/3600, (start%3600)/60, (start%3600)%60,
			end/3600, (end%3600)/60, (end%3600)%60, (int) interval,
			(int) elevmask, (int) minsv, (int) rectype);
		SendCommand(pSerial, buffer, FALSE, TRUE);
		if (pSerial->WaitFor(3000, 6) == 0) {
			debug(D_STD_V3,"MicroZ::SetReceiverSession: Session set ACK'ed\n");
		}
	}
	SendCommand(pSerial,"SAV,Y",FALSE,TRUE);

	status = VerifyReceiverSession(pSerial);
	if (status != 0) {
		debug(D_WARN,"MicroZ::SetReceiverSession: session mismatch after setting\n");
		return(FAILURE);
	}
	status = CloseOpenFile(pSerial);
	debug(D_FLOW,"MicroZ::SetReceiverSession: Exiting...\n");
	return(SUCCESS);

}

//  ===================================================================
//  NAME:		MicroZ::DownloadReceiverFAT
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int MicroZ::DownloadReceiverFAT(Serial *pSerial) {

	int filesfound = 0;
	int totalfiles = 10;

	while (filesfound < totalfiles) {
		char buffer[2048];
		sprintf(buffer,"FIF,%d",filesfound);
		SendQuery(pSerial, buffer);
		int c = pSerial->Read(buffer, 2048, 4000);
		if (c <= 0) break;

		debug(D_SERINFO,"DownloadReceiverFAT: %s\n",buffer);
		string buf = buffer;
		string fields[200];
		for (int i = 0; i < 200; i++) {
			fields[i] = "";
		}

		int pos = 0;
		for (int i = 0; i < 200; i++) {
			size_t endpos = buf.find_first_of(",",pos);
			if (endpos == string::npos) break;
			fields[i] = buf.substr(pos,endpos - pos);
			debug(D_VALUES,"field %d is %s\n",i , fields[i].c_str());
			pos = endpos+1;
		}

		// This should always be in the first two fields;
		if (fields[0].compare("$PASHR") != 0) return(FAILURE);
		if (fields[1].compare("FIF")    != 0) return(FAILURE);


		//int bytesleft    = atoi(fields[2].c_str());
		// reset total files from 10 to number returned
		totalfiles       = atoi(fields[3].c_str());
		int filesinquery = atoi(fields[4].c_str());

// ignore field 5 and start loop at field 6
		int curfield = 5;
		for (int i = filesfound; i < filesinquery+filesfound; i++) {
			char buf[200];
			curfield++; // ignore file number and start at filename
			// Get site name from chars 2-5 of filename
			ReceiverFAT[i].fileName = fields[curfield];
			UZ_FileNames[i]         = fields[curfield];
			debug(D_VALUES,"MicroZ::DownloadReceiverFAT: filename is %s\n",ReceiverFAT[i].fileName.c_str());
			ReceiverFAT[i].siteName.erase(0,ReceiverFAT[i].siteName.length());
			ReceiverFAT[i].siteName += fields[curfield].substr(1,4);
			debug(D_VALUES,"MicroZ::DownloadReceiverFAT: sitename is %s\n",ReceiverFAT[i].siteName.c_str());
			curfield++;

			// Get file start time
			int week, day, hour, minute;
			sscanf(fields[curfield].c_str(), "%4d%1d%2d%2d", &week, &day, &hour, &minute);
			double seconds = ((day-1) * 86400) + (hour * 3600) + (minute * 60);
			ReceiverFAT[i].fileStart.set_time(week, seconds);
			debug(D_VALUES,"MicroZ::DownloadReceiverFAT: Start time is %s\n",ReceiverFAT[i].fileStart.chartime("%D %R %j %y",buf,100));
			curfield++;

			// Get file end time
			sscanf(fields[curfield].c_str(), "%4d%1d%2d%2d", &week, &day, &hour, &minute);
			seconds = ((day-1) * 86400) + (hour * 3600) + (minute * 60);
			ReceiverFAT[i].fileEnd.set_time(week, seconds-1);
			debug(D_VALUES,"MicroZ::DownloadReceiverFAT: End time is %s\n",ReceiverFAT[i].fileEnd.chartime("%D %R",buf,100));
			curfield++;

			// Get size of file
			ReceiverFAT[i].fileSize = atoi(fields[curfield].c_str()) * 1024;
			debug(D_VALUES,"MicroZ::DownloadReceiverFAT: File Size is %d\n",ReceiverFAT[i].fileSize);
			curfield++;

			ReceiverFAT[i].active = ((i+1) == totalfiles) ? TRUE : FALSE;
			// Don't use next field;
			curfield++;
			FindSession(i);
			CreateFileName(i);
		}
		filesfound += filesinquery;

	}
	numFATEntries = filesfound;

	return(SUCCESS);
}

