/* 
#    $Id: Navcom.h,v 1.1 2011/04/04 17:29:34 gps Exp $
#
#    Navcom.h -- Include file for the Navcom Receiver
#
#    Copyright (C) 2000 US Geological Survey
#    Written by Keith F. Stark
#    
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifndef _NAVCOM_H
#define _NAVCOM_H

#include <fstream>
#include <string>
#include <vector>

#include <gpscommon.h>
#include <NavcomDefs.h>
#include <Receiver.h>
#include <gpstime.h>

#define FILELENGTH 12
#define NAVBYTEORDER __BIG_ENDIAN

#if 0
struct Endian_t {
	long	_byte;
	short	_type;
};
typedef Endian_t Endian;
vector<Endian> EndianList;
Endian tmpEndian;
#endif

#define addEndian (x, y) \
	tmpEndian._byte = (x); tmpEndian._type = (y); EndianList.pushback(tmpEndian);

#pragma pack(1)

struct navcom_msg_t {
	U08	STX_;
	U08	Preamble1;
	U08	Preamble2;
	U08	CommandID;
	U16	MessageLength;
	
	union {
		struct navcom_msg_0x11 {
			struct {
				uint	_pad:1;
				uint	nvram:1;
				uint	ack:1;
				uint	dflt:1;
				uint	_pad2:4;
			} w1;
			struct navcom_msg_0x11_ports {
				struct {
					uint	port:1;
					uint	baud:3; // 0=1200,1=2400,...,7=115200
					uint	parity:2; // 0=NONE, 1=ODD, 2 = EVEN
					uint	_pad:2;
				} w2;
				struct {
					uint	_pad;
				} w3;
			} ports[(NAVCOM_MAX_MSG_SIZE - 9)/2];
		} type_0x11;
				
		struct navcom_msg_0x1a {
			struct {
				uint	_pad:1;
				uint	nvram:1;
				uint	_pad1:1;
				uint	dflt:1;
				uint	_pad2:2;
				uint	user_nvram:1;
				uint	_pad3:1;
			} w1;
			struct {
				uint	diag:1;
				uint	control:1;
				uint	data:1;
				uint	nmea:1;
				uint	rtcm:1;
				uint	cmrin:1;
				uint	cmrout:1;
				uint	_pad:5;
				uint	spare:4;
			} port[4];
#if 0
			} portA;
			struct {
				uint	diag:1;
				uint	control:1;
				uint	data:1;
				uint	nmea:1;
				uint	rtcm:1;
				uint	cmrin:1;
				uint	cmrout:1;
				uint	_pad:5;
				uint	spare:4;
			} portB;
			struct {
				uint	diag:1;
				uint	control:1;
				uint	data:1;
				uint	nmea:1;
				uint	rtcm:1;
				uint	cmrin:1;
				uint	cmrout:1;
				uint	_pad:5;
				uint	spare:4;
			} CANPort;
			struct {
				uint	diag:1;
				uint	control:1;
				uint	data:1;
				uint	nmea:1;
				uint	rtcm:1;
				uint	cmrin:1;
				uint	cmrout:1;
				uint	_pad:5;
				uint	spare:4;
			} Radio;
#endif
		} type_0x1a;
#define ENDIAN_0X1A

		struct navcom_msg_0x1c {
			struct {
				uint	_pad:2;
				uint	ack:1;
				uint	_pad2:5;
			} w1;
			U08 testID;
			U08 testID2; // set to value of testID
			U08 reserved;
		} type_0x1c;
#define ENDIAN_0X1C

		struct navcom_msg_0x20 {
			struct {
				uint	_pad:1;
				uint	nvram:1;
				uint	_pad1:1;
				uint	dflt:1;
				uint	clear:1;
				uint	_pad2:1;
				uint	user_nvram:1;
				uint	_pad3:1;
			} w1;
			U08 numrequests;
			struct navcom_msg_0x11_ports {
				U08 blockID;
				struct {
					uint	diag:1;
					uint	control:1;
					uint	data:1;
					uint	network:1;
					uint	reserved:1;
					uint	log:1;
					uint	_pad:2;
				} w2;
				U16 dataRate;
			} requests[(NAVCOM_MAX_MSG_SIZE - 9)/4];
		} type_0x20;

#define ENDIAN_0X20 \
	for (int i = 0; i < (NAVCOM_MAX_MSG_SIZE - 9)/4; i++ ) { \
		addEndian(((i*4) + 2 + 2), RFI_USHORT); \
	}

	
		struct navcom_msg_0x06 {
			U08 IDEcho;
			U08 PhysicalPort;
		} type_0x06;
#define ENDIAN_0X06

		struct navcom_msg_0x15 {
			U08 IDEcho;
			struct navcom_msg_0x15_errors{
				U08 ID;
				U08 Desc;
			} errors[6];
			U08 PhysicalPort;

		} type_0x15;
#define ENDIAN_0X15

		struct navcom_msg_0xae {
			U08 config;
			U08 ASIC;
			struct {
				U08 Major;
				U08 Minor;
			} version;
			struct {
				U16 serial_num;
				U08 serial_class;
			} DigitalCardNumber;
			
			struct {
				U16 serial_num;
				U08 serial_class;
			} RFCardNumber;
			U08	softwareTimeMark[16];
			U08	bootString[16];
			U08	reserved[8];
			struct {
				U08 Major;
				U08 Minor;
			} IOPVersion;
			U08	IOPTimeMark[16];
			U08	PicVersion;
			U08	StarlightBuild;
			U08	IOPBuild;
		} type_0xae;

#define ENDIAN_0XAE \
	addEndian(4, RFI_USHORT); \
	addEndian(6, RFI_USHORT);

		struct navcom_msg_0x7e {
			U08 SubID;
			U08 Command;
			union {
				struct { //0x00 request
				} req_0x00;
				struct { //0x00 response
					struct {
						uint	mday:5;
						uint	mon:4;
						uint	year:7;
					} Date;
					struct {
						uint	sec:5;
						uint	min:6;
						uint	hour:5;
					} Time;
					struct {
						uint	avail:1;
						uint	_pad:3;
						uint	read_open:1;
						uint	write_open:1;
						uint	mounted:1;
						uint	mnt_fail:1;
					} Status;
					U08 Reserved;
					U32 MemorySize;
					U32 MemoryFree;
				} res_0x00;
#define ENDIAN_0X7E_req0x00
#define ENDIAN_0X7E_res0x00 \
	addEndian(2 + 4, RFI_FLOAT); \
	addEndian(2 + 8, RFI_FLOAT);

				struct { // 0x01 request
					U16 PageNumber;
					U08 FileName[14];
				} req_0x01;
				struct { // 0x01 response
					struct {
						uint	mday:5;
						uint	mon:4;
						uint	year:7;
					} Date;
					struct {
						uint	sec:5;
						uint	min:6;
						uint	hour:5;
					} Time;
					struct {
						uint	_pad:1;
						uint	no_page:1;
						uint	final_page:1;
						uint	_pad2:4;
						uint	mnt_fail:1;
					} Result;
					U08 NumEntries;
					U16 Page;
					struct {
						struct {
							uint	mday:5;
							uint	mon:4;
							uint	year:7;
						} CreatDate;
						struct {
							uint	sec:5;
							uint	min:6;
							uint	hour:5;
						} CreatTime;
						struct {
							uint	mday:5;
							uint	mon:4;
							uint	year:7;
						} ModDate;
						struct {
							uint	sec:5;
							uint	min:6;
							uint	hour:5;
						} ModTime;
						U08 Attributes;
						U08 Reserved;
						U32 FileSize;
						U08 FileName[14];
					} DirEntry[18];
				} res_0x01;
#define ENDIAN_0X7E_req0x01 addEndian(2 + 0, RFI_USHORT);
#define ENDIAN_0X7E_res0x01 \
	addEndian(2 + 4, RFI_USHORT); \
	for (int i = 0; i < 18; i++) { \
		addEndian(5 + i*24 + 7, RFI_FLOAT); \
	}

				struct { // 0x03 request
#define SPECIAL_0X03 0x80 | 0x40 | 0x10
					U08 special; // value must be 00001011;
					U08 FileName[14];
				} req_0x03;
				struct { //0x03 response
					struct {
						uint	mday:5;
						uint	mon:4;
						uint	year:7;
					} Date;
					struct {
						uint	sec:5;
						uint	min:6;
						uint	hour:5;
					} Time;
					U16 num_deleted;
					struct {
						uint	_pad:1;
						uint	bad_bits:1;
						uint	_pad2:2;
						uint	bad_filename:1;
						uint	_pad3:2;
						uint	mnt_fail:1;
					} Result;
					U08 reserved;
					U08 filename[14];
				} res_0x03;
#define ENDIAN_0X7E_req0x03
#define ENDIAN_0X7E_res0x03 addEndian(2 + 2, RFI_USHORT);

				struct { // 0x08 request
					U16 reserved;
					U08 FileName[14];
				} req_0x08;
				struct { // 0x08 response
					struct {
						uint	no_file:1;
						uint	open_err:1;
						uint	other_file:1;
						uint	_pad:5;
					} Result;
					U08 spare;
					U16 reserved;
					U16 fh;
					U32 sec_size;
					U32 byte_size;
					struct {
						uint	mday:5;
						uint	mon:4;
						uint	year:7;
					} CreatDate;
					struct {
						uint	sec:5;
						uint	min:6;
						uint	hour:5;
					} CreatTime;
					struct {
						uint	mday:5;
						uint	mon:4;
						uint	year:7;
					} ModDate;
					struct {
						uint	sec:5;
						uint	min:6;
						uint	hour:5;
					} ModTime;
					U16 attributes;
					struct {
						uint	mday:5;
						uint	mon:4;
						uint	year:7;
					} AccDate;
					U08 FileName[14];
				} res_0x08;
#define ENDIAN_0X7E_req0x08 addEndian(2 + 0, RFI_USHORT);
#define ENDIAN_0X7E_res0x08 \
	addEndian(2 + 2, RFI_USHORT); \
	addEndian(2 + 4, RFI_USHORT); \
	addEndian(2 + 8, RFI_FLOAT); \
	addEndian(2 + 12, RFI_FLOAT); \
	addEndian(2 + 17, RFI_USHORT);

				struct { //0x09 request
					U16 fh;
					U32 sec_number;
					U08 spare;
				} req_0x09;
				struct { //0x09 response
					struct {
						uint	fh_fail:1;
						uint	sec_fail:1;
						uint	read_fail:1;
						uint	_pad:4;
						uint	mnt_fail:1;
					} Result;
					U08 spare;
					U16 fh;
					U32 sec_number;
					U32 bytes;
					U08 data[512];
				} res_0x09;
#define ENDIAN_0X7E_req0x09 \
	addEndian(2 + 0, RFI_USHORT); \
	addEndian(2 + 2, RFI_FLOAT);
#define ENDIAN_0X7E_res0x09 \
	addEndian(2 + 2, RFI_USHORT); \
	addEndian(2 + 4, RFI_FLOAT); \
	addEndian(2 + 8, RFI_FLOAT);
				struct { //0x0a request
					U16 reserved;
					struct {
						uint	_pad:1;
						uint	read_close:1;
						uint	write_close:1;
						uint	_pad2:5;
					} Control;
				} req_0x0a;
				struct { //0x0a response
					U16 reserved;
					struct {
						uint	_pad:1;
						uint	read_close:1;
						uint	write_close:1;
						uint	_pad2:5;
					} Control;
					struct {
						uint	rclose_nofile:1;
						uint	rclose_error:1;
						uint	wclose_nofile:1;
						uint	wclose_error:1;
						uint	_pad:3;
						uint	mnt_fail:1;
					} Result;
				} res_0x0a;
#define ENDIAN_0X7E_req0x0a addEndian(2 + 0, RFI_USHORT);
#define ENDIAN_0X7E_res0x0a addEndian(2 + 0, RFI_USHORT);

			} subID_type;
		} type_0x7e;
#define ENDIAN_0X7E
	} msg_type;
};

typedef navcom_msg_t Navcom_Msg;

class Navcom : public Receiver {
public:
	Navcom();

	static string DownloadReceiverID(Serial *pSerial);
// Download fat
	static int GetFATEntryAtIndex(SFATEntry *pFAT, int index);
	static int GetReceiverID();
	static int Stream(Serial *pSerial, int format, double rate, char port);
	static int BinexStream(Serial *pSerial, double rate, char port);
	static long AutoBaud(Serial *pSerial);
	static int GetPortInfo(Serial *pSerial, char *port, long *speed);
	static int SetPortInfo(Serial *pSerial, long baud);
	static int Login(Serial *pSerial, int direct, string login, string password);
	static int TerminalSession(Serial *pSerial);
	static int CompareSessions(SNavcomSession *tmpsession);



	static int CloseOpenFile(Serial *pSerial);

	// Download File routines
	int DownloadFileByTime(Serial *pSerial, gpstime *start, gpstime *stop);
	int DownloadFileAtIndex(Serial *pSerial, int index);


	int DeleteFileByTime(Serial *pSerial, gpstime *start, gpstime *stop);
	int DeleteFileAtIndex(Serial *pSerial, int index);
	int WriteFiles();
	int setImageFile(unsigned char *myImage, int fileSize);

	void AddImage(Navcom *newNavcom);
	void AddImage(unsigned char *image, int fileSize, SNavcomFATEntry *myFAT);

	void AddToIndexBuffer(int index, unsigned char *in, size_t size);

	static int DownloadReceiverFAT(Serial* pSerial);
	static int FindNCTPacket(Serial *pSerial, time_t timeout, int packet = -1);
	static int BuildDataRequest(int blockID,  int dataRate, char outputPorts, bool newReq = TRUE);
	static int BuildFileRequest(int subID);
	static int BuildFileRequest(int subID, int Page, const char * fileSpec); // 0x01
	static int BuildFileRequest(int subID, char * fileSpec); // 0x08
	static int BuildFileRequest(int subID, U16 fh, U32 sector); // 0x09

	static int DecodeNCTPacket(int id, U16 msg_size, unsigned char *buffer, int index);

	static int SendQuery(Serial *pSerial, string QueryString);
	static int SendCommand(Serial *pSerial, string CommandString, int DoCheckSum = FALSE, int WaitAck = TRUE);
	static int CheckAck(Serial *pSerial);
	
	virtual unsigned char *getImageFile() { return(imageFile); }
	virtual int getImageFileSize() { return(cimageFile); }
	virtual int getIndex() { return(myIndex); }
	
protected:

	static vector <int> blockRequests;

	// File Data
	SNavcomFATEntry *pFATentry;
	RawImageHeader2 *pImageHead;	
	
	int cimageFile;
	int myIndex;

	// sitename and times
	string sitename;
	
	// Actual RFile image
	unsigned char *imageFile;
	unsigned char *header;
	unsigned char *fatEntry;
	
	unsigned char *indexData[MAX_RCVR_FILES];
	char session;

};

#endif
