/* 
#    $Id: NavcomDefs.h,v 1.1 2011/04/04 17:29:34 gps Exp $
#
#    NavcomDefs.h -- Data Structures 
#
#    Copyright (C) 2000 US Geological Survey
#    Written by Keith F. Stark
#    
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifndef _NAV_DEFS_H
#define _NAV_DEFS_H

//NAVCOM type definitions
typedef unsigned char	U08;
typedef unsigned short	U16;
typedef unsigned long	U32;

typedef char		S08;
typedef short		S16;
typedef long		S32;

typedef float		R32;
typedef double		R64;

#define MAX_REQUESTS 20

//NAVCOM message structure values
#define NC_STX  0x02
#define NC_PREAMBLE_1 0x99
#define NC_PREAMBLE_2 0x66
#define NC_ETX  0x03
#define NC_QUERY_ID 0x20

//NAVCOM Port info
#define PORT_DIAG	0x1
#define PORT_CONTROL	0x2
#define PORT_DATA	0x4
#define PORT_LOG	0x8
#define PORT_ALL	0xFF

//Data Rate Definitions
#define RATE_ONCE	0x0
#define RATE_2HZ	0x1
#define RATE_5HZ	0x2
#define RATE_10HZ	0x3
#define RATE_25HZ	0x4
#define RATE_50HZ	0x5
#define RATE_SECS(a)	((a) * 0xa)
#define RATE_EVENT	0x2000
#define RATE_CHANGE	0x4000
#define RATE_SV(a)	((a) + 0x8000)

/****************************************************************************/
/* these defs are for the new development                                   */
/****************************************************************************/

#define GENERAL_TIMEOUT 10000L
#define DELETION_TIMEOUT 120000L
#define NAVCOM_XMODEM_CLEANUP_TIMEOUT 10000L

#define MAX_PROG_SESSIONS 24
#define MAX_NAVCOM_FILES 100
#define INTERMEDIATEHEADERLEN 20

/****************************************************************************/
#define SITENAME_LEN 4    /* The number of charactes used by the site name. */
#define OPERATOR_LEN 12   /* Number of characters for operator field.       */
#define PROJECT_LEN  34   /* Number of characters for project field.        */
/****************************************************************************/

// Define records for reading in Navcom data and navigation records
// These definitions are as printed from the Navcom filetool program

// this is in bytes (max message is 512 byte data message + overhead)
#define NAVCOM_MAX_MSG_SIZE 576

struct RawImageHeader2 {
	char receiverType[20];
	char version[20];
	char oneFileMode;
	char res;
	long totalMemory;
	long FATstart;
	long FATend;
	long photoStart;
	long photoEnd;
	long almanacStart;
	long almanacEnd;
	long dataStart;
	long dataEnd;
	char spare[71];
	char imageHeadVers;
};

#define RAWIMAGEHEADERSIZE_F 150

struct NavcomFATEntry {
	long         segbeg;
	long         wordcount;
	char         segname  [SITENAME_LEN]; /* 4 */
	char         ranger_mode;
	char         reserv;
	short        wnbeg;
	long         timebeg;
	char         oper[4];
	char         project  [PROJECT_LEN]; /* 34 */
	short        segwn; /* was int */
	long         segtime;
	unsigned short anthibef;  /* was unsigned int */
	unsigned short anthiaft;  /* was unsigned int */
	char         tdrybef;
	char         tdryaft;
	char         twetbef;
	char         twetaft;
	char         humidbef;
	char         humidaft;
	short        pressurebef;     /* was int */
	short        pressureaft;     /* was int */
	unsigned short checksum;       /* was unsigned int */
};
typedef struct NavcomFATEntry SNavcomFATEntry;

#define NAVCOMFATENTRYSIZE_F 80

#define  NUM_SESS 24
struct NavcomSessionStruct {
	short	refday;
	short	offset;
	short	inuse;
	char	flag[NUM_SESS];
	char	colstat[NUM_SESS];
	long	start[NUM_SESS];
	long	end[NUM_SESS];
	long	span[NUM_SESS];
	float	interval[NUM_SESS];
	char	elevmask[NUM_SESS];
	char	minsv[NUM_SESS];
	char	type[NUM_SESS];
	unsigned short	checksum;
};

typedef struct NavcomSessionStruct SNavcomSession;

#endif /*define NAV_DEFS_H */
