/* 
#    $Id: Receiver.cpp,v 1.1 2011/04/04 17:29:34 gps Exp $
#
#    Receiver.h -- Basic Receiver class definition
#
#    Copyright (C) 2000 US Geological Survey
#    Written by Keith F. Stark
#    
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <iostream>
#include <fstream>
#include <iomanip>
#include <string>
#include <sys/types.h>
#include <dirent.h>

#include "gpscommon.h"
#include "gpstime.h"
#include "Receiver.h"

string Receiver::FileTemplate;
string Receiver::FileTemplateStrings[MAXTEMPLATESTRINGS];
string Receiver::FilePath;

//  ===================================================================
//  NAME:		Receiver::AddToIndexList
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
void Receiver::AddToIndexList(int index) {
	if (numlist < MAXINDEXLIST) {
		indexList[numlist++] = index;
	}
}

//  ===================================================================
//  ===================================================================
//  NAME:		Receiver::FindSession
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
void Receiver::FindSession(int index) {
	int year, day, iyear, iday, i;
	char buffer[10];

	debug(50,"Receiver::FindSession: Entering...\n");
	ReceiverFAT[index].session = 'A';
	iyear = atoi(ReceiverFAT[index].fileStart.chartime("%y",buffer, 3));
	iday = atoi(ReceiverFAT[index].fileStart.chartime("%j",buffer, 4));

	for (i = 0; i < index; i++) {
		year = atoi(ReceiverFAT[i].fileStart.chartime("%y",buffer, 3));
		day = atoi(ReceiverFAT[i].fileStart.chartime("%j",buffer, 4));
		if ((iyear == year) && (iday == day)) ReceiverFAT[index].session++;
	}
	debug(50,"Receiver::FindSession: Exiting...\n");
}

//  ===================================================================
//  NAME:		Receiver::CreateFileName
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
void Receiver::CreateFileName(int index, gpstime *time) {
	char buffer[256];

	debug(50,"Receiver::CreateFileNames: Entering...\n");
	int i = ReceiverFAT[index].siteName.length();
	for (; i < 4; i++) {
		ReceiverFAT[index].siteName[i] = '_';
	}
	for (i = 1; i < 4; i++) {
		if (ReceiverFAT[index].siteName[i] == '?')
			ReceiverFAT[index].siteName[i] = '_';
	}

	string filename = FileTemplate;
	while (filename.find("++") != string::npos) {
		filename.replace(filename.find("++"),2,"+");
	}
	while (filename.find("+S") != string::npos) {
		filename.replace(filename.find("+S"),2,ReceiverFAT[index].siteName);
	}
	while (filename.find("+s") != string::npos) {
		filename.replace(filename.find("+s"),2,1,ReceiverFAT[index].session);
	}

	while (filename.find("+0") != string::npos) {
		filename.replace(filename.find("+0"),2,Receiver::FileTemplateStrings[0]);
	}
	while (filename.find("+1") != string::npos) {
		filename.replace(filename.find("+1"),2,Receiver::FileTemplateStrings[1]);
	}
	while (filename.find("+2") != string::npos) {
		filename.replace(filename.find("+2"),2,Receiver::FileTemplateStrings[2]);
	}
	while (filename.find("+3") != string::npos) {
		filename.replace(filename.find("+3"),2,Receiver::FileTemplateStrings[3]);
	}
	while (filename.find("+4") != string::npos) {
		filename.replace(filename.find("+4"),2,Receiver::FileTemplateStrings[4]);
	}
	while (filename.find("+5") != string::npos) {
		filename.replace(filename.find("+5"),2,Receiver::FileTemplateStrings[5]);
	}
	while (filename.find("+6") != string::npos) {
		filename.replace(filename.find("+6"),2,Receiver::FileTemplateStrings[6]);
	}
	while (filename.find("+7") != string::npos) {
		filename.replace(filename.find("+7"),2,Receiver::FileTemplateStrings[7]);
	}
	while (filename.find("+8") != string::npos) {
		filename.replace(filename.find("+8"),2,Receiver::FileTemplateStrings[8]);
	}
	while (filename.find("+9") != string::npos) {
		filename.replace(filename.find("+9"),2,Receiver::FileTemplateStrings[9]);
	}

	if (time != NULL) {
			debug(40,"Receiver::CreateFileNames: Using %s for file naming\n", time->chartime("%Y/%j/%T", buffer, 100));

			filename = time->chartime(filename.c_str(), buffer, 255);
			debug(50,"Receiver::CreateFileNames: Using given time to produce %s\n", filename.c_str());
	} else {
			debug(50,"Receiver::CreateFileNames: Using file time\n");
			filename = ReceiverFAT[index].fileStart.chartime(filename.c_str(), buffer, 255);
	}

	ReceiverFAT[index].fileName = filename;

	debug(25,"Receiver::CreateFileNames: filename is %s\n",filename.c_str());
	debug(50,"Receiver::CreateFileNames: Exiting...\n");
}

//  ===================================================================
//  NAME:		Receiver::FindIndexesByTime
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int Receiver::FindIndexesByTime(gpstime *start, gpstime *stop, int *indexes, int completely_within) {
	int cFAT,act;
	int numindex = 0;

	debug(50,"FindIndexesByTime: Entering...\n");
	for (cFAT = 0; cFAT < numFATEntries; cFAT++) {
		int fatstart = GetStartTime(cFAT).gettime_t() + 1;
		int fatend = GetEndTime(cFAT).gettime_t();
		int rangestart = start->gettime_t();
		int rangeend = stop->gettime_t();
		act = FALSE;
		if (completely_within) {
			//File Starts and stops within range
			if ((fatstart >= rangestart) && (fatend   <= rangeend)) act = TRUE;
		} else {
			// File starts or stops within range
			if ((fatstart >= rangestart) && (fatstart <= rangeend)) act = TRUE;
			if ((fatend   >= rangestart) && (fatend   <= rangeend)) act = TRUE;
			// file encompasses range but starts on either side
			if ((fatstart <= rangestart) && (fatend   >= rangeend)) act = TRUE;
		}
		if (act) {
			if (numlist == 0) {
					indexes[numindex] = cFAT;
					numindex++;
			} else {
				for (int i = 0; i < numlist; i++) {
					if (indexList[i] == cFAT) {
						debug(25,"FindIndexByTime: Adding index %d as entry #%d\n",cFAT, numindex);
						indexes[numindex] = cFAT;
						numindex++;
						break;
					}
				}
			}
		}
	}
	debug(50,"FindIndexesByTime: Exiting...\n");
	return(numindex);
}

//  ===================================================================
//  NAME:		Receiver::OpenFile
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int Receiver::OpenFile(ifstream *pfd, string filename) {

	string tmpfile;
	debug(50,"Receiver::OpenFile(ifstream, filename): Entering...\n");

	if (FilePath.length() == 0) {
		FilePath = "./";
	}
	if (FilePath.rfind('/') != FilePath.length()) {
		FilePath += "/";
	}

	tmpfile = FilePath + filename;

	pfd->open(tmpfile.c_str());
	if (!*pfd) {
		debug(5,"Can't open %s\n",tmpfile.c_str());
		return(FAILURE);
	}
	debug(50,"Receiver::OpenFile: Exiting\n");
	return(SUCCESS);

}

//  ===================================================================
//  NAME:		Receiver::OpenFile
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int Receiver::OpenFile(ofstream *pfd, string filename) {
	string tmpfile;

	debug(50,"Receiver::OpenFile(ofstream, filename): Entering...\n");
	if (FilePath.length() == 0) {
		FilePath = "./";
	}
	if (FilePath.rfind('/') != FilePath.length()) {
		FilePath += "/";
	}

	tmpfile = FilePath + filename;

	pfd->open(tmpfile.c_str());
	if (!*pfd) {
		debug(5,"Can't open %s\n",tmpfile.c_str());
		return(FAILURE);
	}
	debug(50,"Receiver::OpenFile: Exiting\n");
	return(SUCCESS);
}

//  ===================================================================
//  NAME:		Receiver::AddSession
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int Receiver::AddSession(long start, long end, float interval, short mask, short minsv) {
	debug(50,"Receiver::AddSession: Entering...\n");
	ReceiverSessions[numSessions].start = start;
	ReceiverSessions[numSessions].end = end;
	ReceiverSessions[numSessions].interval = interval;
	ReceiverSessions[numSessions].elevmask = mask;
	ReceiverSessions[numSessions].minsv = minsv;
	numSessions++;
	debug(50,"Receiver::AddSession: Exiting...\n");
	return(SUCCESS);
}

//  ===================================================================
//  NAME:		Receiver::ResetSessions
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int Receiver::ResetSessions() {
	numSessions = 0;
	debug(50,"Receiver::ResetSession: Entering...\n");
	for (int i = 0; i < MAX_RCVR_SESSIONS; i++) {
		AddSession(0,0,0.0,0,0);
	}
	numSessions = 0;
	debug(50,"Receiver::ResetSession: Exiting...\n");
	return(numSessions);
}

