/* 
#    $Id: Receiver.h,v 1.3 2012/02/10 00:48:57 gps Exp $
#
#    Receiver.h -- Basic Receiver class definition
#
#    Copyright (C) 2000 US Geological Survey
#    Written by Keith F. Stark
#    
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifndef _RECEIVER_H
#define _RECEIVER_H

#define VIRTUAL virtual
#include <iostream>
#include <fstream>
#include <string>
#include <gpstime.h>
#include <gpscommon.h>
#include <serial.h>


#define MAXINDEXLIST 500

class Receiver;

struct FATEntry {
	string siteName;
	long fileSize;
	gpstime fileStart;
	gpstime fileEnd;
	string fileName;
	string fatName;
	char session;
	short active;
};
typedef struct FATEntry SFATEntry;

struct FileOpsStruct {
	class Receiver *pReceiver;
	int downLoad;
	int downLoadOK;
	int purge;
	int index;

	int indexlistnum;
	int indexlist[MAXINDEXLIST]; // used to limit time range search
	gpstime *fileStart;
	gpstime *fileEnd;
	char session;

};
typedef struct FileOpsStruct SFileOps;

struct SessionStruct {
	long start;
	long end;
	float interval;
	short elevmask;
	short minsv;
};
typedef struct SessionStruct SSession;

struct RecInfoStruct {
	//modem stuff
	short usemodem;
	string modemserialport;
	string modemtype; // from ATI3?
	int modembaud;
	string modeminitstring;
	string modemconfigmode; // ie. AT&F1\r\n
	string modemname;
	string modemdata2command; //+++AT
	string modemcommand2data; //ATO\r\n
	
	//site stuff
	string sessionname;
	string recserialnum;
	string antserialnum;
	int	month;
	int day;
	string operatorid;
	string usercomment;
	double antennaheight;
	int drytemp;
	int wettemp;
	int humidity;
	int barometric;
};
typedef struct RecInfoStruct SRecInfo;
	

#define MAX_RCVR_FILES 2048
#define MAX_RCVR_SESSIONS 24
#define MAXTEMPLATESTRINGS 10

struct OpsStruct {
	int verifySiteCode;
	int writeFAT;
	int getFAT;
	int getID;
	int setSessions;
	int joinFiles;
	int upperCaseNames;
	int useTerminal;
	int forceTerminal;
	int retries;
#ifdef USE_STREAM
	int stream;
	double streamSample;
	char streamPort;
	int streamType;
	int stats;
	int leapsec;
// ntrip connection to stream
	int isNTRIP;
	string ntripMount;
	string ntripUser;
	string ntripPassword;

#ifdef RTCM_ENABLE
	bool translateRTCM;
	short RTCMVersion;
	string RTCMantennafile;
#endif
	int forceStream;
#endif

	int sessionOffset;
	int numFileOps;
	int numFiles;
	string siteCodeToVerify;

	int ReceiverType;
	string ReceiverTypeString;
	string FileTemplate;
	string TemplateStrings[MAXTEMPLATESTRINGS];
	int numTemplateStrings;
	int FileTimeFrom;

	string FATFileName;
	string SUMFileName;

	string DataDir;

	int purgeAll;
	int downLoadAll;

	SFileOps *fileOps[10*MAX_RCVR_FILES];
};
typedef struct OpsStruct SOps;

class Receiver {

public:
	Receiver() { return; };
	virtual ~Receiver() { };
// Able to access receiver even if Class doesn't exist
	static string DownloadReceiverID(Serial *pSerial)
			{ return(""); };
	static int DownloadReceiverFAT(Serial *pSerial)
			{ return(1); };
	static int GetFATEntryAtIndex(int index)
			{ return(1); };
	static string GetReceiverID()
			{ return RxID; };
	static int GetNumFATEntries()
			{ return numFATEntries; };
	static int GetFileSize(int index)
			{ return ReceiverFAT[index].fileSize; };
	static gpstime GetStartTime(int index)
			{ return ReceiverFAT[index].fileStart; };
	static gpstime GetEndTime(int index)
			{ return ReceiverFAT[index].fileEnd; };
	static string GetFileName(int index)
			{ return ReceiverFAT[index].fileName; };
	static string GetSiteName(int index)
			{ return ReceiverFAT[index].siteName; };
	static char GetSession(int index)
			{ return ReceiverFAT[index].session; };
	static int IsFileActive(int index)
			{ return ReceiverFAT[index].active; };
	static int MemoryUsed()
			{ return memoryused; };
	static int MemoryAvailable()
			{ return memoryavailable; };
	static int Login(Serial *pSerial, int direct, string login, string password)
			{ return(0); };
	static int Logout(Serial *pSerial)
			{ return(0); };
	static int Stream(Serial *pSerial, int format, double rate, char port)
			{ return(0); };
	static int TerminalSession (Serial *)
			{ return(0); };

	static void FindSession(int index);
	static void CreateFileName(int index, gpstime *time = NULL);

	static int AddSession(long start, long end, float interval, short mask, short minsv);
	static int ResetSessions();
	void AddToIndexList(int index);
	static int GetNumSessions()
			{ return(numSessions); };
	static long GetSessionStartTime(int session)
			{ return(ReceiverSessions[session].start); };
	static long GetSessionEndTime(int session)
			{ return(ReceiverSessions[session].end); };
	static float GetSessionInterval(int session)
			{ return(ReceiverSessions[session].interval); };
	static short GetSessionElevationMask(int session)
			{ return(ReceiverSessions[session].elevmask); };
	static short GetSessionMinimumSV(int session)
			{ return(ReceiverSessions[session].minsv); };
	static void SetFileTemplate(string mytemplate)
			{ Receiver::FileTemplate = mytemplate; };
	static void SetTemplateStrings(string ts[MAXTEMPLATESTRINGS])
			{ for (int i = 0; i < MAXTEMPLATESTRINGS; Receiver::FileTemplateStrings[i] = ts[i], i++); };
	static int SetFilePath(string pathname)
			{ Receiver::FilePath = pathname; return(0); };

	VIRTUAL void SetSession(int index, char session)
			{ ReceiverFAT[index].session = session; };
	VIRTUAL int FindIndexesByTime(gpstime *start, gpstime *stop, int *indexes, int completely_within = FALSE);
	VIRTUAL int UsesFAT() { return(usesFAT); };
	VIRTUAL int DownloadFileByTime(Serial *pSerial, gpstime *start, gpstime *stop) = 0;
	VIRTUAL int DownloadFileAtIndex(Serial *pSerial, int index) = 0;
	VIRTUAL int DeleteFileByTime(Serial *pSerial, gpstime *start, gpstime *stop) = 0;
	VIRTUAL int DeleteFileAtIndex(Serial *pSerial, int index) = 0;
	VIRTUAL int WriteFiles() = 0;
	VIRTUAL string GetSiteName() { return(""); };
	VIRTUAL int OpenFile(ofstream *, string);
	VIRTUAL int OpenFile(ifstream *, string);
	VIRTUAL int getIndex() = 0;

protected:
	static string RxID;
	static int numFATEntries;
	static int numSessions;
	static int memoryused;
	static int memoryavailable;
	static SFATEntry ReceiverFAT[MAX_RCVR_FILES];
	static SSession ReceiverSessions[MAX_RCVR_SESSIONS];
	static string FileTemplate;
	static string FileTemplateStrings[MAXTEMPLATESTRINGS];
	static string FilePath;
	static SRecInfo ReceiverInfo;


	gpstime *FileTimeFrom;
	int indexList[MAXINDEXLIST];
	int numlist;
	int usesFAT;
	
};

#endif
