#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdbool.h>
#include <ctype.h>

#include "gpscommon.h"
#include "StreamDBEN.h"

#if USE_STREAM

int StreamDBEN::GetRecordTime(long *gpsMinutes, unsigned short *gpsm_Seconds)
{
	*gpsMinutes = gpsMinute;
	*gpsm_Seconds = gpsFrac;
	return(0);
}

long long int StreamDBEN::GetRecordInfo(int INFOTYPE)
{
	return(-1000000);
}


//
//
// Ashtech Routines
//
//
unsigned long StreamDBEN::extract_int (unsigned char *rec, unsigned char s, unsigned long offset, unsigned long len)
{
	/* offset and len are in bits (8 bits = 1 byte)
	*/
	unsigned long	ul;
	unsigned char	uc;
	unsigned long	first_byte, final_byte, first_bit, final_bit;
	unsigned short	i;
	unsigned char	bit[8];
	bit[0]= 0x01; bit[1]= 0x02; bit[2]= 0x04; bit[3]= 0x08; bit[4]= 0x10; bit[5]= 0x20; bit[6]= 0x40; bit[7]= 0x80;

	first_byte=  offset           /8;
	final_byte= (offset + len - 1)/8;

	first_bit=  offset            - 8*first_byte;
	final_bit= (offset + len - 1) - 8*final_byte;

	switch (first_byte == final_byte) {
	case 1:
		uc= rec[first_byte] >> (7-final_bit);
		for (i= 7; i > final_bit - first_bit; i--)
			uc&= ~bit[i];
		ul= (unsigned long)uc;
		break;
	case 0:
		/* bits from first byte:
		*/
		uc= rec[first_byte];
		for (i= 0; i < first_bit; i++)
			uc&= ~bit[7-i];
		ul= (unsigned long)uc;

		/* bits from middle bytes:
		*/
		for (i= first_byte+1; i < final_byte; ul+= (unsigned long)rec[i++])
			ul<<= 8;

		/* bits from final byte:
		*/
		ul<<= final_bit+1;
		ul+= (unsigned long)(rec[final_byte] >> (7-final_bit));
		break;
	}

	if (s && rec[first_byte] & bit[7-first_bit]) {
		switch (len) {
			case 31: ul|= 0x80000000; break;
			case 30: ul|= 0xc0000000; break;
			case 29: ul|= 0xe0000000; break;
			case 28: ul|= 0xf0000000; break;
			case 27: ul|= 0xf8000000; break;
			case 26: ul|= 0xfc000000; break;
			case 25: ul|= 0xfe000000; break;
			case 24: ul|= 0xff000000; break;
			case 23: ul|= 0xff800000; break;
			case 22: ul|= 0xffc00000; break;
			case 21: ul|= 0xffe00000; break;
			case 20: ul|= 0xfff00000; break;
			case 19: ul|= 0xfff80000; break;
			case 18: ul|= 0xfffc0000; break;
			case 17: ul|= 0xfffe0000; break;
			case 16: ul|= 0xffff0000; break;
			case 15: ul|= 0xffff8000; break;
			case 14: ul|= 0xffffc000; break;
			case 13: ul|= 0xffffe000; break;
			case 12: ul|= 0xfffff000; break;
			case 11: ul|= 0xfffff800; break;
			case 10: ul|= 0xfffffc00; break;
			case  9: ul|= 0xfffffe00; break;
			case  8: ul|= 0xffffff00; break;
			case  7: ul|= 0xffffff80; break;
			case  6: ul|= 0xffffffc0; break;
			case  5: ul|= 0xffffffe0; break;
			case  4: ul|= 0xfffffff0; break;
			case  3: ul|= 0xfffffff8; break;
			case  2: ul|= 0xfffffffc; break;
			case  1: ul|= 0xfffffffe; break;
		}
	}

	return (ul);
}


/********************************************************************/

#ifdef RTCM_ENABLE
int StreamDBEN::DecomposePacket ()
{
	/* note: the Ashtech records have the 11-bytes header replaced with a 2-bytes header:
		byte 0 = custom record type definition
		byte 1 = original record length - 13 (in bytes), i.e. length of body of record
	         also, the 2-byte trailer (\r\n) for each record is gone
	*/
	unsigned char * rec;
	unsigned char * rec_p;
	unsigned long	ul;
	unsigned long SVs;

	unsigned char ret;
	unsigned char	id;
	unsigned short	bit, i, n;

	if (packet_type != DBEN_RECOGNIZED) {
		gpsMinute = 0;
		gpsFrac = 0;
		return(0);
	}
	size_t rec_len = cur_packet.size();
	debug(100,"StreamDBEN::DecomposePacket: Entering...\n");
	rec = rec_p = new unsigned char[rec_len];
	for (i = 0; i < rec_len; i++) {
		rec[i] = cur_packet[i];
	}

	rec += 11;

	/* extract receive_time and convert to seconds
	*/
	ul= extract_int(rec + 2, 0x00, 0, 30);

	// have to handle week changes correctly
	// if computer time is in first hour of week
	// and gps time is in last day of week
	// subtract a week because of clock issues

	gpstime computer_time;
	int week; double sec;

	gpsMinute = ul / (60*1000);
	gpsFrac = ul % (60*1000);
	computer_time.set_time();
	computer_time.getgps(&week,&sec); 

	if ((sec < 3600) && (gpsMinute > (6 * 24 * 60))) {
		debug(10,"StreamDBen:: Subtracting week because of Sunday night problem. week is %d\n",week);
		week--;
	}
	gpsMinute += (week * 7 * 24 * 60);

	InitRTCMStruct();

	id= rec[0];
	SVs= extract_int(rec + 2, 0x00, 62, 32);
	rec+= 2;
	for (i= n= bit = 0; i < 0x20; i++) {
		if (SVs >> i & BIT_0) {

			SetSessionPRN(n, i+0x01);

			switch (id) {
				case ASHTECH_DBN_RCA:
					Ashtech_DBN_phs_rng(n, rec, &bit, RTCM_L1, RTCM_C1);

					rtcm.msg_data.obs_data[RTCM_L1].obs_avail = true;
					rtcm.msg_data.obs_data[RTCM_L1].obs[n].cacode = 0;
					rtcm.msg_data.obs_data[RTCM_L1].obs[n].usable = 1;

					rtcm.msg_data.obs_data[RTCM_C1].obs_avail = true;
					rtcm.msg_data.obs_data[RTCM_C1].obs[n].cacode = 0;
					rtcm.msg_data.obs_data[RTCM_C1].obs[n].usable = 1;

					rtcm.msg_data.numobs++;
					break;
				case ASHTECH_DBN_RP1:
					Ashtech_DBN_phs_rng(n, rec, &bit, RTCM_L1, RTCM_C1);

					rtcm.msg_data.obs_data[RTCM_L1].obs_avail = true;
					rtcm.msg_data.obs_data[RTCM_L1].obs[n].cacode = 0;
					rtcm.msg_data.obs_data[RTCM_L1].obs[n].usable = 1;

					rtcm.msg_data.obs_data[RTCM_C1].obs_avail = true;
					rtcm.msg_data.obs_data[RTCM_C1].obs[n].cacode = 0;
					rtcm.msg_data.obs_data[RTCM_C1].obs[n].usable = 1;

					Ashtech_DBN_phs_rng(n, rec, &bit, RTCM_L1, RTCM_P1);

					rtcm.msg_data.obs_data[RTCM_L1].obs_avail = true;
					rtcm.msg_data.obs_data[RTCM_L1].obs[n].cacode = 0;
					rtcm.msg_data.obs_data[RTCM_L1].obs[n].usable = 1;

					rtcm.msg_data.obs_data[RTCM_P1].obs_avail = true;
					rtcm.msg_data.obs_data[RTCM_P1].obs[n].cacode = 1;
					rtcm.msg_data.obs_data[RTCM_P1].obs[n].usable = 1;

					rtcm.msg_data.numobs++;
					break;
				case ASHTECH_DBN_RP2:
					Ashtech_DBN_phs_rng(n, rec, &bit, RTCM_L1, RTCM_C1);

					rtcm.msg_data.obs_data[RTCM_L1].obs_avail = true;
					rtcm.msg_data.obs_data[RTCM_L1].obs[n].cacode = 0;
					rtcm.msg_data.obs_data[RTCM_L1].obs[n].usable = 1;

					rtcm.msg_data.obs_data[RTCM_C1].obs_avail = true;
					rtcm.msg_data.obs_data[RTCM_C1].obs[n].cacode = 0;
					rtcm.msg_data.obs_data[RTCM_C1].obs[n].usable = 1;
					Ashtech_DBN_phs_rng(n, rec, &bit, RTCM_L2, RTCM_P2);

					rtcm.msg_data.obs_data[RTCM_L2].obs_avail = true;
					rtcm.msg_data.obs_data[RTCM_L2].obs[n].cacode = 0;
					rtcm.msg_data.obs_data[RTCM_L2].obs[n].usable = 1;

					rtcm.msg_data.obs_data[RTCM_P2].obs_avail = true;
					rtcm.msg_data.obs_data[RTCM_P2].obs[n].cacode = 1;
					rtcm.msg_data.obs_data[RTCM_P2].obs[n].usable = 1;

					rtcm.msg_data.numobs++;
					break;
				case ASHTECH_DBN_RPC:
					Ashtech_DBN_phs_rng(n, rec, &bit, RTCM_L1, RTCM_P1);

					rtcm.msg_data.obs_data[RTCM_L1].obs_avail = true;
					rtcm.msg_data.obs_data[RTCM_L1].obs[n].cacode = 0;
					rtcm.msg_data.obs_data[RTCM_L1].obs[n].usable = 1;

					rtcm.msg_data.obs_data[RTCM_P1].obs_avail = true;
					rtcm.msg_data.obs_data[RTCM_P1].obs[n].cacode = 1;
					rtcm.msg_data.obs_data[RTCM_P1].obs[n].usable = 1;
					Ashtech_DBN_phs_rng(n, rec, &bit, RTCM_L2, RTCM_P2);

					rtcm.msg_data.obs_data[RTCM_L2].obs_avail = true;
					rtcm.msg_data.obs_data[RTCM_L2].obs[n].cacode = 0;
					rtcm.msg_data.obs_data[RTCM_L2].obs[n].usable = 1;

					rtcm.msg_data.obs_data[RTCM_P2].obs_avail = true;
					rtcm.msg_data.obs_data[RTCM_P2].obs[n].cacode = 1;
					rtcm.msg_data.obs_data[RTCM_P2].obs[n].usable = 1;

					rtcm.msg_data.numobs++;
					break;
			}
			n++;
		}
	}

	ret = rec_p[0];
	debug(100,"StreamDBEN::DecomposePacket: Deleting record from memory...\n");
	if (rec_p != NULL) delete rec_p;
	debug(100,"StreamDBEN::DecomposePacket: Exiting...\n");
	return(ret);
}


void StreamDBEN::Ashtech_DBN_phs_rng (int index, unsigned char *rec, unsigned short *bit, unsigned char phs_index, unsigned char rng_index)
{
	/*
         * note: still might be able to add testing for bad phase values using
	 * the warning bit, and might be able to use that to set BIT 0 of .lli
	 * and appropriate bit of .err in the phase structure
	 */

// set pseudorange and associated values
	double pseudorange = (double)(long int)extract_int(rec, 0x00,  94 + *bit, 31) * c_WGS84/1e10;
	rtcm.msg_data.obs_data[rng_index].obs[index].value = pseudorange;

// set phase and associated values
	double phase =	   (extract_int(rec, 0x00, 126 + *bit, 1) ? -1 : 1) *
		   ((double)extract_int(rec, 0x00, 127 + *bit, 28)
		  + (double)extract_int(rec, 0x00, 155 + *bit, 11) * 5e-4);
	rtcm.msg_data.obs_data[phs_index].obs[index].value = phase;

	(*bit)+= 72;
}

#endif

#if 0
typedef enum {
	GROUND_STATE,
	PASH_DOLLAR,
	PASH_P_ASH,
	PASH_PA_SH,
	PASH_PAS_H,
	PASH_PASH,
	PASH_RESPONSE,
	PASH_COMMA1,
	DBEN_SEND_ID1,
	DBEN_SEND_ID2,
	DBEN_REC_ID1,
	DBEN_REC_ID2,
	DBEN_COMPRESS,
	DBEN_COMMA1,
	DBEN_PACKET_TYPE1,
	DBEN_PACKET_TYPE2,
	DBEN_PACKET_TYPE3,
	DBEN_COMMA2,
	DBEN_LENGTH,
	DBEN_PAYLOAD,
	DBEN_PAYLOAD_END,
	DBEN_EOL1,
	MBEN_PACKET_TYPE1,
	MBEN_PACKET_TYPE2,
	MBEN_PACKET_TYPE3,
	MBEN_COMMA2,
	MBEN_ASCII,
	MBEN_BINARY,
	MBEN_PAYLOAD_END,
	MBEN_EOL1,
	PBEN_PACKET_TYPE1,
	PBEN_PACKET_TYPE2,
	PBEN_PACKET_TYPE3,
	PBEN_COMMA2,
	PBEN_PAYLOAD,
	PBEN_PAYLOAD_END,
	PBEN_EOL1,
	DBEN_RECOGNIZED,
	MBEN_RECOGNIZED,
	PBEN_RECOGNIZED,
};
#endif

#define MBEN_BINMPC_LENGTH 97


void StreamDBEN::nextstate(int *packet_state, unsigned char c)
{
	static int packet_length = 0;
#ifndef CRCCHECK
#else
	static unsigned short chktst = 0;
#endif

	switch(*packet_state)
	{
		case GROUND_STATE:
			packet_length = 0;
			if (c == '$')
				*packet_state = PASH_DOLLAR;
			break;
// ASHTECH HEADER
		case PASH_DOLLAR:
			if (c == 'P')
				*packet_state = PASH_P_ASH;
			else if (c =='B')	/* DBEN Sender ID */
				*packet_state = DBEN_SEND_ID1;
			else
				*packet_state = GROUND_STATE;
			break;
		case PASH_P_ASH:
			if (c == 'A') {
				/* This is possibly a $PASHR, ashtech DBEN,PBEN,MBEN record */
				*packet_state = PASH_PA_SH;
			}
			else if (!isprint(c))
				*packet_state = GROUND_STATE;
			break;
		case PASH_PA_SH:
			if (c == 'S')
				*packet_state = PASH_PAS_H;
			else
				*packet_state = GROUND_STATE;
			break;
		case PASH_PAS_H:
			if (c == 'H')
				*packet_state = PASH_PASH;
			else
				*packet_state = GROUND_STATE;
			break;
		case PASH_PASH:
			if (c == 'R')
				*packet_state = PASH_RESPONSE;
			else
				*packet_state = GROUND_STATE;
			break;
		case PASH_RESPONSE:
			if (c == ',')
				*packet_state = PASH_COMMA1;
			else
				*packet_state = GROUND_STATE;
			break;
		case PASH_COMMA1:
			if (c == 'R')	/* DBEN RPC Record */
				*packet_state = DBEN_PACKET_TYPE1;
			else if (c == 'M')	/* MBEN Record */
				*packet_state = MBEN_PACKET_TYPE1;
			else if (c == 'P')	/* PBEN Record */
				*packet_state = PBEN_PACKET_TYPE1;
			else
				*packet_state = GROUND_STATE;
			break;
		case DBEN_SEND_ID1:
			if (c == 'S')
				*packet_state = DBEN_SEND_ID2;
			else
				*packet_state = GROUND_STATE;
			break;
		case DBEN_SEND_ID2:
			if (c == '?')
				*packet_state = DBEN_REC_ID1;
			else
				*packet_state = GROUND_STATE;
			break;
		case DBEN_REC_ID1:
			if (c == '?')
				*packet_state = DBEN_REC_ID2;
			else
				*packet_state = GROUND_STATE;
			break;
		case DBEN_REC_ID2:
			if (c == 'R')
				*packet_state = DBEN_COMPRESS;
			else
				*packet_state = GROUND_STATE;
			break;
		case DBEN_COMPRESS:
			if (c == ',')
				*packet_state = DBEN_COMMA1;
			else
				*packet_state = GROUND_STATE;
			break;
		case DBEN_COMMA1:
			if (c == 'R')
				*packet_state = DBEN_PACKET_TYPE1;
			else
				*packet_state = GROUND_STATE;
			break;
		case DBEN_PACKET_TYPE1:
			if (c == 'P')
				*packet_state = DBEN_PACKET_TYPE2;
			else
				*packet_state = GROUND_STATE;
			break;
		case DBEN_PACKET_TYPE2:
			if (c == 'C')
				*packet_state = DBEN_PACKET_TYPE3;
			else
				*packet_state = GROUND_STATE;
			break;
		case DBEN_PACKET_TYPE3:
			if (c == ',')
				*packet_state = DBEN_COMMA2;
			else
				*packet_state = GROUND_STATE;
			break;
		case DBEN_COMMA2:
			packet_length = 0;
			packet_length |= (uint) c<<8;
			*packet_state = DBEN_LENGTH;
			break;
		case DBEN_LENGTH:
			packet_length |= (uint) c;
			*packet_state = DBEN_PAYLOAD;
#ifndef CRCCHECK
			packet_length += 2; // add 2 for CRC values
#endif
			break;
#ifndef CRCCHECK
		case DBEN_PAYLOAD:
			if (--packet_length == 0)
				*packet_state = DBEN_PAYLOAD_END;
			break;
#else
		case DBEN_PAYLOAD:
			if (--packet_length == 0) {
				*packet_state = DBEN_CRC_CHECK;
				chktst = 0;
				chktst |= (uint) c<<8;
			} else {
				chksum += (uint) c;
			}
				
			break;
		case DBEN_CRC_CHECK:
			chktst |= (uint) c;
			if (chksum != chktst)
				*packet_state = GROUND_STATE;
			else 
				*packet_state = DBEN_PAYLOAD_END;
			break;
#endif
		case DBEN_PAYLOAD_END:
			if (c == '\r')
				*packet_state = DBEN_EOL1;
			else
				*packet_state = GROUND_STATE;
			break;
		case DBEN_EOL1:
			if (c == '\n')
				*packet_state = DBEN_RECOGNIZED;
			else
				*packet_state = GROUND_STATE;
			break;
//MBEN STUFF
		case MBEN_PACKET_TYPE1:
			if (c == 'P')
				*packet_state = MBEN_PACKET_TYPE2;
			else
				*packet_state = GROUND_STATE;
			break;
		case MBEN_PACKET_TYPE2:
			if (c == 'C')
				*packet_state = MBEN_PACKET_TYPE3;
			else
				*packet_state = GROUND_STATE;
			break;
		case MBEN_PACKET_TYPE3:
			if (c == ',')
				*packet_state = MBEN_COMMA2;
			else
				*packet_state = GROUND_STATE;
			break;
		case MBEN_COMMA2:
			// Can't decode MBEN yet
			packet_length = MBEN_BINMPC_LENGTH;
			*packet_state = MBEN_BINARY;
			packet_length--;
			packet_length--;
			packet_length--;
			break;
		case MBEN_ASCII:
			if (!isprint(c)) 
				*packet_state = MBEN_BINARY;
			if (c == '\r') 
				*packet_state = MBEN_EOL1;
			packet_length--;
			break;
		case MBEN_BINARY:
			if (--packet_length == 0)
				*packet_state = MBEN_PAYLOAD_END;
			break;
		case MBEN_PAYLOAD_END:
			if (c == '\r')
				*packet_state = MBEN_EOL1;
			else
				*packet_state = GROUND_STATE;
			break;
		case MBEN_EOL1:
			if (c == '\n')
				*packet_state = MBEN_RECOGNIZED;
			else
				*packet_state = GROUND_STATE;
			break;

// PBEN Stuff
		case PBEN_PACKET_TYPE1:
			if (c == 'B')
				*packet_state = PBEN_PACKET_TYPE2;
			else
				*packet_state = GROUND_STATE;
			break;
		case PBEN_PACKET_TYPE2:
			if (c == 'N')
				*packet_state = PBEN_PACKET_TYPE3;
			else
				*packet_state = GROUND_STATE;
			break;
		case PBEN_PACKET_TYPE3:
			if (c == ',')
				*packet_state = PBEN_COMMA2;
			else
				*packet_state = GROUND_STATE;
			break;
		case PBEN_COMMA2:
			packet_length = 56; /* 56 data, 11 header, 3 read */
			*packet_state = PBEN_PAYLOAD;
		case PBEN_PAYLOAD:
			if (--packet_length == 0)
				*packet_state = PBEN_PAYLOAD_END;
			break;
		case PBEN_PAYLOAD_END:
			if (c == '\r')
				*packet_state = PBEN_EOL1;
			else
				*packet_state = GROUND_STATE;
			break;
		case PBEN_EOL1:
			if (c == '\n')
				*packet_state = PBEN_RECOGNIZED;
			else
				*packet_state = GROUND_STATE;
			break;
	}
}

int StreamDBEN::ProcessInput(int fd, time_t timeout)
{
	debug(100,"StreamDBEN::ProcessInput: Entering...\n");
	static int packet_state = GROUND_STATE;
	if (inbuf_pos >= inbuffer.size()) ReadInput(fd, timeout);
//	for (size_t i = 0; i < newdata; i++ ) {
//		inbuffer.push_back(newbuf[i]);
//	}
	while (inbuf_pos < inbuffer.size()) {
		nextstate(&packet_state, inbuffer[inbuf_pos++]);

		if (packet_state == GROUND_STATE) {
			inbuffer.erase(inbuffer.begin(), inbuffer.begin()+1);
			inbuf_pos = 0;
		} else {
#ifdef RTCM_ENABLE
			translateRTCM = TRUE;
#endif
			switch(packet_state) {
				case PBEN_RECOGNIZED:
#ifdef RTCM_ENABLE
					translateRTCM = FALSE;
#endif
					gpsMinute = -1;
					gpsFrac = -1;
				case DBEN_RECOGNIZED:
					debug(100, "StreamDBEN::ProcessInput: Found packet\n");
					for (size_t i = 0; i < inbuf_pos; i++ ) {
						cur_packet.push_back(inbuffer[i]);
					}
					inbuffer.erase(inbuffer.begin(), inbuffer.begin()+inbuf_pos);
					inbuf_pos = 0;
					packet_type = packet_state;
					packet_state = GROUND_STATE;
	debug(100,"StreamDBEN::ProcessInput: inbuffer is now %d chars long...\n",inbuffer.size());
					return(PACKET_FOUND);
					break;
			}
		}
	}
	debug(100,"StreamDBEN::ProcessInput: inbuffer is now %d chars long...\n",inbuffer.size());
	debug(100,"StreamDBEN::ProcessInput: Exiting...\n");
	return (packet_state);
}

#if 0
#define MAXDBENRECORDS 2
#define MAXCHANNELS 14
#pragma pack(1)

struct dben_header_msg_t {
    char    open;
    char    sender[2];
    char    receiver[2];
    char    comma1;
    char    type[3];
    char    comma2;
    char    size_h;
    char    size_l;
} dben_header_msg;

struct dben_msg_t {
    uint    receive_time:30;
    char    sitecode[4];
    uint    SVs[32];
    struct messages {
        uint    pseudo:31;
        uint    warn:1;
        uint    sign:1;
        uint    phase_int:28;
        uint    phase_fract:11;
    } PRNData[MAXDBENRECORDS * MAXCHANNELS];
} dben_msg;
#endif
#if 0
#define L1 0
#define C1 1
#define L2 2
#define P1 3
#define P2 4
#endif

#endif
