/* 
#    $Id: StreamDBEN.h,v 1.2 2017/09/07 17:40:35 gps Exp $
#
#    StreamDBEN.h -- Include file for the Ashtech Receiver
#
#    Copyright (C) 2000 US Geological Survey
#    Written by Keith F. Stark
#    
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifndef _STREAMDBEN_H
#define _STREAMDBEN_H

#if USE_STREAM

#include <fstream>
#include <string>

#include "Ashtech.h"
#include "AshtechDefs.h"
#include "Receiver.h"
#include "gpstime.h"
#include "gpscommon.h"
#include "StreamType.h"

#define FILELENGTH 12
#define ASHBYTEORDER __BIG_ENDIAN

#define DEBUG 0
#define ASHTECH_DBN_RCA (int)'A'
#define ASHTECH_DBN_RP1 (int)'1'
#define ASHTECH_DBN_RP2 (int)'2'
#define ASHTECH_DBN_RPC 0
#define c_WGS84 299792458.0
#define SAT_INFO int

enum {
        GROUND_STATE,
        PASH_DOLLAR,
        PASH_P_ASH,
        PASH_PA_SH,
        PASH_PAS_H,
        PASH_PASH,
        PASH_RESPONSE,
        PASH_COMMA1,
        DBEN_SEND_ID1,
        DBEN_SEND_ID2,
        DBEN_REC_ID1,
        DBEN_REC_ID2,
        DBEN_COMPRESS,
        DBEN_COMMA1,
        DBEN_PACKET_TYPE1,
        DBEN_PACKET_TYPE2,
        DBEN_PACKET_TYPE3,
        DBEN_COMMA2,
        DBEN_LENGTH,
        DBEN_PAYLOAD,
        DBEN_PAYLOAD_END,
#ifdef CRCCHECK
	DBEN_CRC_CHECK,
#endif
        DBEN_EOL1,
        MBEN_PACKET_TYPE1,
        MBEN_PACKET_TYPE2,
        MBEN_PACKET_TYPE3,
        MBEN_COMMA2,
        MBEN_ASCII,
        MBEN_BINARY,
        MBEN_PAYLOAD_END,
        MBEN_EOL1,
        PBEN_PACKET_TYPE1,
        PBEN_PACKET_TYPE2,
        PBEN_PACKET_TYPE3,
        PBEN_COMMA2,
        PBEN_PAYLOAD,
        PBEN_PAYLOAD_END,
        PBEN_EOL1,
        DBEN_RECOGNIZED,
        MBEN_RECOGNIZED,
        PBEN_RECOGNIZED,
};




class StreamDBEN : public StreamType {
public:
	StreamDBEN() { };
	~StreamDBEN() { };

	int ProcessInput(int fd, time_t timeout);
	int DecomposePacket ();
	int GetRecordTime(long *gpsMinutes, unsigned short *gpsm_Seconds);
	long long int GetRecordInfo(int INFOTYPE);

	void nextstate(int *packet_state, unsigned char c);
	unsigned long extract_int (unsigned char *rec, unsigned char s, unsigned long offset, unsigned long len);
	void Ashtech_DBN_phs_rng (int index, unsigned char *rec, unsigned short *bit, unsigned char phs_index, unsigned char rng_index);            


private:

};

#endif

#endif
