/* 
#    $Id: StreamType.cpp,v 1.4 2012/02/10 00:48:57 gps Exp $
#
#    Copyright (C) 2000 US Geological Survey
#    Written by Keith F. Stark
#    
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* system includes */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <string>
#include <iomanip>
#include <math.h>
#include <signal.h>
#include <errno.h>

#include "gpscommon.h"
#include "sharcrt.h"
#include "StreamType.h"

#pragma pack(1)

#if USE_STREAM

int StreamRaw::ProcessInput(int fd, time_t timeout)
{
        debug(100,"StreamType::ProcessInput: Entering...\n");
	size_t newdata = ReadInput(fd, timeout);

        int status = PACKET_FOUND;
	if (inbuffer.size() != newdata) {
		debug(10,"StreamType::ProcessInput: Mismatch in data read for some reason\n");
		newdata = inbuffer.size();
	}
        for (size_t i = 0; i < newdata; i++ ) {
                cur_packet.push_back(inbuffer[i]);
        }
	inbuffer.clear();
        debug(100,"StreamType::ProcessInput: Exiting (%d)...\n",status);
        return (status);
}

int StreamRaw::GetRecordTime(long *gpsMinutes, unsigned short *gpsm_Seconds)
{

	int week; double sec;
	gpstime computer_time;
	computer_time.set_time();
	computer_time.getgps(&week,&sec);
	*gpsMinutes = (week * 7 * 1440) + ((int)sec / 60);
	*gpsm_Seconds = ((int)sec*1000) % (60*1000);
	return(0);
}

int StreamRaw::DecomposePacket()
{

	return(0);
}

long long int StreamRaw::GetRecordInfo(int INFOTYPE)
{
	return(-1000000);
}


StreamType::StreamType() {
	inbuf_pos = 0;
	head_len = 0;
	body_len = 0;
	other_len = 0;
#ifdef RTCM_ENABLE
	rtcm_info_count = 9999;
#endif
	crc = 0;
}

int StreamType::GetHeader(unsigned char *out, unsigned int max)
{
        return(0);
}

int StreamType::GetBody(unsigned char *out, unsigned int max)
{
        return(0);
}

int StreamType::GetRecord(vector<unsigned char> &out)
{
	debug(D_SERINFO,"StreamType::GetRecord: copying vector\n");
	for (size_t i = 0; i < cur_packet.size(); i++) {
		out.push_back(cur_packet[i]);
	}
	debug(D_SERINFO,"StreamType::GetRecord: done copying vector\n");
	return((int) out.size());
}

#define READMAX 1024
size_t StreamType::ReadInput(int fd, time_t timeout)
{
	size_t len = 0;
	int timer;
	int rc;
	unsigned char ctmp;

	while (len == 0) {
		timer = timeout;
		while(len < READMAX)
		{
			timer = isCharAvail(fd, timer);
			if (timer == 0) break;
	
			rc = read(fd, &ctmp, 1);
			if (rc == -1) break;

			inbuffer.push_back(ctmp);
			len++;
		}
	}
	return(len);
}


#ifdef RTCM_ENABLE
int StreamType::MakeRTCMInfoRecords()
{

	enum {rtcm_3, rtcm_22, rtcm_23, rtcm_24, rtcm_1005, rtcm_1006, rtcm_1007, rtcm_1008, rtcm_MAX};
	struct rtcm_t rtcm_info[8]; // space for all record types

	char buf[BUFSIZ];
	unsigned char buf2[BUFSIZ];
	int i;
	bool rtcm22 = FALSE;
	int offset = 0;
	char ch;
	isgps30bits_t isgpsbuf[RTCM_WORDS_MAX];
	unsigned char ucbuf[1024];
	debug(D_FLOW,"StreamType::MakeRTCMInfoRecords: Entering...\n");

	if (rtcm_info_count < rtcm_antfreq) {
		debug(D_FLOW,"StreamType::MakeRTCMInfoRecords: Exiting (rtcm_info_count = %d, rtcm_antfreq = %d)\n",rtcm_info_count, rtcm_antfreq);
		rtcm_info_count++;
		return(0);
	}
	rtcm_info_count = 0;

// line 1 = version 2.2 or 2.3
	debug(D_FLOW,"StreamType::MakeRTCMInfoRecords: Opening %s\n",rtcm_antfile.c_str());

	FILE *in = fopen(rtcm_antfile.c_str(),"r");
	if (in == NULL) {
		debug(D_WARN,"StreamType::MakeRTCMInfoRecords: Unable to open %s: %s\n",rtcm_antfile.c_str(), strerror(errno));
		return(-1);
	}
	fgets(buf, BUFSIZ, in);
        int fldcount = sscanf(buf, "%c\t%u\t%u\n",&ch,&rtcm_refstaid,&rtcm_antfreq);
	switch (fldcount) {
		case 1:
			rtcm_refstaid = 0;
		case 2:
			rtcm_antfreq = RTCM_ANTFREQ;
		case 3:
			if (ch == '2') rtcm22 = TRUE;
		default:
			break;
	}
	debug(D_VALUES, "StreamType::MakeRTCMInfoRecords: read config line(fldcount = %d): RTCM v2.%c, rtcm_refstaid=%d, frequency=%d\n",fldcount,ch,rtcm_refstaid,rtcm_antfreq);

	int haverec = 0;
	rtcm_t tmp;
	memset(&tmp, 0, sizeof(rtcm_t));
	while (fgets(buf, BUFSIZ, in)) {
		i = rtcm_undump(&tmp, buf);
//		if (i < 0) break;
		if (i == 0) {
			unsigned short s = rtcm_MAX;
			if (tmp.type == 3) s = rtcm_3;
			if (tmp.type == 22) s = rtcm_22;
			if (tmp.type == 23) s = rtcm_23;
			if (tmp.type == 24) s = rtcm_24;
			if (tmp.type == 1005) s = rtcm_1005;
			if (tmp.type == 1006) s = rtcm_1006;
			if (tmp.type == 1007) s = rtcm_1007;
			if (tmp.type == 1008) s = rtcm_1008;
			haverec |= (1 << s);
			memcpy(&rtcm_info[s], &tmp, sizeof(rtcm_t));
			memset(&tmp, 0, sizeof(rtcm_t));
			rtcm_info[s].refstaid = rtcm_refstaid;
		}
	}
	fclose(in);

// RTCM 2.X handling

	debug(D_FLOW,"StreamType::MakeRTCMInfoRecords: Creating rtcmoutput\n");
	offset = 0;
	if (rtcm22) {
		if (haverec & (1 << rtcm_3)) {
			rtcm_repack(&rtcm_info[rtcm_3],isgpsbuf);
			rtcm_buffer_magnavox(isgpsbuf, buf2, &offset);
			debug(D_VALUES,"StreamType::MakeRTCMInfoRecords: packed rtcm_3 with %d characters\n",offset);
		}
		if (haverec & (1 << rtcm_22)) {
			rtcm_repack(&rtcm_info[rtcm_22],isgpsbuf);
			rtcm_buffer_magnavox(isgpsbuf, buf2, &offset);
			debug(D_VALUES,"StreamType::MakeRTCMInfoRecords: packed rtcm_22 with %d characters\n",offset);
		}
	} else {
		if (haverec & (1 << rtcm_23)) {
			rtcm_repack(&rtcm_info[rtcm_23],isgpsbuf);
			rtcm_buffer_magnavox(isgpsbuf, buf2, &offset);
			debug(D_VALUES,"StreamType::MakeRTCMInfoRecords: packed rtcm_23 with %d characters\n",offset);
		}
		if (haverec & (1 << rtcm_24)) {
			rtcm_repack(&rtcm_info[rtcm_24],isgpsbuf);
			rtcm_buffer_magnavox(isgpsbuf, buf2, &offset);
			debug(D_VALUES,"StreamType::MakeRTCMInfoRecords: packed rtcm_24 with %d characters\n",offset);
		}
	}

	debug(D_VALUES,"StreamType::MakeRTCMInfoRecords: Copying %d characters into RTCM2 Packet\n",offset);
	for (int i = 0; i < offset; i++) {
		rtcm_packet.push_back(buf2[i]);
	}

// Handle RTCM 3 packets

	if (haverec & (1 << rtcm_1006)) {
		offset = rtcm3_repack(&rtcm_info[rtcm_1006], ucbuf);
		debug(D_VALUES,"StreamType::MakeRTCMInfoRecords: packed rtcm_1006 with %d characters\n",offset);
	} else if ( haverec & (1 << rtcm_1005)) {
		offset = rtcm3_repack(&rtcm_info[rtcm_1005], ucbuf);
		debug(D_VALUES,"StreamType::MakeRTCMInfoRecords: packed rtcm_1005 with %d characters\n",offset);
	}
	debug(D_VALUES,"StreamType::MakeRTCMInfoRecords: Copying %d characters into RTCM3 Packet\n",offset);
	for (int i = 0; i < offset; i++) {
//		debug(1,"byte %d = %x\n",i,ucbuf[i]);
		rtcm3_packet.push_back(ucbuf[i]);
	}

	if (haverec & (1 << rtcm_1008)) {
		offset = rtcm3_repack(&rtcm_info[rtcm_1008], ucbuf);
		debug(D_VALUES,"StreamType::MakeRTCMInfoRecords: packed rtcm_1008 with %d characters\n",offset);
	} else if ( haverec & (1 << rtcm_1007)) {
		offset = rtcm3_repack(&rtcm_info[rtcm_1007], ucbuf);
		debug(D_VALUES,"StreamType::MakeRTCMInfoRecords: packed rtcm_1007 with %d characters\n",offset);
	}
	debug(D_VALUES,"StreamType::MakeRTCMInfoRecords: Copying %d characters into RTCM3 Packet\n",offset);
	for (int i = 0; i < offset; i++) {
//		debug(1,"byte %d = %x\n",i,ucbuf[i]);
		rtcm3_packet.push_back(ucbuf[i]);
	}

	debug(D_FLOW,"StreamType::MakeRTCMInfoRecords: Exiting with station ID %d\n",rtcm_refstaid);

	return(rtcm_refstaid);
}

void StreamType::InitRTCM(bool trans, time_t freq, string antfile)
{
	translateRTCM = trans;
	rtcm_antfreq = freq;
	rtcm_antfile = antfile;
	rtcm_refstaid = 0;
}

void StreamType::InitRTCMStruct()
{
	unsigned long long receive_time;
	unsigned int gnss;
	unsigned long tow;
	double zcount;
	
	debug(D_FLOW,"InitRTCMStruct: Entering...\n");

	memset(&rtcm, 0 , sizeof(rtcm_t));

//get hour portion of receive time
	debug(D_VALUES,"InitRTCMStruct: Minute = %d, Frac = %d\n",gpsMinute, gpsFrac);
        receive_time = (unsigned long long) gpsMinute;
        receive_time *= 60ULL * 1000ULL;
	receive_time += gpsFrac;
	tow = receive_time % (86400 * 7 * 1000);
	debug(D_VALUES,"InitRTCMStruct: receive_time = %lld, tow = %ld\n", receive_time, tow);
	receive_time = receive_time % (3600 * 1000);

// divide receive_time into zcount and gnss time
	gnss = receive_time % 600;
	zcount = ((double)receive_time - (double)gnss)/1e3;
	gnss *= 1000;

	debug(D_VALUES,"InitRTCMStruct: receive_time = %f, gnss = %d, zcount = %f\n", receive_time, gnss, zcount);

// set basic header information on all of the types
	rtcm.length = 0;
	rtcm.refstaid = rtcm_refstaid;
	rtcm.stathlth = 0;
	rtcm.msg_data.gpstow = tow;
	rtcm.zcount = zcount;

	rtcm.msg_data.obs_data[RTCM_L1].frequency = 0;
	rtcm.msg_data.obs_data[RTCM_L2].frequency = 2;
	rtcm.msg_data.obs_data[RTCM_C1].frequency = 0;
	rtcm.msg_data.obs_data[RTCM_P1].frequency = 0;
	rtcm.msg_data.obs_data[RTCM_P2].frequency = 2;

	rtcm.msg_data.numobs = 0;
	rtcm.msg_data.gnsstime = gnss;
	for (int j = 0; j < RTCMMAXTYPES; j++) {
		rtcm.msg_data.obs_data[j].obs_avail = false;
	}

	debug(D_FLOW,"InitRTCMStruct: Exiting...\n");
}

void StreamType::SetSessionPRN(int index, int prn)
{
	debug(D_FLOW,"SetSessionPRN: Entering...\n");

	for (int j = 0; j < RTCMMAXTYPES; j++) {
		rtcm.msg_data.obs_data[j].obs[index].prn = prn;
		rtcm.msg_data.obs_data[j].obs[index].GPS = 0; 
		rtcm.msg_data.obs_data[j].obs[index].multiple = 1;
	}
	debug(D_FLOW,"SetSessionPRN: Exiting...\n");
}

int StreamType::GetRTCMRecords(vector<unsigned char> &rtcm_out, vector<unsigned char> &rtcm3_out)
{

	debug(D_FLOW,"StreamType::GetRTCMRecord: Entering...\n");

	if (! allowRTCM()) return(-1);

	debug(D_FLOW,"StreamType::GetRTCMRecord: Initializing RTCM Structure\n");
	InitRTCMStruct();

	debug(D_FLOW,"StreamType::GetRTCMRecord: Calling DecomposePacket\n");
	DecomposePacket();

	debug(D_FLOW,"StreamType::GetRTCMRecord: Making RTCM Antenna Info Structure\n");
	MakeRTCMInfoRecords();

	debug(D_FLOW,"StreamType::GetRTCMRecord: Making RTCM 2 Structure\n");
	MakeRTCMRecord();

	for (size_t i = 0; i < rtcm_packet.size(); i++) {
		rtcm_out.push_back(rtcm_packet[i]);
	}
	rtcm_packet.clear();

	debug(D_FLOW,"StreamType::GetRTCMRecord: Making RTCM 3 Structure\n");
	MakeRTCM3Record();

	for (size_t i = 0; i < rtcm3_packet.size(); i++) {
		rtcm3_out.push_back(rtcm3_packet[i]);
	}
	rtcm3_packet.clear();

	debug(D_FLOW,"StreamType::GetRTCMRecord: Exiting...\n");
        return(0);
}


int StreamType::MakeRTCMRecord()
{

	int i;
	static int seqnum = 0;
	unsigned char buf[BUFSIZ * 4];
	int offset2 = 0;

	isgps30bits_t isgpsbuf[RTCM_WORDS_MAX];
	
	debug(D_FLOW,"MakeRTCMRecord: Entering...\n");

	for (i = RTCMMAXTYPES - 1; i >= 0; i--) {
		if (rtcm.msg_data.obs_data[i].obs_avail) {
			debug(D_FLOW,"MakeRTCMRecord: setting multiple for %d to 0\n",i);
			rtcm.msg_data.obs_data[i].obs[rtcm.msg_data.numobs - 1].multiple = 0;
			break;
		}
	}

	for (i = 0; i< RTCMMAXTYPES; i++) {
		if (! rtcm.msg_data.obs_data[i].obs_avail)
			continue;
		rtcm.length = 1 + (rtcm.msg_data.numobs * 2);
		switch (i) {
			case RTCM_L1:
			case RTCM_L2:
				rtcm.msg_data.phase = &(rtcm.msg_data.obs_data[i]);
				rtcm.type = 18;
				break;
			case RTCM_C1:
			case RTCM_P1:
			case RTCM_P2:
				rtcm.msg_data.pseudo = &(rtcm.msg_data.obs_data[i]);
				rtcm.type = 19;
				break;
		}
		rtcm.seqnum = seqnum++;
		seqnum &= 0x7;
		debug(D_FLOW,"MakeRTCMRecord: packing rtcm for session[%d] of length %d\n",i,rtcm.length);
		(void)rtcm_repack(&rtcm,isgpsbuf);
		debug(D_FLOW,"MakeRTCMRecord: getting output from rtcm_buffer_magnavox\n");
		rtcm_buffer_magnavox(isgpsbuf, buf, &offset2);
/*
		char buf2[BUFSIZ];
		rtcm_dump(&rtcm, buf2, BUFSIZ);
		debug(110,"dump\n%s\n",buf2);
*/
	}

	for (int i = 0; i < offset2; i++ ) {
		rtcm_packet.push_back(buf[i]);
	}

	debug(D_VALUES,"output buffer: length = %d\n",offset2);

	debug(D_FLOW,"MakeRTCMRecord: Exiting...\n");
	return(offset2);
}

int StreamType::MakeRTCM3Record() {

        unsigned char buf[BUFSIZ * 4];
	unsigned char offset = 0;

	rtcm.type = 1004;
	offset = rtcm3_repack(&rtcm, buf);

/*
	struct rtcm_t rtcm_test;

	memset(&rtcm_test, '\0', sizeof(struct rtcm_t));
	char buf2[BUFSIZ * 4];
	rtcm_dump(&rtcm, buf2, BUFSIZ * 4);
	buf2[BUFSIZ * 4 - 5] = '\0';
	printf("dump\n%s\n",buf2);

	rtcm3_unpack(&rtcm_test, buf);
	rtcm_dump(&rtcm_test, buf2, BUFSIZ *4);
	buf2[BUFSIZ * 4 - 5] = '\0';
	printf("dump\n%s\n",buf2);
*/
	for (int i = 0; i < offset; i++ ) {
		rtcm3_packet.push_back(buf[i]);
	}
		
	return(rtcm3_packet.size());
}

#endif

#ifdef SOC1_ENABLE
int StreamType::MakeSOC1Record() {

return(1);
}

int StreamType::GetSOC1Record(vector<unsigned char> &out) {

return(1);
}
#endif
#endif


