/* 
#    $Id: StreamType.h,v 1.3 2018/02/27 03:10:35 gps Exp $
#
#    Copyright (C) 2000 US Geological Survey
#    Written by Keith F. Stark
#    
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _STREAMTYPE_H
#define _STREAMTYPE_H

#define STREAMTYPE_H

#if USE_STREAM

/* system includes */
#include <vector>

using namespace std;

#include "gpscommon.h"

#ifdef RTCM_ENABLE
extern "C" {
#include <gps.h>
extern int rtcm_undump(/*@out@*/struct rtcm_t *, char *);
extern int rtcm_dump(/*@out@*/struct rtcm_t *, char *, size_t);
extern bool rtcm_repack(struct rtcm_t *, isgps30bits_t *);
extern int rtcm3_repack(struct rtcm_t *, unsigned char *);
extern int rtcm3_unpack(struct rtcm_t *, unsigned char *);
extern void rtcm_buffer_magnavox(isgps30bits_t * ip, unsigned char *, int *);
}
#endif

// Basic class definition for reading streaming data
class StreamType {

public:
	StreamType();
	virtual ~StreamType() { };
	virtual int ProcessInput(int fd, time_t timeout) = 0;
	size_t ReadInput(int fd, time_t timeout);
	int GetHeader(unsigned char *out, unsigned int max);
	int GetBody(unsigned char *out, unsigned int max);
	int GetRecord(vector<unsigned char> &out);
	int GetHeaderLength() { return(head_len); };
	int GetBodyLength() { return(body_len); };
	int GetRecordLength() { return(cur_packet.size()); };
	void ResetPacket() { cur_packet.clear(); };
	virtual int GetRecordTime(long *gpsMinutes, unsigned short *gpsm_Seconds) = 0;
        virtual long long int GetRecordInfo(int INFOTYPE) = 0;


#ifdef RTCM_ENABLE
	bool allowRTCM() { return(translateRTCM); };
	virtual int DecomposePacket() = 0;
	void InitRTCM(bool trans, time_t freq, string antfile);
	void InitRTCMStruct();
	void SetSessionPRN(int index, int prn);

	int GetRTCMRecords(vector<unsigned char> &rtcm, vector<unsigned char> &rtcm3);
	int GetAntennaBuffer(unsigned char ** an_buf, size_t *an_buflen, time_t *an_frequency);

	int MakeRTCMRecord();
	int MakeRTCMInfoRecords();
	int MakeRTCM3Record();
	int MakeRTCM3InfoRecords();
	int cpstruct(rtcm_t *in, rtcm_t *out, uint type);

#endif
#ifdef SOC1_ENABLE
	int MakeSOC1Record();
	int GetSOC1Record(vector<unsigned char> &out);
#endif

protected:
	int head_len;
	int body_len;
	int other_len;;
	int crc;
        vector<unsigned char> inbuffer;
        vector<unsigned char> cur_packet;
        unsigned int inbuf_pos;
	int packet_type;
	unsigned int   gpsMinute;
	unsigned short gpsFrac;


#ifdef RTCM_ENABLE
	bool translateRTCM;
	unsigned int rtcm_antfreq;
	unsigned int rtcm_info_count;
	string rtcm_antfile;
	struct rtcm_t rtcm;
        vector<unsigned char> rtcm_packet;
        vector<unsigned char> rtcm3_packet;
	int rtcm_refstaid;

#endif
};

class StreamRaw : public StreamType {
public:
	int ProcessInput(int fd, time_t timeout);
	int DecomposePacket();
	int GetRecordTime(long *gpsMinutes, unsigned short *gpsm_Seconds);
	long long int GetRecordInfo(int INFOTYPE);

};


#endif

#endif
