/* 
#    $Id: TRogue.cpp,v 1.1 2011/04/04 17:29:34 gps Exp $
#
#	 TRogue.cpp -- This file will be the major interaction with
#                   an trogue receiver/file
#    Copyright (C) 2000 US Geological Survey
#    Written by Keith F. Stark
#    
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <iostream>
#include <iomanip>
#include <fstream>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <stdio.h>
#include <time.h>
#include <sys/time.h>
#include <sys/types.h>

#include "gpscommon.h"
#include "TRogue.h"

STRogueFATEntry TRogue::pFAT[MAX_RCVR_FILES];

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
TRogue::TRogue () {

	debug(50,"TRogue::TRogue: Entering...\n");
	pFATentry = NULL;

	usesFAT = FALSE;
	myIndex = -1;

	// Actual RFile image
	cimageFile = 0;
	imageFile = NULL;
	header = NULL;
	fatEntry = NULL;
	session = ' ';
	debug(50,"TRogue::TRogue: Exiting\n");

}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int TRogue::setImageHeader(string rxID) {

	debug(50,"TRogue::setImageHeader: Entering...\n");

	debug(50,"TRogue::setImageHeader: Exiting\n");

	return(SUCCESS);
}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int TRogue::setFatEntry(STRogueFATEntry myFAT) {

	debug(50,"TRogue::setFatEntry: Entering...\n");

	debug(50,"TRogue::setFatEntry: Exiting\n");

	return(SUCCESS);
}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int TRogue::setImageFile(unsigned char *myImage, int fileSize) {

	debug(50,"TRogue::setImageFile: Entering...\n");

	if (imageFile != NULL) {
		delete(imageFile);
	}

	imageFile = new unsigned char[fileSize];

	memcpy(imageFile, myImage, fileSize);
	cimageFile = fileSize;
	debug(50,"TRogue::setImageFile: Exiting\n");
	return(SUCCESS);

}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int TRogue::WriteFiles() {

	ofstream rfile;
	debug(50,"TRogue::WriteRFile: Entering...\n");

	if (OpenFile(&rfile, GetFileName(myIndex).c_str()) != 0) return(FAILURE);

	debug(20,"cimageFile = %d\n",cimageFile);
	rfile.write((const char *)imageFile, cimageFile);
	rfile.close();

	debug(50,"TRogue::WriteRFile: Exiting\n");
	return(SUCCESS);

}
//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int TRogue::DownloadFileAtIndex(Serial *pSerial, int index) {

	debug(50,"TRogue::DownloadFileAtIndex: Entering...\n");

	debug(50,"TRogue::DownloadFileAtIndex: Exiting\n");
	return(SUCCESS);
}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int TRogue::DownloadFileByTime(Serial *pSerial, gpstime *start, gpstime *stop) {
//	int i;
//	int numindex, indexes[MAX_RCVR_FILES];
	time_t t_start, t_used;
	int downLoadOK = FALSE;
	int index;
	string xdate;

	const char *prompt[1] = { "TurboRogue" };
	const char *offload[6] = {
		"NOW!",				// start offload
		"Go ahead with offload",	// start offload
		"No data found",		// quit successfully
		"Data Not",			// quit successfully
		"XMODEM: no data present",	// quit successfully
		"Searching for data"		// loop again
	};

	debug(50,"TRogue::DownloadFileByTime: Entering...\n");

// Set Date string to download

// Set offload port to current port
	pSerial->Write("\n");
	pSerial->Write("offload <~\n"); pSerial->WaitFor(2000,prompt,1);
// Skip setting info
	char buffer[100];
//	start->chartime("%%G/0/0:0:%%g",buffer,100);
	start->chartime("%%G/%w/%H:%M:%S",buffer,100);
	xdate = buffer;
//	stop->chartime("%%G/0/0:0:%%g",buffer,100);
	stop->chartime("%%G/%w/%H:%M:%S",buffer,100);
	xdate = xdate + "-" + buffer;
// Send command to get turbo binary
//	string offloadcmd = "offload -f " + xdate + " .tb\n";
	string offloadcmd = "offload -tf -h hit .tb <oxeu " + xdate + "\n";

	debug(20,"TRogue::DownloadFileByTime: Using command %s\n",offloadcmd.c_str());
	pSerial->Write((char *)offloadcmd.c_str());

	while (1) {
		int status = pSerial->WaitFor(120000,offload,6);
		if (status == -1) break;
		if (status == 1) continue;
		if (status == 0) {
			ReceiverFAT[numFATEntries].siteName = "____";
			ReceiverFAT[numFATEntries].fileSize = 0;
			ReceiverFAT[numFATEntries].fileStart = *start;
			ReceiverFAT[numFATEntries].fileStart = *stop;
			ReceiverFAT[numFATEntries].session = ' ';
			index = numFATEntries;
			FindSession(index);
			CreateFileName(index);
			numFATEntries++;
			debug(5,"XMODEM: starting\n");
			t_start = time(NULL);
			long fileSize = 0;
			pSerial->ClearBuffers();
			unsigned char *myfile = pSerial->xmodemreceive(&fileSize, 1800, "");
			t_used = time(NULL) - t_start;
			debug(25,"TRogue::DownloadFileByTime: Returned From Xmodem Transfer, fileSize = %d, Time Used = %d\n", fileSize,t_used);

			if ((myfile != NULL) && (fileSize >= 0)) {
//				addToSumFile(" Index: %d\n",index);
				debug(25,"TRogue::DownloadFileAtIndex: Download OK, Copying image to TRogue Class\n");
				setImageFile(myfile, (int) fileSize);
				myIndex = index;
				downLoadOK = TRUE;
				char buffer[100];
				addToSumFile(" Start Time: %s\n",ReceiverFAT[index].fileStart.chartime("%%G %%g", buffer, 100));
				addToSumFile(" End Time: %s\n",ReceiverFAT[index].fileEnd.chartime("%%G %%g", buffer, 100));
				addToSumFile(" File: %s\n",GetFileName(myIndex).c_str());
				addToSumFile(" Bytes: %d\n",fileSize);
				addToSumFile(" Time: %d seconds\n",t_used);
				delete [] myfile;
			} else {
				downLoadOK = FALSE;
				debug(1, "TRogue::DownloadFileAtIndex: Download Failed\n");
				debug(1, "TRogue::DownloadFileAtIndex: Clearing XMODEM Transfer\n");
				pSerial->ClearBuffers(); sleep(1); pSerial->ClearBuffers();
			}
			debug(5,"XMODEM: ");
			break;
		} else if ((status == 3) || (status == 4)) {
			debug(1,"TRogue::DownloadFileByTime: NO DATA\n");
	//            &io'screen("\nXMODEM: NO DATA\n");
	//            &io'zwaitc("\n") && &io'when(30,"TurboRogue>");
	//            $xstat = 1;
			break;
		} else if (status == 5) {
			debug(1,"TRogue::DownloadFileByTime: Searching for Data\n");
		}
	}
	debug(50,"DownloadFileByTime: Exiting\n");
	return(downLoadOK);
}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int TRogue::DeleteFileAtIndex(Serial *pSerial, int index) {

	debug(50,"DeleteFileAtIndex: Entering...\n");

	debug(50,"DeleteFileAtIndex: Exiting\n");
	return(SUCCESS);
}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int TRogue::DeleteFileByTime(Serial *pSerial, gpstime *start, gpstime *stop) {
	int i,j;
	int numindex, indexes[MAX_RCVR_FILES];
	int deleteOK = FALSE;

	debug(50,"TRogue::DeleteFileByTime: Entering...\n");

	numindex = FindIndexesByTime(start, stop, indexes, TRUE);

	for (j = (MAX_RCVR_FILES-1); j >= 0; j--) {
		for (i = 0; i < numindex; i++) {
			if (indexes[i] != j) continue;
			debug(25,"TRogue::DeleteFileByTime: Deleteing file at index %d\n", indexes[i]);
			deleteOK = DeleteFileAtIndex(pSerial, indexes[i]);
		}
	}
	debug(50,"DeleteFiles: Exiting\n");
	return(deleteOK);
}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
string TRogue::DownloadReceiverID(Serial *pSerial) {

	char buffer[100];
	int ret;
	unsigned tmp = 0;

	debug(50,"TRogue::DownloadReceiverID: Entering...\n");
	if (! pSerial->isConnected()) {
		debug(1,"Not connected to the receiver\n");;
		return("");
	}
	debug(1,"Querying Receiver ID:\n");

	pSerial->Write("recid\n");
	while ((ret = pSerial->Readline(buffer, 100, 1000)) != 0) {
		RxID += buffer;
	}
	if (RxID == "") {
		debug(1,"Unable to read Receiver ID\n");
		return(RxID);
	}

//	if ((tmp = RxID.find("RID")) == string::npos) {
//		debug(10,"Incorrect return string\n");
//		RxID = "";
//		return(RxID);
//	}
	addToSumFile(" Receiver ID: %s\n",RxID.c_str());
	RxID.erase(0, RxID.find(",", tmp)+1);
	debug(1,"Found: %s\n",RxID.c_str());
	debug(50,"TRogue::DownloadReceiverID: Exiting\n");
	return(RxID);
}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int TRogue::SetReceiverSession(Serial *pSerial) {

	debug(50,"SetReceiverSession: Entering...\n");

	debug(50,"SetReceiverSession: Exiting...\n");
	return(SUCCESS);
}


//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
#define MemInfoSize 10
#define FileInfoSize 40
#define MaxFileNumbers 100
int TRogue::DownloadReceiverFAT(Serial *pSerial) {

	debug(50,"TRogue::DownloadReceiverFAT: Entering...\n");

	debug(50,"TRogue::DownloadReceiverFAT: Exiting\n");
	return(SUCCESS);

}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int TRogue::Logout(Serial *pSerial) {

	debug(50,"TRogue::Logout: Entering...\n");
	pSerial->Write("\n\n");

	pSerial->ClearBuffers();
	pSerial->Write("logout\n");
	const char *mydefault[1] = { "default" };
	int status = pSerial->WaitFor(30000,mydefault,1);
	if (status == 0) pSerial->Write("y\n");
	const char *logout[3] = {"User Logout", "\\+\\+\\+", "ath" };
	pSerial->WaitFor(30000,logout,3);
	if (status >= 0 && status <= 2) {
		debug(1,"TRouge::Logout: Successfully logged out\n");
	} else {
		debug(1,"TRouge::Logout: Problem Logging out\n");
	}

	debug(50,"TRogue::Logout: Exiting...\n");
	return(SUCCESS);

}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int TRogue::Login(Serial *pSerial, int direct, string login, string password) {

	int start, now;
	int _login = FALSE;
	int k = 0;
	int j = 0;
	int answer;

	const char *pattern[7] = {
		"Login:",
		"Password:",
		"Sorry",
		"Username",
		"Welcome",
		"F10",
		"TurboRogue>"
	};

	debug(50,"TRogue::Login: Entering...\n");
	pSerial->Write("\n\n");

	start = time(NULL);
	while (!_login && (k < 10)) {
		//pSerial->Write("");
		now = time(NULL);
		if ((now - start) > 30) {
			debug(5,"TRogue::Login: No prompt found\n");
			//pSerial->Write("\cX\r");
		} else if ((answer = pSerial->WaitFor(5000, pattern, 7)) != -1) {
			pSerial->ClearBuffers();
			switch (answer) {
				case 0:
					debug(5,"TRogue::Login: Sending Username\n");
					pSerial->Write("tom\n");
					j++;
					if (j <= 3) k = 0;
					break;
				case 1:
					debug(5,"TRogue::Login: Sending Password\n");
					pSerial->Write("warumono\n");
					pSerial->Write("\n");
					break;
				case 2:
					debug(5,"TRogue::Login: bad password\n");
					sleep(5);
					pSerial->Write("\n");
					break;
				case 3:
					debug(5,"TRogue::Login: bad user name\n");
					sleep(5);
					pSerial->Write("\n");
					break;
				case 4:
				case 5:
					_login = TRUE;
					debug(5,"TRogue::Login: logged in\n");
					sleep(5);
					pSerial->Write("\n");
					break;
				case 6:
					debug(5,"TRogue::Login: already logged in\n");
					_login = TRUE;
					break;
			}
		} else if (k > 6) {
			debug(5,"TRogue::Login: %d _logins & %d other tries...\n", j, k);
			//pSerial->Write("\cX\r");
		}
		k++;
	}
	return(!_login);
}

