/* 
#    $Id: TRogue.h,v 1.1 2011/04/04 17:29:34 gps Exp $
#
#    TRogue.h -- Include file for the TurboRogue Receiver
#
#    Copyright (C) 2000 US Geological Survey
#    Written by Keith F. Stark
#    
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifndef _TROGUE_H
#define _TROGUE_H

#include <gpstime.h>
#include <string>
#include <gpscommon.h>
#include <TRogueDefs.h>
#include <Receiver.h>

#define FILELENGTH 12

class TRogue : public Receiver {
public:
	TRogue();

	static string DownloadReceiverID(Serial *pSerial);
	static int DownloadReceiverFAT(Serial *pSerial);
	static int GetFATEntryAtIndex(SFATEntry *pFAT, int index);
	static int GetReceiverID();
	static int SetReceiverSession(Serial *pSerial);

	VIRTUAL int DownloadFileByTime(Serial *pSerial, gpstime *start, gpstime *stop);
	VIRTUAL int DownloadFileAtIndex(Serial *pSerial, int index);
	VIRTUAL int DeleteFileByTime(Serial *pSerial, gpstime *start, gpstime *stop);
	VIRTUAL int DeleteFileAtIndex(Serial *pSerial, int index);
	VIRTUAL int WriteFiles();
	int setFatEntry(STRogueFATEntry myFAT);
	int setImageFile(unsigned char *myImage, int fileSize);

	int setImageHeader(string rxID);
	void AddImage(TRogue *newTRogue);
	void AddImage(unsigned char *image, int fileSize, STRogueFATEntry *myFAT);

	static int SendQuery(Serial *pSerial, string QueryString);
	static int SendCommand(Serial *pSerial, string CommandString, int DoCheckSum = FALSE, int WaitAck = TRUE);
	static void ReverseFATEntry(unsigned char *outFAT, unsigned char *inFAT);
	static int Login(Serial *pSerial, int direct, string login, string password);
	static int Logout(Serial *pSerial);
	
	unsigned char *getImageFile() { return(imageFile); }
	int getImageFileSize() { return(cimageFile); }
	VIRTUAL int getIndex() { return(myIndex); }
	
private:
	// File Data
	STRogueFATEntry *pFATentry;
	
	// Clock Offset information
	int cimageFile;
	int myIndex;

	// sitename and times
	string sitename;
	
	// Actual RFile image
	unsigned char *imageFile;
	unsigned char *header;
	unsigned char *fatEntry;
	
	char session;

	static STRogueFATEntry pFAT[MAX_RCVR_FILES];

};
#endif
