/* 
#    $Id: Topcon.h,v 1.1 2011/04/04 17:29:34 gps Exp $
#
#    Topcon.h -- Include file for the Topcon Receiver
#
#    Copyright (C) 2000 US Geological Survey
#    Written by Keith F. Stark
#    
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifndef _TOPCON_H
#define _TOPCON_H

#include <gpstime.h>
#include <string>
#include <gpscommon.h>
#include <TopconDefs.h>
#include <Receiver.h>

#define FILELENGTH 12

class Topcon : public Receiver {
public:
	Topcon();

	static string DownloadReceiverID(Serial *pSerial);
	static int DownloadReceiverFAT(Serial *pSerial);
	static int GetReceiverID();
	static int SetReceiverSession(Serial *pSerial);
	static string GRILCommand(Serial *pSerial, const char *command);
	static int Login(Serial *pSerial, int direct, string login, string password);
	static int TerminalSession(Serial *pSerial);

	static int Stream(Serial *pSerial, int format, double rate, char port);
	static int BinexStream(Serial *pSerial, double rate, char port);


	VIRTUAL int DownloadFileByTime(Serial *pSerial, gpstime *start, gpstime *stop);
	VIRTUAL int DownloadFileAtIndex(Serial *pSerial, int index);
	int ReceiveA1File(Serial *pSerial, U_4_INT indexFile);

	VIRTUAL int DeleteFileByTime(Serial *pSerial, gpstime *start, gpstime *stop);
	VIRTUAL int DeleteFileAtIndex(Serial *pSerial, int index);
	VIRTUAL int WriteFiles();
	int setImageFile(vector <unsigned char> *datafile);

	void AddImage(Topcon *newTopcon);
	static void ReverseFATEntry(unsigned char *outFAT, unsigned char *inFAT);
	
	unsigned char *getImageFile() { return(imageFile); }
	int getImageFileSize() { return(cimageFile); }
	virtual int getIndex() { return(myIndex); }
	
private:
	// File Data
	int myIndex;

	// sitename and times
	string sitename;
	
	// Actual RFile image
	unsigned char * imageFile;
	int numMessages;
	int cimageFile;
	ReportStruct reportstruct;
	
	char session;

};
#endif
