/* 
#    $Id: TopconDefs.h,v 1.3 2012/02/10 00:48:57 gps Exp $
#
#    TopconDefs.h -- Data Structures 
#
#    Copyright (C) 2000 US Geological Survey
#    Written by Keith F. Stark
#    
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifndef _TOPCON_DEFS_H
#define _TOPCON_DEFS_H
/****************************************************************************/
/* these defs are for the new development                                   */
/****************************************************************************/


#define MAXLB2SIZE 2048
#define GENERAL_TIMEOUT 10000L
#define DELETION_TIMEOUT 120000L
#define TOPCON_XMODEM_CLEANUP_TIMEOUT 10000L

#define NUM_EXTRA_TOPCON_XMODEM_EOT 4
#define DELETION_COMPLETE_REPLY 'C'
#define MAX_PROG_SESSIONS 24
#define INTERMEDIATEHEADERLEN 20

#define MAX_TRIES_FOR_ID 10
#define MAX_MESSAGES_BEFORE_RESPONSE 10

#define MAX_TOPCON_FILES 2048

#define MAX_RCVR_PORTS 4

#define MAX_CHAN 12

#define CN0_SCALER 25

#define  W_L1 0.1902936727983649

#define  W_L2 0.2442102134245683

#define bAE84 6378137.0
#define bFLAT84 0.00335281066474


typedef char S_1_CHAR;
typedef unsigned char U_1_CHAR;
typedef unsigned char U_1_INT;
typedef int S_4_INT;
typedef unsigned int U_4_INT;
typedef short S_2_INT;
typedef unsigned short U_2_INT;
typedef float S_4_REAL;
typedef double S_8_REAL;
typedef long double S_10_REAL;

#define TPS_MAXVECTORS 30
#define TPS_ACK 0x06
#define TPS_NACK 0x015
#define TPS_ORDINAL 0x02
#define TPS_LAST 0x04
#define TPS_ABORT 0x23
#define TPS_BLOCKSIZE 1024
#define TPS_EOB 0x03

struct TPSBlock {
	U_1_CHAR type;	// Block type:
			// ORDINAL (SOB, 0x02)
			// LAST (EOT, 0x04)
			// ABORT (#, 0x23)

	// The following fields do not exist for block type 'ABORT'.
	union {
		U_2_INT number;	// Block number for block of type
				// ORDINAL (0 - based).
		S_2_INT count;	// Number of bytes of data in the
				// block of type LAST. Value -1
				// indicates transfer error. In this
				// case 'data' field of the block
				// contains a zero-terminated string
				// describing the error type.
	};

	U_1_CHAR data[TPS_BLOCKSIZE];	// Data. In the block of type LAST
			// only the first 'count' bytes are
			// valid. The rest of the bytes are filled
			// with zeroes.
	U_2_INT cs;	// Checksum of bytes starting from
			// the 'type' field up to, but not
			// including, 'cs' field. It is calculated
			// through all blocks starting from block 0.
	U_1_CHAR eob;	// End Of Block marker (EOB, 0x03).
};
typedef struct TPSBlock STPSBlock;

#endif
