/* 
#    $Id: Trimble.cpp,v 1.2 2018/02/23 04:34:13 gps Exp $
#
#	 Trimble.cpp -- This file will be the major interaction with
#                   an Trimble receiver/file
#    Copyright (C) 2000 US Geological Survey
#    Written by Keith F. Stark
#    
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <iostream>
#include <iomanip>
#include <fstream>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <stdio.h>
#include <time.h>
#include <sys/time.h>
#include <sys/types.h>

#include "gpscommon.h"
#include "Trimble.h"

int Trimble::ReceiverType;
string Trimble::UZ_FileNames[MAX_RCVR_FILES];

//  ===================================================================
//  NAME:		Trimble::Trimble
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
Trimble::Trimble () {

	debug(50,"Trimble::Trimble: Entering...\n");
	usesFAT = TRUE;
	myIndex = -1;

	// Actual RFile image
	cimageFile = 0;
	imageFile = NULL;
	header = NULL;
	fatEntry = NULL;
	session = ' ';
	ReceiverType = 0;
	debug(50,"Trimble::Trimble: Exiting\n");

}


//  ===================================================================
//  NAME:		Trimble::AddImage
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
void Trimble::AddImage(Trimble *newTrimble) {

/*
	debug(50,"Trimble::AddImage(Trimble): Entering...\n");
	AddImage(newTrimble->getImageFile(), newTrimble->getImageFileSize(), &pFAT[newTrimble->getIndex()]);
	debug(50,"Trimble::AddImage(Trimble): Entering...\n");
*/

}

//  ===================================================================
//  NAME:		Trimble::setImageHeader
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int Trimble::setImageHeader(string rxID) {

/*
	debug(50,"Trimble::setImageHeader: Entering...\n");

	if (pImageHead != NULL) {
		delete(pImageHead);
	}
	pImageHead = new RawImageHeader();

	if (header != NULL) {
		delete(header);
	}
	header = new unsigned char[RAWIMAGEHEADERSIZE_F];

	memset(pImageHead, 0, RAWIMAGEHEADERSIZE_F);
	strcpy(pImageHead->receiverType, "Z-12");
	memset(pImageHead->version,0,20);
	strcpy(pImageHead->version, rxID.substr(3,19).c_str());

	pImageHead->oneFileMode = (char) 1;
	pImageHead->res = (char) 0;
	memcpy((void *) header, (void *) pImageHead, RAWIMAGEHEADERSIZE_F);
	debug(50,"Trimble::setImageHeader: Exiting\n");

*/
	return(SUCCESS);
}


//  ===================================================================
//  NAME:		Trimble::
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int Trimble::setImageFile(unsigned char *myImage, int fileSize) {

	debug(50,"Trimble::setImageFile: Entering...\n");

	if (imageFile != NULL) {
		delete(imageFile);
	}

	imageFile = new unsigned char[fileSize];

	memcpy(imageFile, myImage, fileSize);
	cimageFile = fileSize;
	debug(50,"Trimble::setImageFile: Exiting\n");
	return(SUCCESS);

}

//  ===================================================================
//  NAME:		Trimble::
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int Trimble::WriteFiles() {

	ofstream rfile;

	debug(50,"Trimble::WriteRFile: Entering...\n");

	if (OpenFile(&rfile, GetFileName(myIndex).c_str()) != 0) return(FAILURE);

	debug(20,"cimageFile = %d\n",cimageFile);
	rfile.write((const char *)imageFile, cimageFile);
	rfile.close();
	debug(50,"Trimble::WriteRFile: Exiting\n");
	return(SUCCESS);

}

//  ===================================================================
//  NAME:		Trimble::
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int Trimble::DownloadFileAtIndex(Serial *pSerial, int index) {
	unsigned char *myfile = NULL;
	char buffer[100];
	time_t t_used = 0;
	int downLoadOK = FALSE;

	debug(50,"Trimble::DownloadFileAtIndex: Entering...\n");

	debug(25,"Trimble::DownloadFileAtIndex: Download at index %d\n",index);
	addToSumFile("\nDownload:\n");
	long fileSize = 0;

	sleep(1);
	pSerial->ClearBuffers();
	long expectSize = 0;
	sprintf(buffer,"%s -D%s -B%d -P%c --rtscts -version",RFILE, "/dev/ttyS0" , 38400, 'n');
	char *output = SendCommand(pSerial, buffer);

	if (output == NULL) cout << "hi";
	debug(25,"Trimble::DownloadFileAtIndex: Running Query %s\n", buffer);
//Read in file downloaded
	if ((myfile != NULL) && (fileSize >= expectSize)) {
		addToSumFile(" Index: %d\n",index);
		debug(25,"Trimble::DownloadFileAtIndex: Download OK, Copying image to Trimble Class\n");
		setImageFile(myfile, expectSize);
		myIndex = index;
		downLoadOK = TRUE;
		char buffer[100];
		addToSumFile(" Start Time: %s\n",ReceiverFAT[index].fileStart.chartime("%%G %%g", buffer, 100));
		addToSumFile(" End Time: %s\n",ReceiverFAT[index].fileEnd.chartime("%%G %%g", buffer, 100));
		addToSumFile(" File: %s\n",GetFileName(myIndex).c_str());
		addToSumFile(" Bytes: %d\n",expectSize);
		addToSumFile(" Time: %d seconds\n",t_used);
		delete [] myfile;
	} else {
		downLoadOK = FALSE;
		debug(1, "Trimble::DownloadFileAtIndex: Download Failed\n");
		debug(1, "Trimble::DownloadFileAtIndex: Clearing XMODEM Transfer\n");
		pSerial->ClearBuffers(); sleep(1); pSerial->ClearBuffers();
	}
	addToSumFile("\n");
	debug(50,"DownloadFiles: Exiting\n");
	return(downLoadOK);
}

//  ===================================================================
//  NAME:		Trimble::
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int Trimble::DownloadFileByTime(Serial *pSerial, gpstime *start, gpstime *stop) {
	int i;
	int numindex, indexes[MAX_RCVR_FILES];
	int downLoadOK = FALSE;
	char buffer[100];
	Trimble *tmp;

	debug(50,"Trimble::DownloadFileByTime: Entering...\n");

	numindex = FindIndexesByTime(start, stop, indexes, FALSE);

	if (numindex > 0) {
		debug(25,"Trimble::DownloadFileByTime: Downloading file at index %d\n", indexes[0]);
		downLoadOK = DownloadFileAtIndex(pSerial, indexes[0]);
	}
	if (downLoadOK && (numindex > 1)) {
		for (i = 1; i < numindex; i++) {
			tmp = new Trimble;
			debug(25,"Trimble::DownloadFileByTime: Downloading file at index %d\n", indexes[i]);
			downLoadOK = tmp->DownloadFileAtIndex(pSerial, indexes[i]);
			if (!downLoadOK) {
				delete tmp;
				break;
			}
			AddImage(tmp);
			delete tmp;
			addToSumFile(" Span: ");
			addToSumFile(start->chartime("%%G/%%g-",buffer,100));
			addToSumFile(stop->chartime("%%G/%%g\n",buffer,100));
		}
	}
	debug(50,"DownloadFileByTime: Exiting\n");
	return(downLoadOK);
}

//  ===================================================================
//  NAME:		Trimble::
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int Trimble::CloseOpenFile(Serial *pSerial) {

	debug(50,"CloseOpenFile: Entering...\n");
	debug(1,"CloseOpenFile:Closing Open File\n");
/*
	SendCommand(pSerial, "FIL,C", FALSE, FALSE);
*/
	debug(50,"CloseOpenFile: Exiting...\n");
	return(SUCCESS);
}

//  ===================================================================
//  NAME:		Trimble::
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int Trimble::DeleteFileAtIndex(Serial *pSerial, int index) {
	string Check;
	int found = 0;
	char buffer[100];
	int deleteOK = FALSE;

	sprintf(buffer,"%s -D%s -B%d -P%c --rtscts -version",RFILE, "/dev/ttyS0" , 38400, 'n');
	char *output = SendCommand(pSerial, buffer);

	if (output == NULL) cout << "hi";
	debug(1,"\n\nDeleting file at index %d -- ",index);
	if (found) {
		deleteOK = TRUE;
		debug(1," Deleted\n");
	} else {
		deleteOK = FALSE;
		debug(1," NOT Deleted\n\n");
	}
	debug(50,"DeleteFiles: Exiting\n");
	return(deleteOK);
}

//  ===================================================================
//  NAME:		Trimble::
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int Trimble::DeleteFileByTime(Serial *pSerial, gpstime *start, gpstime *stop) {
	int i,j;
	int numindex, indexes[MAX_RCVR_FILES];
	int deleteOK = FALSE;

	debug(50,"Trimble::DeleteFileByTime: Entering...\n");

	numindex = FindIndexesByTime(start, stop, indexes, TRUE);

	for (j = (MAX_RCVR_FILES-1); j >= 0; j--) {
		for (i = 0; i < numindex; i++) {
			if (indexes[i] != j) continue;
			debug(25,"Trimble::DeleteFileByTime: Deleteing file at index %d\n", indexes[i]);
			deleteOK = DeleteFileAtIndex(pSerial, indexes[i]);
		}
	}
	debug(50,"DeleteFiles: Exiting\n");
	return(deleteOK);
}
//  ===================================================================
//  NAME:		Trimble::
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
string Trimble::DownloadReceiverID(Serial *pSerial) {

	char buffer[1024];
	size_t tmp;

	debug(50,"Trimble::DownloadReceiverID: Entering...\n");
	if (! pSerial->isConnected()) {
		debug(1,"Not connected to the receiver\n");;
		return("");
	}
	debug(1,"Querying Receiver ID:\n");
	//sprintf(buffer,"%s -D%s -B%d -P%c --rtscts -version",RSTATUS, pSerial->portname(), pSerial->baudrate(), pSerial->parity());
	sprintf(buffer,"%s -D%s -B%d -P%c --rtscts -version",RSTATUS, "/dev/ttyS0" , 38400, 'n');

	char *output = SendCommand(pSerial, buffer);
	if (output == NULL) cout << "hi";

	if (RxID == "") {
		debug(1,"Unable to read Receiver ID\n");
		return(RxID);
	}

	if ((tmp = RxID.find("RID")) == string::npos) {
		debug(10,"Incorrect return string\n");
		RxID = "";
		return(RxID);
	}
	addToSumFile(" Receiver ID: %s\n",RxID.c_str());
	RxID.erase(0, RxID.find(",", tmp)+1);

	debug(50,"Trimble::DownloadReceiverID: Exiting\n");
	return(RxID);
}

//  ===================================================================
//  NAME:		Trimble::
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int Trimble::VerifyReceiverSession(Serial *pSerial) {
	debug(50,"Trimble::VerifyReceiverSession: Entering...\n");

/*
	STrimbleSession tmpsess;
	int c;
	unsigned char buffer[1025];

	SendQuery(pSerial,"SCRN,70");
	c = pSerial->Read(buffer, 1024, 4000);

	int indexMem = 0;
	indexMem += CopyType(RFI_SHORT,(void *)&tmpsess.refday, (void *)&buffer[indexMem], 1, ASHBYTEORDER);
	indexMem += CopyType(RFI_SHORT,(void *)&tmpsess.offset, (void *)&buffer[indexMem], 1, ASHBYTEORDER);
	indexMem += CopyType(RFI_SHORT,(void *)&tmpsess.inuse, (void *)&buffer[indexMem], 1, ASHBYTEORDER);
	for (int i = 0; i < NUM_SESS; i++) {
		indexMem += CopyType(RFI_CHAR,(void *)&tmpsess.flag[i], (void *)&buffer[indexMem], 1, ASHBYTEORDER);
	}
	for (int i = 0; i < NUM_SESS; i++) {
		indexMem += CopyType(RFI_CHAR,(void *)&tmpsess.colstat[i], (void *)&buffer[indexMem], 1, ASHBYTEORDER);
	}
	for (int i = 0; i < NUM_SESS; i++) {
		indexMem += CopyType(RFI_LONG,(void *)&tmpsess.start[i], (void *)&buffer[indexMem], 1, ASHBYTEORDER);
	}
	for (int i = 0; i < NUM_SESS; i++) {
		indexMem += CopyType(RFI_LONG,(void *)&tmpsess.end[i], (void *)&buffer[indexMem], 1, ASHBYTEORDER);
	}
	for (int i = 0; i < NUM_SESS; i++) {
		indexMem += CopyType(RFI_LONG,(void *)&tmpsess.span[i], (void *)&buffer[indexMem], 1, ASHBYTEORDER);
	}
	for (int i = 0; i < NUM_SESS; i++) {
		indexMem += CopyType(RFI_FLOAT,(void *)&tmpsess.interval[i], (void *)&buffer[indexMem], 1, ASHBYTEORDER);
	}
	for (int i = 0; i < NUM_SESS; i++) {
		indexMem += CopyType(RFI_CHAR,(void *)&tmpsess.elevmask[i], (void *)&buffer[indexMem], 1, ASHBYTEORDER);
	}
	for (int i = 0; i < NUM_SESS; i++) {
		indexMem += CopyType(RFI_CHAR,(void *)&tmpsess.minsv[i], (void *)&buffer[indexMem], 1, ASHBYTEORDER);
	}
	for (int i = 0; i < NUM_SESS; i++) {
		indexMem += CopyType(RFI_CHAR,(void *)&tmpsess.type[i], (void *)&buffer[indexMem], 1, ASHBYTEORDER);
	}
	indexMem += CopyType(RFI_USHORT,(void *)&tmpsess.checksum, (void *)&buffer[indexMem], 1, ASHBYTEORDER);


	unsigned short test = 0;
	unsigned char *ptr = (unsigned char*)buffer;
	for( int i = 0; i < c-2; i++)
	{
		test += *ptr;
		ptr++;
	}

	if (test != tmpsess.checksum) {
		debug(5,"Trimble::VerifyReceiverSession: checksum %d != %d\n", test, tmpsess.checksum);
		return(FAILURE);
	}

	addToSumFile("\nSession Programming:\n RefDay = %d, Offset = %d, inuse = %d\n",
				tmpsess.refday, tmpsess.offset, tmpsess.inuse);
	addToSumFile("inuse start      end  interval mask minsv type\n");
	for (int i = 0; i < NUM_SESS; i++) {
		addToSumFile("%2d %c      %02d:%02d    %02d:%02d %03d      %03d  %03d    %03d\n",
				i,
				tmpsess.flag[i],
				tmpsess.start[i]/3600, (tmpsess.start[i]%3600)/60,
				tmpsess.end[i]/3600, (tmpsess.end[i]%3600)/60,
				(int)tmpsess.interval[i], (int)tmpsess.elevmask[i],
				(int)tmpsess.minsv[i], (int)tmpsess.type[i]);
	}

	int found;
	for (int i = 0; i < numSessions; i++) {
		long start    = GetSessionStartTime(i);
		long end      = GetSessionEndTime(i);
		float interval = GetSessionInterval(i);
		int elevmask = GetSessionElevationMask(i);
		int minsv    = GetSessionMinimumSV(i);
		found = FALSE;
		for (int j = 0; (j < NUM_SESS) && (!found); j++) {
			if (tmpsess.flag[j] == 'N') continue;
			if (start != tmpsess.start[j]) continue;
			if (end != tmpsess.end[j]) continue;
			if (interval != tmpsess.interval[j]) continue;
			if (elevmask != tmpsess.elevmask[j]) continue;
			if (minsv != tmpsess.minsv[j]) continue;
			debug(25,"Trimble::VerifyReceiverSession: Found Match for Session %d\n",j);
			found = TRUE;
		}
	}
	debug(50,"Trimble::VerifyReceiverSession: Exiting...\n");
	if (found) return(SUCCESS);
*/
	return(FAILURE);
}

//  ===================================================================
//  NAME:		Trimble::
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int Trimble::SetReceiverSession(Serial *pSerial) {

	debug(50,"Trimble::SetReceiverSession: Entering...\n");

	debug(50,"Trimble::SetReceiverSession: Exiting...\n");
	return(SUCCESS);
}

//  ===================================================================
//  NAME:		Trimble::
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int Trimble::DownloadReceiverFAT(Serial *pSerial) {
	int i;
	char buffer[100];

	debug(50,"Trimble::DownloadReceiverFAT: Entering...\n");

	sprintf(buffer,"%s -D%s -B%d -P%c --rtscts -version",RFILE, "/dev/ttyS0" , 38400, 'n');
	char *output = SendCommand(pSerial, buffer);

	if (output == NULL) cout << "hi";
	debug(50,"Trimble::DecodeReceverFAT: Entering...\n");
	numFATEntries = 0;
	debug(10,"Trimble::DecodeReceiverFAT: There are %d files\n",numFATEntries);
	for (i = 0; i < numFATEntries; i++) {
		debug(25,"Trimble::DecodeReceiverFAT: decoding file number %d\n",i);
		int k;
		ReceiverFAT[i].siteName.erase(0,ReceiverFAT[i].siteName.length());
		for (k = 0; k < 4 ; k++) {
			ReceiverFAT[i].siteName += "_";
		}
		debug(30,"Trimble::DownloadReceiverFAT: sitename is %s\n",ReceiverFAT[i].siteName.c_str());
		ReceiverFAT[i].fileSize = 0;
		ReceiverFAT[i].fileStart.set_time(0, 0);
		ReceiverFAT[i].fileEnd.set_time(0, 0);
		FindSession(i);
		CreateFileName(i);
	}
	debug(50,"Trimble::DownloadReceiverFAT: Exiting...\n");
	debug(50,"Trimble::DownloadReceiverFAT: Exiting\n");
	return(SUCCESS);

}

//  ===================================================================
//  NAME:		Trimble::
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
char * Trimble::SendCommand(Serial *pSerial, string CommandString) {

/*
	char buffer[101];
	string Send = "$PASHS,";

	debug(50,"Trimble::SendCommand: Entering...\n");
	Send += CommandString;

	if (! pSerial->isConnected()) {
		debug(1,"Not connected to the receiver\n");
		return(FAILURE);
	}
	int Checksum = 0;
	if (DoCheckSum) {
		for (int i = 1; i < (int)Send.length(); i++) {
			Checksum ^= Send[i];
		}
		sprintf(buffer, "*%02X", Checksum);
		Send += buffer;
	}
	debug(25,"Trimble::SendCommand: Sending %s\n", Send.c_str());
	Send += "\r\n";
	pSerial->ClearBuffers();
	pSerial->Write((char *)Send.c_str());

	int found = FALSE;
	if (WaitAck) {
		string Check;
		int ticks = 3000;
		int ret;
		while ((ticks > 0) && (found == FALSE)) {
			if ((ret = pSerial->Readline(buffer,100,500)) != 0) {
				Check += buffer;
				if (Check.find("ACK") != string::npos) found = TRUE;
				if (Check.find("NAK") != string::npos) ticks = 0;
			}
			debug(50,"Trimble::SendCommand: Waiting for response -%s- %d\r",Check.c_str(), ticks);
			ticks -= 500;
		}
		debug(25,"\nTrimble::SendCommand: %s, found = %d\n",Check.c_str(), found);
	} else {
		found = TRUE;
	}
	debug(50,"SendCommand: Exiting\n");
	return(found);
*/
	return((char *)"");
}

