/* 
#    $Id: Trimble.h,v 1.3 2012/02/10 00:48:57 gps Exp $
#
#    Trimble.h -- Include file for the Trimble Receiver
#
#    Copyright (C) 2000 US Geological Survey
#    Written by Keith F. Stark
#    
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifndef _TRIMBLE_H
#define _TRIMBLE_H

#include <string>

#include <gpscommon.h>
#include <TrimbleDefs.h>
#include <Receiver.h>
#include <gpstime.h>

#define FILELENGTH 12
#define TRIMBLEBYTEORDER __BIG_ENDIAN

class Trimble : public Receiver {
public:
	Trimble();

	static string DownloadReceiverID(Serial *pSerial);
// Download fat
	static int DownloadReceiverFAT(Serial *pSerial);
	static int GetFATEntryAtIndex(SFATEntry *pFAT, int index);
	static int GetReceiverID();
// Set Sessions
	static int SetReceiverSession(Serial *pSerial);

	static int VerifyReceiverSession(Serial *pSerial);
	static int CloseOpenFile(Serial *pSerial);
	static int TerminalSession(Serial *pSerial) {return(1);};


	VIRTUAL int DownloadFileByTime(Serial *pSerial, gpstime *start, gpstime *stop);
	// Download File routines
	VIRTUAL int DownloadFileAtIndex(Serial *pSerial, int index);

	VIRTUAL int DeleteFileByTime(Serial *pSerial, gpstime *start, gpstime *stop);
	VIRTUAL int DeleteFileAtIndex(Serial *pSerial, int index);
	VIRTUAL int WriteFiles();
	int setImageFile(unsigned char *myImage, int fileSize);

	int setImageHeader(string rxID);
	void AddImage(Trimble *newTrimble);

	static char * SendCommand(Serial *pSerial, string CommandString);
	static void ReverseFATEntry(unsigned char *outFAT, unsigned char *inFAT);
	
	virtual unsigned char *getImageFile() { return(imageFile); }
	virtual int getImageFileSize() { return(cimageFile); }
	virtual int getIndex() { return(myIndex); }
	
private:
	// Clock Offset information
	int cimageFile;
	int myIndex;

	// sitename and times
	string sitename;
	
	// Actual RFile image
	unsigned char *imageFile;
	unsigned char *header;
	unsigned char *fatEntry;
	
	char session;

	// Receiver Type
	static int ReceiverType;

	static string UZ_FileNames[MAX_RCVR_FILES];

};
#endif
