/* 
#    $Id: Z12.cpp,v 1.1 2011/04/04 17:29:34 gps Exp $
#
#	 Z12.cpp -- This file will be the major interaction with
#                   an ashtech Z12 receiver
#    Copyright (C) 2000 US Geological Survey
#    Written by Keith F. Stark
#    
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <iostream>
#include <fstream>
#include <iomanip>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <stdio.h>
#include <time.h>
#include <sys/time.h>
#include <sys/types.h>

#include "gpscommon.h"
#include "Z12.h"

SAshtechFATEntry Z12::pFAT[MAX_RCVR_FILES];

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
Z12::Z12 () {

	debug(50,"Z12::Ashtech: Entering...\n");
	pFATentry = NULL;
	pImageHead = NULL;

	usesFAT = TRUE;

	// Actual RFile image
	header = NULL;
	fatEntry = NULL;
	debug(50,"Z12::Ashtech: Exiting\n");

}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
void Z12::AddImage(unsigned char *image, int fileSize, SAshtechFATEntry *myFAT) {
	unsigned char *tmp;

	debug(50,"Z12::AddImage(char, int, FAT): Entering...\n");

	tmp = new unsigned char[cimageFile+fileSize];
	memcpy(tmp, imageFile, cimageFile);
	memcpy(&tmp[cimageFile], image, fileSize);
	setImageFile(tmp, cimageFile+fileSize);
	debug(25,"Z12::Addimage(char,int,FAT): fileSize is %d\n",cimageFile);
	delete [] tmp;


	pFATentry->segbeg = 10L+((long)(MAX_RCVR_FILES)*(long)(ASHTECHFATENTRYSIZE_F))/2L;
	pFATentry->wordcount = myFAT->segbeg - pFATentry->segbeg;
	pFATentry->segwn = myFAT->segwn;
	pFATentry->segtime = myFAT->segtime;
	pFATentry->anthiaft = myFAT->anthiaft;
	pFATentry->tdryaft = myFAT->tdryaft;
	pFATentry->twetaft = myFAT->twetaft;
	pFATentry->humidaft = myFAT->humidaft;
	pFATentry->pressureaft = myFAT->pressureaft;

	setFatEntry(*pFATentry);
	debug(50,"Z12::AddImage(char, int, FAT): Exiting...\n");

}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
void Z12::AddImage(Ashtech *newAshtech) {

	debug(50,"Z12::AddImage(Ashtech): Entering...\n");
	AddImage(newAshtech->getImageFile(), newAshtech->getImageFileSize(), &pFAT[newAshtech->getIndex()]);
	debug(50,"Z12::AddImage(Ashtech): Entering...\n");

}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int Z12::setImageHeader(string rxID) {

	debug(50,"Z12::setImageHeader: Entering...\n");

	if (pImageHead != NULL) {
		delete(pImageHead);
	}
	pImageHead = new RawImageHeader();

	if (header != NULL) {
		delete(header);
	}
	header = new unsigned char[RAWIMAGEHEADERSIZE_F];

	memset(pImageHead, 0, RAWIMAGEHEADERSIZE_F);
	strcpy(pImageHead->receiverType, "Z-12");
	memset(pImageHead->version,0,20);
	strcpy(pImageHead->version, rxID.substr(3,19).c_str());

	pImageHead->oneFileMode = (char) 1;
	pImageHead->res = (char) 0;
	memcpy((void *) header, (void *) pImageHead, RAWIMAGEHEADERSIZE_F);
	debug(50,"Z12::setImageHeader: Exiting\n");

	return(SUCCESS);
}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int Z12::setFatEntry(SAshtechFATEntry myFAT) {

	int indexMem = 0;

	debug(50,"Z12::setFatEntry: Entering...\n");

	if (pFATentry != NULL) delete(pFATentry);
	if (fatEntry != NULL) delete(fatEntry);

	pFATentry = new SAshtechFATEntry;
	memcpy(pFATentry, (void *) &myFAT, sizeof(myFAT));

	pFATentry->segbeg = 10L+((long)(MAX_RCVR_FILES)*(long)(ASHTECHFATENTRYSIZE_F))/2L;

	fatEntry = new unsigned char[MAX_RCVR_FILES * ASHTECHFATENTRYSIZE_F];
	ReverseFATEntry((unsigned char *) fatEntry, (unsigned char *) pFATentry);
	indexMem += sizeof(SAshtechFATEntry);
	memset((void *) &fatEntry[indexMem], 0, (MAX_RCVR_FILES - 1) * ASHTECHFATENTRYSIZE_F);

	debug(50,"Z12::setFatEntry: Exiting\n");

	return(SUCCESS);
}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int Z12::WriteFiles() {

	ofstream rfile;
	unsigned char intermed[INTERMEDIATEHEADERLEN];
	int status = SUCCESS;

	debug(50,"Z12::WriteRFile: Entering...\n");

	memset(intermed, 0, INTERMEDIATEHEADERLEN);

	try
	{

		status = WriteFile(NULL, GetFileName(myIndex).c_str(), header, RAWIMAGEHEADERSIZE_F, FALSE); 
		if (status == FAILURE) throw string("failed writing header\n");

		status = WriteFile(NULL, GetFileName(myIndex).c_str(), intermed, INTERMEDIATEHEADERLEN, TRUE); 
		if (status == FAILURE) throw string("failed writing intermediate header\n");

		status = WriteFile(NULL, GetFileName(myIndex).c_str(), fatEntry, MAX_RCVR_FILES * ASHTECHFATENTRYSIZE_F, TRUE); 
		if (status == FAILURE) throw string("failed writing FAT entry\n");

		status = WriteFile(NULL, GetFileName(myIndex).c_str(), imageFile, cimageFile, TRUE); 
		if (status == FAILURE) throw string("failed writing imageFile contents\n");

	}
	catch (string pmessage)
	{
		debug(1,"%s\n",pmessage.c_str());
	}

	debug(50,"Z12::WriteRFile: Exiting\n");
	return(status);

}


//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int Z12::DownloadFileByTime(Serial *pSerial, gpstime *start, gpstime *stop) {
	int i;
	int numindex, indexes[MAX_RCVR_FILES];
	int status = FAILURE;
	char buffer[100];

	debug(50,"Ashtech::DownloadFileByTime: Entering...\n");


// Trying ring buffer download
//
// Instead, I looked at the source code from firmware and CGREMOTE side and
// find description in
// attached document. The $PASHQ,RIN command has a format of
// 
//  sprintf ( cmd_str,"$PASHQ,RIN,%05d,%c,%05d,%07ld,%05d,%07ld",
//            (int)(ringdisp->dnldintv*10),
//            dnld_mode, begweek, begsec, endweek, endsec);
//
//ringdisp->dnldintv is in units of second. 
//
//dnld_mode is either 'F' or 'P',
//
//begweek and endweek are just week number, e.g. 1225
//
//begsec and endsec are in units of 0.1 second.
//
//Beyond that, I will need more effort to document this or we need to send him
//some CGREMOTE source code.
//

#if 0
	char buffer2[200];
	sprintf(buffer,"RIN,%05d,%c,%05d,%07ld,%05d,%07ld",300,'P', atoi(start->chartime("%%G",buffer2,100)), atol(start->chartime("%%g",buffer2,100)), atoi(stop->chartime("%%G",buffer2,100)), atol(stop->chartime("%%g",buffer2,100)));
	debug(25,"Z12::DownloadFileAtIndex: Running Query %s\n", buffer);
	SendQuery(pSerial, buffer);
	sleep(1);
	long fileSize = 10000;
	int index = 1;
	unsigned char * myfile = pSerial->xmodemreceive(&fileSize, 2400, GetFileName(index).c_str());

	return(SUCCESS);

#else
	numindex = FindIndexesByTime(start, stop, indexes, FALSE);

	if (numindex > 0) {
		debug(25,"Ashtech::DownloadFileByTime: Downloading file at index %d\n", indexes[0]);
		status = DownloadFileAtIndex(pSerial, indexes[0]);
	} else {

		debug(25,"Ashtech::DownloadFileByTime: No Files found for time range\n");
		status = SUCCESS_NOFILE;
	}
	if ((status == SUCCESS) && (numindex > 1)) {
		for (i = 1; i < numindex; i++) {
			debug(25,"Ashtech::DownloadFileByTime: Downloading file at index %d\n", indexes[i]);
			Z12 *pTmp = new Z12;
			status = pTmp->DownloadFileAtIndex(pSerial, indexes[i]);
			if (status == FAILURE) {
				delete(pTmp);
				break;
			}
			AddImage(pTmp);
			delete(pTmp);
			addToSumFile(" Span: ");
			addToSumFile(start->chartime("%%G/%%g-",buffer,100));
			addToSumFile(stop->chartime("%%G/%%g\n",buffer,100));
		}
	}
	debug(50,"DownloadFileByTime: Exiting\n");
	return(status);
#endif
}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int Z12::DownloadFileAtIndex(Serial *pSerial, int index) {
	unsigned char *myfile = NULL;
	char buffer[100];
	time_t t_start;
	time_t t_used = 0;
	int downLoadOK = FALSE;

	debug(50,"Z12::DownloadFileAtIndex: Entering...\n");

	debug(25,"Z12::DownloadFileAtIndex: Download at index %d\n",index);

	if (index >= numFATEntries) {
		debug(25,"Z12::DownloadFileAtIndex: Index %d does not exist\n",index);
		return(FAILURE);
	}

	addToSumFile("\nDownload:\n");
	long fileSize;

	long expectSize = pFAT[index].wordcount * 2;
	if (indexData[index] == NULL) {
		sleep(1);
		pSerial->ClearBuffers();
		sprintf(buffer,"BLK,%lX,%lX",pFAT[index].segbeg, expectSize);
		debug(25,"Z12::DownloadFileAtIndex: Running Query %s\n", buffer);
		SendQuery(pSerial, buffer);
		sleep(1);
		t_start = time(NULL);
		debug(25,"Z12::DownloadFileAtIndex: Calling Xmodem Transfer, timeout = %d, Start Time = %d\n",2400,t_start);
		fileSize = expectSize;
		myfile = pSerial->xmodemreceive(&fileSize, 2400, GetFileName(index).c_str());
		t_used = time(NULL) - t_start;
		debug(25,"Z12::DownloadFileAtIndex: Returned From Xmodem Transfer, fileSize = %d, Time Used = %d\n", fileSize,t_used);
	} else {
		myfile = new unsigned char[expectSize];
		if (myfile == NULL) return(FAILURE);
		memcpy(myfile, indexData[index], expectSize);
		fileSize = expectSize;
	}
	if ((myfile != NULL) && (fileSize >= expectSize)) {
		addToSumFile(" Index: %d\n",index+1);
		debug(25,"Z12::DownloadFileAtIndex: Download OK, Copying image to Ashtech Class\n");
		setImageHeader(RxID);
		setFatEntry(pFAT[index]);
		setImageFile(myfile, expectSize);
		AddToIndexBuffer(index, myfile, expectSize);
		myIndex = index;
		downLoadOK = TRUE;
		char buffer[100];
		addToSumFile(" Start Time: %s\n",ReceiverFAT[index].fileStart.chartime("%%G %%g", buffer, 100));
		addToSumFile(" End Time: %s\n",ReceiverFAT[index].fileEnd.chartime("%%G %%g", buffer, 100));
		addToSumFile(" File: %s\n",GetFileName(myIndex).c_str());
		addToSumFile(" Bytes: %d\n",expectSize);
		addToSumFile(" Time: %d seconds\n",t_used);
		delete [] myfile;
	} else {
		downLoadOK = FALSE;
		debug(1, "Z12::DownloadFileAtIndex: Download Failed\n");
		debug(1, "Z12::DownloadFileAtIndex: Clearing XMODEM Transfer\n");
		pSerial->ClearBuffers(); sleep(1); pSerial->ClearBuffers();
	}
	addToSumFile("\n");
	debug(50,"DownloadFileAtIndex: Exiting\n");
	return((downLoadOK) ? SUCCESS : FAILURE);
}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int Z12::DeleteFileAtIndex(Serial *pSerial, int index) {
	string Check;
	int found;
	char buffer[100];
	int status = FAILURE;

	sprintf(buffer,"FIL,D,%d",index);
	debug(D_STD,"\n\nDeleting file #%d",index+1);
	debug(D_FLOW," (index #%d)", index);
	debug(D_STD," -- ");
	SendCommand(pSerial, buffer, FALSE, FALSE);
	found = (pSerial->WaitFor(180000, 'C') == 0);
	if (found) status = SUCCESS;
	debug(D_STD,(status == SUCCESS) ? " Deleted\n" :  "NOT Deleted\n\n");
	debug(D_FLOW,"DeleteFiles: Exiting\n");
	return(status);
}


//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
/*
 * Sorry for the delay with the response. There is undocumented command:
 * $PASHQ,SCRN,70 
 * 
 * In response it will send you the following binary structure:
 * 
 * #define  NUM_SESS 24
 * struct sessionstrct {
 *    short  refday;
 *    short  offset;
 *    short  inuse;
 *    char   flag[NUM_SESS];
 *    char   colstat[NUM_SESS];
 *    long   start[NUM_SESS];
 *    long   end[NUM_SESS];
 *    long   span[NUM_SESS];
 *    float  interval[NUM_SESS];
 *    char   elevmask[NUM_SESS];
 *    char   minsv[NUM_SESS];
 *    char   type[NUM_SESS];
 *    unsigned short checksum;
 * }session;
 * 
 * You will need to change the byte order from big endian to the byte order on
 * the host machine and check the checksum, which is computed in the receiver
 * the following way:
 * 
 * ptr = (unsigned char*)&session;
 * for( i = 0; i < sizeof(session)-2; i++)
 * {
 *         session.checksum += *ptr;
 *         ptr++;
 * }
 * 
 */

int Z12::VerifyReceiverSession(Serial *pSerial) {
	debug(50,"Z12::VerifyReceiverSession: Entering...\n");

	SAshtechSession tmpsess;
	int c;
	unsigned char buffer[1025];

	pSerial->set_raw(1);
	SendQuery(pSerial,"SCRN,70");
	c = pSerial->Read(buffer, 1024, 4000);
	pSerial->set_raw(0);

	int indexMem = 0;
	indexMem += CopyType(RFI_SHORT,(void *)&tmpsess.refday, (void *)&buffer[indexMem], 1, ASHBYTEORDER);
	indexMem += CopyType(RFI_SHORT,(void *)&tmpsess.offset, (void *)&buffer[indexMem], 1, ASHBYTEORDER);
	indexMem += CopyType(RFI_SHORT,(void *)&tmpsess.inuse, (void *)&buffer[indexMem], 1, ASHBYTEORDER);
	for (int i = 0; i < NUM_SESS; i++) {
		indexMem += CopyType(RFI_CHAR,(void *)&tmpsess.flag[i], (void *)&buffer[indexMem], 1, ASHBYTEORDER);
	}
	for (int i = 0; i < NUM_SESS; i++) {
		indexMem += CopyType(RFI_CHAR,(void *)&tmpsess.colstat[i], (void *)&buffer[indexMem], 1, ASHBYTEORDER);
	}
	for (int i = 0; i < NUM_SESS; i++) {
		indexMem += CopyType(RFI_LONG,(void *)&tmpsess.start[i], (void *)&buffer[indexMem], 1, ASHBYTEORDER);
	}
	for (int i = 0; i < NUM_SESS; i++) {
		indexMem += CopyType(RFI_LONG,(void *)&tmpsess.end[i], (void *)&buffer[indexMem], 1, ASHBYTEORDER);
	}
	for (int i = 0; i < NUM_SESS; i++) {
		indexMem += CopyType(RFI_LONG,(void *)&tmpsess.span[i], (void *)&buffer[indexMem], 1, ASHBYTEORDER);
	}
	for (int i = 0; i < NUM_SESS; i++) {
		indexMem += CopyType(RFI_FLOAT,(void *)&tmpsess.interval[i], (void *)&buffer[indexMem], 1, ASHBYTEORDER);
	}
	for (int i = 0; i < NUM_SESS; i++) {
		indexMem += CopyType(RFI_CHAR,(void *)&tmpsess.elevmask[i], (void *)&buffer[indexMem], 1, ASHBYTEORDER);
	}
	for (int i = 0; i < NUM_SESS; i++) {
		indexMem += CopyType(RFI_CHAR,(void *)&tmpsess.minsv[i], (void *)&buffer[indexMem], 1, ASHBYTEORDER);
	}
	for (int i = 0; i < NUM_SESS; i++) {
		indexMem += CopyType(RFI_CHAR,(void *)&tmpsess.type[i], (void *)&buffer[indexMem], 1, ASHBYTEORDER);
	}
	indexMem += CopyType(RFI_USHORT,(void *)&tmpsess.checksum, (void *)&buffer[indexMem], 1, ASHBYTEORDER);


	unsigned short test = 0;
	unsigned char *ptr = (unsigned char*)buffer;
	for( int i = 0; i < c-2; i++)
	{
		test += *ptr;
		ptr++;
	}

	if (test != tmpsess.checksum) {
		debug(5,"Z12::VerifyReceiverSession: checksum %d != %d\n", test, tmpsess.checksum);
		debug(55,"Z12::VerifyReceiverSession: length %d != %d\n", c, sizeof(tmpsess));
		return(FAILURE);
	}

	return(CompareSessions(&tmpsess));
#if 0
	addToSumFile("\nSession Programming:\n RefDay = %d, Offset = %d, inuse = %d\n",
				tmpsess.refday, tmpsess.offset, tmpsess.inuse);
	addToSumFile("inuse start      end  interval mask minsv type\n");
	for (int i = 0; i < NUM_SESS; i++) {
		addToSumFile("%2d %c      %02d:%02d    %02d:%02d %03.1f      %03d  %03d    %03d\n",
				i,
				tmpsess.flag[i],
				tmpsess.start[i]/3600, (tmpsess.start[i]%3600)/60,
				tmpsess.end[i]/3600, (tmpsess.end[i]%3600)/60,
				tmpsess.interval[i], (int)tmpsess.elevmask[i],
				(int)tmpsess.minsv[i], (int)tmpsess.type[i]);
	}

	int found;
	for (int i = 0; i < numSessions; i++) {
		long start    = GetSessionStartTime(i);
		long end      = GetSessionEndTime(i);
		float interval = GetSessionInterval(i);
		int elevmask = GetSessionElevationMask(i);
		int minsv    = GetSessionMinimumSV(i);
		found = FALSE;
		for (int j = 0; (j < NUM_SESS) && (!found); j++) {
			if (tmpsess.flag[j] == 'N') continue;
			if (start != tmpsess.start[j]) continue;
			if (end != tmpsess.end[j]) continue;
			if (interval != tmpsess.interval[j]) continue;
			if (elevmask != tmpsess.elevmask[j]) continue;
			if (minsv != tmpsess.minsv[j]) continue;
			debug(25,"Z12::VerifyReceiverSession: Found Match for Session %d\n",j);
			found = TRUE;
		}
		if (!found) break;
	}
	debug(50,"Z12::VerifyReceiverSession: Exiting...\n");
	if (found) return(SUCCESS);
	return(FAILURE);
#endif
}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int Z12::SetReceiverSession(Serial *pSerial) {
	int refday = 1;
	int offsec = 0;
	int rectype = 3;

	float interval = 30;
	int elevmask = 0;
	int start = 0;
	int end = 0;
	int minsv = 1;
	char inuse = 'N';
	char buffer[200];

	debug(50,"Z12::SetReceiverSession: Entering...\n");

	int status = VerifyReceiverSession(pSerial);
	if (status == SUCCESS) {
		debug(5,"Z12::SetReceiverSessions: Sessions already match, doing nothing\n");
		return(SUCCESS);
	}
	pSerial->ClearBuffers();
	sprintf(buffer, "SCRN,70,Y%03d%04d",refday, offsec);
	SendCommand(pSerial, buffer, TRUE, FALSE);
	if (pSerial->WaitFor(3000, 6) == 0) {
		debug(25,"Z12::SetReceiverSession: Reference day ACK'ed\n");
	}
	debug(25,"Z12::SetReceiverSession: Deleting current sessions\n");
	for (int i = 0; i <= 23; i++) {
		sprintf(buffer, "SCRN,%02d,%05d%05d%04d%02d%01d%01d%c", 
			71+i, start, end, (int) (interval*10), elevmask, minsv, rectype, 'N');
		SendCommand(pSerial, buffer, TRUE, FALSE);
		if (pSerial->WaitFor(3000, 6) == 0) {
			debug(25,"Z12::SetReceiverSession: Session set ACK'ed\n");
		}
	}

	sprintf(buffer, "SCRN,70,Y%03d%04d",refday, offsec);
	SendCommand(pSerial, buffer, TRUE, FALSE);
	if (pSerial->WaitFor(3000, 6) == 0) {
		debug(25,"Z12::SetReceiverSession: Reference day ACK'ed\n");
	}
	debug(25,"Z12::SetReceiverSession: Adding new sessions\n");
	for (int i = 0; i <= 23; i++) {
		if (i < numSessions) {
			start    = GetSessionStartTime(i);
			end      = GetSessionEndTime(i);
			interval = GetSessionInterval(i);
			elevmask = GetSessionElevationMask(i);
			minsv    = GetSessionMinimumSV(i);
			inuse    = 'Y';
		} else {
			start    = 0;
			end      = 0;
			interval = 30;
			elevmask = 0;
			minsv    = 1;
			inuse    = 'N';
		}
		sprintf(buffer, "SCRN,%02d,%05d%05d%04d%02d%01d%01d%c", 
			71+i, start, end, (int) (interval*10), elevmask, minsv, rectype, inuse);
		SendCommand(pSerial, buffer, TRUE, FALSE);
		if (pSerial->WaitFor(3000, 6) == 0) {
			debug(25,"Z12::SetReceiverSession: Session set ACK'ed\n");
		}
	}

	status = VerifyReceiverSession(pSerial);
	if (status != 0) {
		debug(5,"Z12::SetReceiverSession: session mismatch after setting\n");
		return(FAILURE);
	}
	status = CloseOpenFile(pSerial);
	debug(50,"Z12::SetReceiverSession: Exiting...\n");
	return(SUCCESS);
}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
#define MemInfoSize 10
#define FileInfoSize 40
#define MaxFileNumbers 100
int Z12::DownloadReceiverFAT(Serial *pSerial) {

	int FatInfoSize = MemInfoSize + (FileInfoSize * MaxFileNumbers);
	unsigned char *myFAT = NULL;
	struct timeval tv;
	long fatSize;
	int i;
	short memheader[10];
	int index = 0;
	char buffer[100];


	debug(50,"Z12::DownloadReceiverFAT: Entering...\n");
	tv.tv_sec = 1;
	tv.tv_usec = 0;

	fatSize = (unsigned long) FatInfoSize*2;
	sprintf(buffer,"BLK,%lX,%lX",(unsigned long)0,(unsigned long)fatSize);
	debug(10,"buffer = %s\n",buffer);
	SendQuery(pSerial, buffer);
	select(1,NULL,NULL,NULL,&tv);
	myFAT = pSerial->xmodemreceive(&fatSize, 200, NULL);
	debug(10,"fatSize = %d\n",fatSize);
	if ((myFAT == NULL) || (fatSize == 0)) return(FAILURE);

	debug(50,"Z12::DecodeReceverFAT: Entering...\n");
	numFATEntries = 0;
	for (i = 0; i < 10; i++) {
		index += CopyType(RFI_SHORT,  (void *)&memheader[i], (void *)&myFAT[index],1, ASHBYTEORDER);
	}
	numFATEntries = memheader[9] + 1;
	debug(10,"Z12::DecodeReceiverFAT: There are %d files\n",numFATEntries);
	for (i = 0; i < numFATEntries; i++) {
		debug(25,"Z12::DecodeReceiverFAT: decoding file number %d\n",i);
		ReverseFATEntry((unsigned char *) &pFAT[i], (unsigned char *) &myFAT[index]);
		index += sizeof(pFAT[i]);
		int k;
		ReceiverFAT[i].siteName.erase(0,ReceiverFAT[i].siteName.length());
		for (k = 0; k < 4 ; k++) {
			ReceiverFAT[i].siteName += pFAT[i].segname[k];
		}
		debug(30,"Z12::DownloadReceiverFAT: sitename is %s\n",ReceiverFAT[i].siteName.c_str());
		ReceiverFAT[i].fileSize = pFAT[i].wordcount*2;
		ReceiverFAT[i].fileStart.set_time(pFAT[i].wnbeg, pFAT[i].timebeg);
		ReceiverFAT[i].fileEnd.set_time(pFAT[i].segwn, pFAT[i].segtime);
		ReceiverFAT[i].active = ((i+1) == numFATEntries) ? TRUE : FALSE;
		FindSession(i);
		CreateFileName(i);
	}
	debug(50,"Z12::DownloadReceiverFAT: Exiting...\n");
	delete [] myFAT;
	debug(50,"Z12::DownloadReceiverFAT: Exiting\n");
	return(SUCCESS);

}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
void Z12::ReverseFATEntry(unsigned char *outFAT, unsigned char *inFAT) {

	int indexMem = 0;

	indexMem += CopyType(RFI_LONG, (void *)&outFAT[indexMem], (void *)&inFAT[indexMem],1, ASHBYTEORDER);
	indexMem += CopyType(RFI_LONG, (void *)&outFAT[indexMem], (void *)&inFAT[indexMem], 1, ASHBYTEORDER);
	indexMem += CopyType(RFI_CHAR, (void *)&outFAT[indexMem], (void *)&inFAT[indexMem], 4, ASHBYTEORDER);
	indexMem += CopyType(RFI_CHAR, (void *)&outFAT[indexMem], (void *)&inFAT[indexMem], 1, ASHBYTEORDER);
	indexMem += CopyType(RFI_CHAR, (void *)&outFAT[indexMem], (void *)&inFAT[indexMem], 1, ASHBYTEORDER);
	indexMem += CopyType(RFI_SHORT,(void *)&outFAT[indexMem], (void *)&inFAT[indexMem], 1, ASHBYTEORDER);
	indexMem += CopyType(RFI_LONG, (void *)&outFAT[indexMem], (void *)&inFAT[indexMem], 1, ASHBYTEORDER);
	indexMem += CopyType(RFI_CHAR, (void *)&outFAT[indexMem], (void *)&inFAT[indexMem], 4, ASHBYTEORDER);
	indexMem += CopyType(RFI_CHAR, (void *)&outFAT[indexMem], (void *)&inFAT[indexMem], PROJECT_LEN, ASHBYTEORDER);
	indexMem += CopyType(RFI_SHORT,(void *)&outFAT[indexMem], (void *)&inFAT[indexMem], 1, ASHBYTEORDER);
	indexMem += CopyType(RFI_LONG, (void *)&outFAT[indexMem], (void *)&inFAT[indexMem], 1, ASHBYTEORDER);
	indexMem += CopyType(RFI_USHORT,(void *)&outFAT[indexMem], (void *)&inFAT[indexMem], 1, ASHBYTEORDER);
	indexMem += CopyType(RFI_USHORT,(void *)&outFAT[indexMem], (void *)&inFAT[indexMem], 1, ASHBYTEORDER);
	indexMem += CopyType(RFI_CHAR, (void *)&outFAT[indexMem], (void *)&inFAT[indexMem], 1, ASHBYTEORDER);
	indexMem += CopyType(RFI_CHAR, (void *)&outFAT[indexMem], (void *)&inFAT[indexMem], 1, ASHBYTEORDER);
	indexMem += CopyType(RFI_CHAR, (void *)&outFAT[indexMem], (void *)&inFAT[indexMem], 1, ASHBYTEORDER);
	indexMem += CopyType(RFI_CHAR, (void *)&outFAT[indexMem], (void *)&inFAT[indexMem], 1, ASHBYTEORDER);
	indexMem += CopyType(RFI_CHAR, (void *)&outFAT[indexMem], (void *)&inFAT[indexMem], 1, ASHBYTEORDER);
	indexMem += CopyType(RFI_CHAR, (void *)&outFAT[indexMem], (void *)&inFAT[indexMem], 1, ASHBYTEORDER);
	indexMem += CopyType(RFI_SHORT,(void *)&outFAT[indexMem], (void *)&inFAT[indexMem], 1, ASHBYTEORDER);
	indexMem += CopyType(RFI_SHORT,(void *)&outFAT[indexMem], (void *)&inFAT[indexMem], 1, ASHBYTEORDER);
	indexMem += CopyType(RFI_USHORT,(void *)&outFAT[indexMem], (void *)&inFAT[indexMem], 1, ASHBYTEORDER);
}

