/* 
#    $Id: Z12.h,v 1.2 2012/02/10 00:48:57 gps Exp $
#
#    Z12.h -- Include file for the Ashtech Z12 Receiver
#
#    Copyright (C) 2000 US Geological Survey
#    Written by Keith F. Stark
#    
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifndef _Z12_H
#define _Z12_H

#include <string>

#include <gpscommon.h>
#include <AshtechDefs.h>
#include <Ashtech.h>
#include <Receiver.h>
#include <gpstime.h>

#define FILELENGTH 12
#define ASHBYTEORDER __BIG_ENDIAN

class Z12 : public Ashtech {
public:
	Z12();

// Download fat
	static int DownloadReceiverFAT(Serial *pSerial);
// Set Sessions
	static int SetReceiverSession(Serial *pSerial);

	static int VerifyReceiverSession(Serial *pSerial);

	int setFatEntry(SAshtechFATEntry myFAT);
	int setImageHeader(string rxID);
	static void ReverseFATEntry(unsigned char *outFAT, unsigned char *inFAT);
	void AddImage(Ashtech *newAshtech);
	void AddImage(unsigned char *image, int fileSize, SAshtechFATEntry *myFAT);

// Download File routines
    VIRTUAL int DownloadFileByTime(Serial *pSerial, gpstime *start, gpstime *stop);
	VIRTUAL int DownloadFileAtIndex(Serial *pSerial, int index);

	VIRTUAL int DeleteFileAtIndex(Serial *pSerial, int index);
	VIRTUAL int WriteFiles();

private:
	// File Data
	SAshtechFATEntry *pFATentry;
	RawImageHeader *pImageHead;
	unsigned char *header;
	unsigned char *fatEntry;
	static SAshtechFATEntry pFAT[MAX_RCVR_FILES];

};

#endif
