/* Copyright 1997, 1998, 1999 University Corporation for Atmospheric Research

	This software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

	Author: Louis H. Estey <lou@unavco.ucar.edu>, Doug Hunt <dhunt@ucar.edu>
	Date:   July 1999
		part of the BINEX source code release, see also:

	http://www.unavco.ucar.edu/software/binex
*/
#ifndef _BINEX_H
#define _BINEX_H

#if USE_STREAM

#include "gpscommon.h"

#define TRUE 1
#define FALSE 0
#define BINEX_MAXFLOAT 99e99

#define BINEX_F_stx_LE			(0xc2)
#define BINEX_F_stx_BE			(0xe2)
#define BINEX_FBstx_LE			(0xd2)
#define BINEX_FBstx_BE			(0xf2)
#define BINEX_FBetx_LE			(0xb4)
#define BINEX_FBetx_BE			(0xb0)

#define D_NO_DATA -9.99e-99              // No data indicator

#define BINEX_MAX_REC_SIZE		BUFSIZ

#define BINEX_SITE			(0x00000000)
#define BINEX_GNSS_NAV			(0x00000001)
#define BINEX_GNSS_DATA			(0x00000002)
#define BINEX_SITE_DATA			(0x00000003)
#define BINEX_GNSS_DATA_PROTO		(0x0000007f)
#define BAD_BINEX_CHECKSUM		(0xfffffffc) /* there will never be a BINEX record/subrecord/length with this value */
#define BAD_BINEX_READ			(0xfffffffd) /* there will never be a BINEX record/subrecord/length with this value */
#define NORMAL_BINEX_EOF		(0xfffffffe) /* there will never be a BINEX record/subrecord/length with this value */
#define NO_KNOWN_BINEX			(0xffffffff) /* there will never be a BINEX record/subrecord/length with this value */

#ifndef BIT_0
#define BIT_0				(0x01)
#endif
#define BIT_1				(0x02)
#define BIT_2				(0x04)
#define BIT_3				(0x08)
#define BIT_4				(0x10)
#define BIT_5				(0x20)
#define BIT_6				(0x40)
#define BIT_7				(0x80)

#define ERROPEN -1              // Error opening file
#define BADMODE -2              // Illegal file mode (not 'r' or 'w')
#define ALREADY_OPEN -3         // One file already open, close it first
#define NOTOPEN -4              // No file currently open
#define ENDOFFILE -5            // End of file reached
#define BADREAD -6              // Error reading file
#define BADCRC  -7              // CRC error
#define BADTRAILER -8           // Bad reversible record trailer
#define BADWRITE -9             // Error writing file
#define ERRCLOSE -10            // Error closing file
#define BADFLOATS -11           // This machine does not support IEEE floats -- Very bad!
#define BADFILENAME -12         // Bad input file name:  Must be either ' ' or '\0' terminated
#define BADSEEK     -13         // Failure to seek (beginning or end or file?)
#define NOREVERSE   -14         // Cannot seek backwards in non-reversible BINEX record
#define BADLEN      -15         // Bad length field in a BINEX record (bad UBNXI)
#define BADGFZI     -16         // Error in decoding mGFZi or mGFZI value
#define NODATAGFZI  -17         // GFZI contains 'no data' value (-0)
#define BADPARSETYPE -18        // Illegal type identifier in parse_c
#define BADTYPE      -19        // BINEX record is the wrong type/subtype

#define UNKNOWN_TRUTH -999      // Neither TRUE nor FALSE

// codes for observables
enum {  ZZ, // placeholder for "no observable"
        LC, LA,
        L1, L2, L5, L6, L7, L8,
        C1, C2, C5, C6, C7, C8,
        P1, P2,
        D1, D2, D5, D6, D7, D8,
        S1, S2, S5, S6, S7, S8,
        no_of_OBS_observables }; 

// defs for 7f_05 record
#define meters_per_GPS_L1_cycle 0.1905 // 1,575.42 MHz 
#define meters_per_GPS_L2_cycle 0.2445 // 1,227.60 MHz 
#define meters_per_GPS_L5_cycle 0.2548 // 1,176.45 MHz
#define c_WGS84 299792458.0
// fL1 = GLONASS_f_L1 + fcn*GLONASS_n_L1, where fcn=-7,-6,...,0, ...,6,7
// fL2 = GLONASS_f_L2 + fcn*GLONASS_n_L2, where fcn=-7,-6,...,0, ...,6,7
#define GLONASS_f_L1 1602    // L1 center freq in MHz
#define GLONASS_n_L1 0.5625  // delta fL1
#define GLONASS_f_L2 1246    // L2 center freq in MHz
#define GLONASS_n_L2 0.4375  // delta fL2

#define UNKNOWN_GLONASS_FCN  (-127)

// data types showing number of bytes
typedef unsigned char      uint1;
typedef signed char        sint1;
typedef signed short       esint1; // maybe just use sint2 instead?
typedef unsigned short     uint2;
typedef signed short       sint2;
typedef unsigned long      uint4;
typedef signed long        sint4;
typedef float              real4;
typedef double             real8;
typedef unsigned char      bool1;

// Constellation ID and satellite ID
typedef struct satellite_id {
  unsigned char   sys;
  unsigned char   SVid;
} SAT_ID;

// Types of constellations
enum {
        NAVSTAR_GPS,
        GLONASS,
	SBAS,
	Galileo,
	Beidou,
	QZSS,
	IRNSS,
        no_of_SV_systems
};

typedef struct binexStat {
  bool little_endian;           // This machine is little_endian
} binexStat;

typedef struct binexHandle {
  bool little_endian;           // This binex record is in little endian format
  bool bytes_reversed;          // Do bytes need to be swapped as read in? (Machine endian-ness does not match records)
  bool reversible;              // Is this record readable in reverse?
  FILE *fh;                     // The current file handle
  char mode;                    // read ('r') or write ('w')
  unsigned char hdr_bytes[8];   // The UBNXI binex record type and record length of the last record (for CRC checking)
  unsigned int n_hdr_bytes;     // The length of hdr_bytes
  unsigned int n_len_bytes;     // The length of the ubnxi length bytes
} binexHandle;

//--------------------------------------------------------------------------
// BINEX 0x7f record definitions
//--------------------------------------------------------------------------

#define LIGHTC 299792458.0 // VELOCITY OF LIGHT M/SEC
#define FREQ1 1575420000.0 // L1-CARRIER FREQUENCY   GPS 1/SEC
#define FREQ2 1227600000.0 // L2-CARRIER FREQUENCY   GPS 1/SEC
#define WVL1 (LIGHTC/FREQ1)
#define WVL2 (LIGHTC/FREQ2)
#define L1_L2 (WVL1/WVL2)

// The number of GPS satellites possible 
#define NSATS 32

// ANSI C function prototypes:

//
// High level routines for binex reading and writing
//

// Note:  These high level functions are designed to be callable from FORTRAN
// as well as C and Perl.  For this reason function return values are not used
// and most all varibles are addresses (since FORTRAN passes by reference).

// open a new file read or write (FORTRAN interface)
extern void             open_binex  (char * file_name, char * mode, binexHandle *handle, int *rc); 
extern void             close_binex (binexHandle *handle, int *rc); // close the current file 

// open a new file read or write (perl or C interface)
extern binexHandle *    new_binex  (int fd, const char * mode);
extern binexHandle *    new_binex  (char * file_name, char * mode);
extern int              free_binex (binexHandle *handle);

// Set the endian-ness of a write-mode binex handle.  Use this if you want
// to write a binex record in a non-native endian-ness (i.e. big endian from a little endian machine)
extern void             set_endian (binexHandle *handle, int *little_endian);

// Seek backward one binex record, or to the end of the file
// Seeking backwards only works for reverse-readable binex records.  Returns rc = 0 on success.
extern void             seek_reverse  (binexHandle *handle, int *rc);
extern void             seek_end      (binexHandle *handle, int *rc);

// General seek command on an open binex file handle
// Follows standard seek convention:  0 = seek to this absolute location
//                                    1 = seek offset bytes from current location
//                                    2 = seek offset bytes from end of file (usually negative)
extern void             seek_binex    (binexHandle *handle, int *offset, int *whence, int *rc);

// 'tell' command for binex debugging.  Tells the current byte offset of the binex file pointer
extern void             tell_binex    (binexHandle *handle, int *where);

// Read in the next binex header without checking body
void read_header_no_check (binexHandle *handle, unsigned int *rec_id, unsigned int *rec_len, int *reversible, int *little_endian, int *rc);

// Read in the next binex header
extern void             read_header (binexHandle *handle, unsigned int *rec_id, 
				     unsigned int *rec_len, int *reversible, int *little_endian, int *rc);  

// returns binex record in rec, verifies checksum
extern void             read_body   (binexHandle *handle, unsigned char *rec, unsigned int *len, int *rc);        

// write a binex record, including header
extern void             write_binex (binexHandle *handle, unsigned char *rec, unsigned int *len, unsigned int *type, 
				     int *reversible, unsigned int *bytes_out, int *rc);
// write a binex record, including header
unsigned char *         write_binex_buf (binexHandle *handle, unsigned char *rec, unsigned int *len, unsigned int *type, 
				     int *reversible, unsigned int *bytes_out, int *rc);
// Parse a binex record into numerical values, stored as an array of doubles.
// Note that 'vals' actually must be an array of doubles cast to a char *.
extern void             parse_c (char * string, binexHandle *handle, char *rec, int *offset, char *vals, int *rc);

// Endian-neutral routines for reading/writing data from/to the binex body
// Reads from/writes to the buffer (buf) at offset (off).
// The offset pointer, off, is automatically incremented for the next read/write
extern void		append_uint1  (binexHandle *handle, unsigned char *buf, unsigned int *off, unsigned char uc);
extern void		append_uint2  (binexHandle *handle, unsigned char *buf, unsigned int *off, unsigned short us);
extern void		append_sint2  (binexHandle *handle, unsigned char *buf, unsigned int *off, short us);
extern void		append_uint3  (binexHandle *handle, unsigned char *buf, unsigned int *off, unsigned int ul);
extern void		append_uint4  (binexHandle *handle, unsigned char *buf, unsigned int *off, unsigned int ul);
extern void		append_sint4  (binexHandle *handle, unsigned char *buf, unsigned int *off, long sl);
extern void		append_real4  (binexHandle *handle, unsigned char *buf, unsigned int *off, float f);
extern void		append_real8  (binexHandle *handle, unsigned char *buf, unsigned int *off, double d);

extern void		extract_uint1 (binexHandle *handle, unsigned char *buf, unsigned int *off, unsigned char *val);
extern void		extract_uint2 (binexHandle *handle, unsigned char *buf, unsigned int *off, unsigned short *us);
extern void		extract_sint2 (binexHandle *handle, unsigned char *buf, unsigned int *off, short *s);
extern void		extract_uint3 (binexHandle *handle, unsigned char *buf, unsigned int *off, unsigned int *ul);
extern void		extract_uint4 (binexHandle *handle, unsigned char *buf, unsigned int *off, unsigned int *ul);
extern void		extract_sint4 (binexHandle *handle, unsigned char *buf, unsigned int *off, long *l);
extern void             extract_uint5 (binexHandle *handle, unsigned char *buf, unsigned int *off, unsigned long long *ul);
extern void		extract_real4 (binexHandle *handle, unsigned char *buf, unsigned int *off, float *f);
extern void		extract_real8 (binexHandle *handle, unsigned char *buf, unsigned int *off, double *d);
extern void		extract_2cNb  (binexHandle *handle, uint1 *buf, unsigned int *off, uint1 skip, uint1 sN, bool *valid, double *d);
extern void		extract_xNb   (binexHandle *handle, uint1 *buf, unsigned int *off, uint1 skip, uint1 sN, bool sign, bool *valid, double *d);


// FORTRAN and perl friendly uint4/2 routines.
// !!! NOTICE:  Using the uint4 and uint2 routines from FORTRAN and perl can lead
//     to blindness and insanity!  Neither FORTRAN nor perl has unsigned types.
//     Any attempt to use these from FORTRAN or perl can result in a large unsigned number
//     being interpreted as negative (causing potentially subtle bugs).
extern void		append_uint2_as_int4    (binexHandle *handle, unsigned char *buf, unsigned int *off, int us);
extern void		append_uint4_as_double  (binexHandle *handle, unsigned char *buf, unsigned int *off, double ul);
extern void		extract_uint2_as_int4   (binexHandle *handle, unsigned char *buf, unsigned int *off, int *us);
extern void		extract_uint4_as_double (binexHandle *handle, unsigned char *buf, unsigned int *off, double *ul);

// Lower level utility routines 
extern int              test_little_endian (); // Test if the current system is little endian
extern unsigned int     read_ubnxi (binexHandle *handle, unsigned char *rec, unsigned char *n);
extern void		binex_append_time  (binexHandle *handle, unsigned char *message, unsigned int *offset, 
					    unsigned char sec_resolution, unsigned int minutes, double seconds);
extern void		binex_append_mGFZi  (binexHandle *handle, unsigned char *message, unsigned int *offset, double delta, bool eight_byte);
extern void		binex_append_mGFZI (binexHandle *handle, unsigned char *message, unsigned int *offset, double delta);


extern double		binex_extract_mGFZi (binexHandle *handle, unsigned char *message, 
					     unsigned int *offset, bool *valid, bool eight_byte);

extern double		binex_extract_mGFZI (binexHandle *handle, unsigned char *message, 
					     unsigned int *offset, bool *valid);

// FORTRAN/perl friendly routines
extern void		binex_extract_mgfz_littlei (binexHandle *handle, unsigned char *message, 
				  	            unsigned int *offset, int *rc, int *eight_byte, double *val);
extern void		binex_extract_mgfz_capi (binexHandle *handle, unsigned char *message, 
					         unsigned int *offset, int *rc, double *val);
extern void  		binex_extract_ubnxi (binexHandle *handle, unsigned char *message, 
	                                                 unsigned int *offset, int *rc, int *val);
extern void		binex_append_ubnxi (binexHandle *handle, unsigned char *buf, 
	                                                unsigned int *offset, int *rc, unsigned int val);

int binex_extract_time(binexHandle *handle, unsigned int record_id,  unsigned char *rec, long *minutes, unsigned short *mSec);

extern unsigned char	binex_build_SV_id (SAT_ID *system);
extern bool		binex_extract_SV_id (unsigned char id, SAT_ID *system);
extern unsigned int	uint4_to_ubnxi (binexHandle *handle, unsigned char *, unsigned int *, unsigned int);
extern unsigned int     ubnxi_to_uint4 (binexHandle *handle, unsigned char *, unsigned char);
extern unsigned int	binex_crc (unsigned char *crc, unsigned char *buff, unsigned char *mess, 
				   unsigned int *crc_offset, unsigned int buff_length, 
				   unsigned int mess_length, bool reverse);
extern void		reverse_bytes (unsigned char *rec, int n);
extern void		swap_pp (void **a, void **b);
extern void		swap_c (char *a, char *b);
extern void		swap_uc (unsigned char *a, unsigned char *b);
extern void		swap_i (int *a, int *b);
extern void		swap_d (double *a, double *b);

//---------------------------------------------------------------------------
// Binex record-specific routines
//---------------------------------------------------------------------------

// GNSS Obs record type 0x7f, subtype 0
// data is a 2D matrix: sat id x data type
// data type = ca, p1, p2, l1ca, l1p1, l2p2, l1snr, l2snr, l1rnxsnr, l2rnxsnr, rxchan, satids, error

extern int read_0x7f00 (binexHandle *handle, char *rec, int *offset, int *rc, 
						 double *time, int *nsats, int *rxfmt, char * data);
#ifdef RTCM_ENABLE
extern int read_0x7f02(binexHandle *handle, struct rtcm_t *rtcm, unsigned char *rec, size_t in_length);
extern int read_0x7f03(binexHandle *handle, struct rtcm_t *rtcm, unsigned char *rec, size_t in_length);
extern int read_0x7f05(binexHandle *handle, struct rtcm_t *rtcm, unsigned char *rec, size_t in_length);
#endif

#endif // USE_STREAM

#endif
