/* 
#    $Id: gpscommon.h,v 1.3 2016/12/01 19:25:04 gps Exp $
#
#    Copyright (C) 2000 US Geological Survey
#    Written by Keith F. Stark
#    
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _GPS_COMMON_H
#define _GPS_COMMON_H
using namespace std;

#include <stdio.h>
#include <sys/types.h>
#include <iomanip>
#include <string>

#include "../config.h"
#include "serial.h"
#include "Receiver.h"

#ifdef USE_STREAM
#include "sharcrt.h"
#endif

#ifndef __BIG_ENDIAN
#define __BIG_ENDIAN 4321
#endif

#ifndef __LITTLE_ENDIAN
#define __LITTLE_ENDIAN 1234
#endif

#ifndef BIG_ENDIAN
#define BIG_ENDIAN __BIG_ENDIAN
#endif

#ifndef LITTLE_ENDIAN
#define LITTLE_ENDIAN __LITTLE_ENDIAN
#endif

#ifdef WORDS_BIGENDIAN
#ifndef __BYTE_ORDER
# define __BYTE_ORDER __BIG_ENDIAN
#endif
#ifndef BYTE_ORDER
# define BYTE_ORDER __BIG_ENDIAN
#endif
#else
#ifndef __BYTE_ORDER
# define __BYTE_ORDER __LITTLE_ENDIAN
#endif
#ifndef BYTE_ORDER
# define BYTE_ORDER __LITTLE_ENDIAN
#endif
#endif

#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif

#define D_WARNMAJOR    0
#define D_WARN         5
#define D_WARNMINOR    30

#define D_FATALERROR   -10
#define D_ERRMAJOR     0
#define D_ERR          1
#define D_ERRMINOR     10

#define D_STD_Q   1
#define D_STD     5
#define D_STD_V   10
#define D_STD_V2  15
#define D_STD_V3  20
#define D_CHECK   30
#define D_VALUES  40
#define D_FLOW    50         
#define D_RECINFO 75
#define D_SERINFO 101
#define D_TXFR    200

#define D_USTREAM -101

#define NTRIP_PORT 2101

enum {
	RX_ASHTECH,
	RX_MICROZ,
	RX_Z12,
	RX_LEICA,
	RX_TROGUE,
	RX_TRIMBLE,
	RX_NAVCOM,
	RX_TOPCON
};

#define LOGIN	0x1
#define LOGOUT	0x2
#define GETID	0x4
#define GETFAT	0x8
#define SETSESS	0xf
#define BYINDEX	0x10
#define BYTIME	0x20

#define ACTUALSTART 0
#define ACTUALSTOP 1
#define REQUESTSTART 2
#define REQUESTSTOP 3

#define ALL LOGIN | LOGOUT | GETID | GETFAT | SETSESS | BYINDEX | BYTIME


struct ReceiverTypeStruct {
	int type;
	string name;
	string FileTemplate;
	int capability;
};

typedef struct ReceiverTypeStruct ReceiverType;

static ReceiverType rectype[] = {
	{RX_ASHTECH,  "Ashtech", "R+S+s%y.%j", ALL},
	{RX_MICROZ,   "MicroZ",  "U+S+s%y.%j", ALL},
	{RX_Z12,      "Z12",     "R+S+s%y.%j", ALL},
	{RX_LEICA,    "Leica",   "L+S+s%y.%j", ALL},
	{RX_TROGUE,   "TRogue",  "T+S+s%y.%j", ALL},
	{RX_TRIMBLE,  "Trimble", "A+S+s%y.%j", ALL},
	{RX_NAVCOM,   "Navcom",  "N+S+s%y.%j", ALL},
	{RX_TOPCON,   "Topcon",  "J+S+s%y.%j", ALL}
};

#define NUMRECEIVERTYPES 8
#define DEFAULTTYPE RX_ASHTECH

// These globals will specify the default receiver type and
// allow a subclass to override the type during autodetection
// within a Receiver class. 
extern int RECEIVERTYPE;
extern string RECEIVERSTRING;

#define SUCCESS 0
#define FAILURE 1
#define SUCCESS_NOFILE 2

#define RFI_CHAR 0
#define RFI_USHORT 1
#define RFI_SHORT 2
#define RFI_UINT 3
#define RFI_INT 4
#define RFI_ULONG 5
#define RFI_LONG 6
#define RFI_FLOAT 7
#define RFI_DOUBLE 8

#define BIT_0 1

extern string SUMFILE;
extern int DEBUGLEVEL;
extern string DEBUGFILE;
extern string DEBUGOUT;
extern int DEBUGFD;

struct CommStruct {
	Serial *pSerial;

	string port;
	long baud;
	char parity;
	int dataBits;
	int stopBits;
	int serial_format;
	int serial_flow;

//networking info
	string ipHost;
	int ipPort;

	string modemDial;
	string modemInit;
	string fwRepeaterCBEntry;
	string fwRepeater1;
	string fwRepeater2;
	string resumeDir;

	string login;
	string password;

	int forceSave;
	int maxTransferErrors;
	int timeoutMultiplier;
	int minbps;
	int minbps_secs;

	int isDC;
	int isMC;
	int isFW;
	int isIP;
	int fwUsesRepeater;
};

typedef struct CommStruct SComm;


int stringtokenize(string input, string tokens, string output[], int count);
void localmemcpy(void *out,void *in,int count);
double rnd(double in,int place);
void addToSumFile(const char *fmt, ...);
void debug(int level, const char *fmt, ...);
void InitCommStruct(SComm *c);
void InitWithArgs(SOps *pOps, SComm *pComm);
int SerialFormat(SComm *c);
int OpenConnection(SComm *pComm);
string SetFWRepeaterPath(Serial *pSerial, string repeater1, string repeater2);
void FWCleanup(Serial *pSerial);
void ReadlineUntil(Serial *pSerial, string findme);

int localmemcpy(void *out,void *in,int count, int reverse);
int CopyType(unsigned int type, void *out, void *in, int count, int byteorder = __BYTE_ORDER);
void reverseBytes(char * in, int count);
void TransferReport(ReportStruct *prs);
void TransferReportMinBps(ReportStruct *prs);
void TransferReportStart(ReportStruct *prs);
void TransferReportStop(ReportStruct *prs);
int WriteFile(const char * datadir, const char * filename, unsigned char * contents, int length, int append = FALSE);
int TestDirectory(const char * datadir, char * out = NULL);
int isCharAvail(int _fd, time_t timeout);
int is_recoverable(int error);
int make_path (string path);
int leapsecs_since_gps_epoch();
#endif
