// Copyright (C) 1999 Open Source Telecom Corporation.
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// 

#include <iostream>
#include <fstream>
#include <iomanip>
#include <string>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/ioctl.h>
#include <sys/poll.h>
#include <fcntl.h>
#include <signal.h>
#include <stdlib.h>
#include <errno.h>
#include <stdio.h>
#include <ctype.h>
#include <sys/time.h>
#include <termios.h>

#include "gpscommon.h"
#include "ttylock.h"


extern int errno;

//  ===================================================================
//  NAME:		Serial::set_speed
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
bool	Serial::set_speed(long speed, termios *t)
{
	speed_t	rate = 0;

	switch(speed)
	{
	case 115200:
#ifdef	B115200
		rate = B115200;
		break;
#endif
	case 57600:
#ifdef	B57600
		rate = B57600;
		break;
#endif
	case 38400:
#ifdef	B38400
		rate = B38400;
		break;
#endif
	case 19200:
		rate = B19200;
		break;
	case 9600:
		rate = B9600;
		break;
	case 4800:
		rate = B4800;
		break;
	case 2400:
		rate = B2400;
		break;
	case 1200:
		rate = B1200;
		break;
	case 600:
		rate = B600;
		break;
	case 300:
		rate = B300;
		break;
	case 110:
		rate = B110;
		break;
	}
	if(!rate)
		return false;
	cfsetospeed(t, rate);
	cfsetispeed(t, rate);
	return true;
}

//  ===================================================================
//  NAME:		Serial::set_flow
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
void	Serial::set_flow(int flow, termios *t)
{
	_flow = flow;
	t->c_cflag &= ~CRTSCTS;
	t->c_iflag &= ~(IXON | IXANY | IXOFF);
//	t->c_iflag &= IGNBRK;

	switch(_flow)
	{
	case SERIAL_FLOW_SOFT:
		t->c_iflag |= (IXON | IXANY | IXOFF);
		break;
	case SERIAL_FLOW_BOTH:
		t->c_iflag |= (IXON | IXANY | IXOFF);
	case SERIAL_FLOW_HARD:
		t->c_cflag |= CRTSCTS;
		break;
	case SERIAL_FLOW_XMODEM:
		t->c_iflag &= (IGNBRK|IXOFF|IXON|IXANY);
		t->c_lflag &= ~(ECHO | ICANON | ISIG);
		t->c_oflag &= 0;
//		t->c_cflag &= ~PARENB;
		t->c_cflag |= CS8;

	}
}

//  ===================================================================
//  NAME:		Serial::set_raw
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
void	Serial::set_raw(int yes)
{

	static int saved;

	if (yes) {
		if (!saved) {
			tcgetattr(_fd, &_restore);
			saved = 1;
		}
		tcgetattr(_fd, &_current);
		_current.c_iflag &= ~(IGNBRK|BRKINT|PARMRK|ISTRIP |INLCR|IGNCR|ICRNL|IXON);
		_current.c_oflag &= ~OPOST;
		_current.c_lflag &= ~(ECHO|ECHONL|ICANON|ISIG|IEXTEN);
		_current.c_cflag &= ~(CSIZE|PARENB);
		_current.c_cflag |= CS8;

//		cfmakeraw(&_current);
		tcsetattr(_fd, TCSANOW, &_current);
	} else {
		if (saved) {
			tcsetattr(_fd, TCSANOW, &_restore);
			tcgetattr(_fd, &_current);
			saved = 0;
		}
	}
}

//  ===================================================================
//  NAME:		Serial::set_format
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
void Serial::set_format(int format, termios *t)
{
	unsigned parity = 0;
	unsigned csize = CS8;
	unsigned stop = 0;

	_format = format;
	switch(_format)
	{
	case SERIAL_FORMAT_7E1:
		parity = PARENB;
		csize = CS7;
		break;
	case SERIAL_FORMAT_7O1:
		parity = PARENB | PARODD;
		csize = CS7;
		break;
	case SERIAL_FORMAT_7N1:
		csize = CS7;
		break;
	case SERIAL_FORMAT_7S2:
		csize = CS7;
		stop = CSTOPB;
		break;
	case SERIAL_FORMAT_8E1:
		parity = PARENB;
		csize = CS8;
		break;
	case SERIAL_FORMAT_8O1:
		parity = PARENB;
		csize = CS8;
		break;
	case SERIAL_FORMAT_8N1:
		csize = CS8;
		break;
	case SERIAL_FORMAT_8S2:
		csize = CS8;
		stop = CSTOPB;
	}

	t->c_cflag &= ~CSIZE;
	t->c_cflag &= ~(PARENB | PARODD);
	t->c_cflag &= ~CSTOPB;
	t->c_cflag |= (stop | parity | csize) ;
}

/*
static	void set_vtimer(int fd, termios *t, int min, int timer)
{
	if(min == t->c_cc[VMIN] && timer == t->c_cc[VTIME])
		return;

	t->c_cc[VMIN] = min;
	t->c_cc[VTIME] = timer;
	tcsetattr(fd, TCSANOW, t);
}
*/

//  ===================================================================
//  NAME:		Serial::ToggleDTR
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int Serial::ToggleDTR() {
	int ioctl_flags;
	int ret,i;
#ifndef __CYGWIN32__
	ioctl_flags = TIOCM_DTR | TIOCM_RTS;

	for (i=0; i < 5; i++) {
		if ((ret = ioctl(_fd, TIOCMBIC, &ioctl_flags)) >= 0) {
			sleep(1);
			/* Turn on flags */
			if ((ret = ioctl(_fd, TIOCMBIS, &ioctl_flags)) >= 0) break;
		}
	}
	ret = ioctl(_fd, TIOCMBIS, &ioctl_flags);
#endif
	return(ret);

}

//  ===================================================================
//  NAME:		Serial::~Serial
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
Serial::~Serial()
{
	Close();
}

//  ===================================================================
//  NAME:		Serial::Close
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int Serial::Close()
{
	debug(D_FLOW, "Serial::Close() Closing %s\n", _fname);
	if (isatty(_fd)) {
		tcdrain(_fd);
		tcflush(_fd, TCIOFLUSH);
		ToggleDTR();
		tcsetattr(_fd, TCSANOW, &_restore);
	}
	if(close(_fd) < 0) {
	    debug(D_ERR, "Error: Serial::Close() failed to close %s, errno=%d\n",
		  _fname, errno);
	    return FAILURE;
	}
	if (isatty(_fd)) {
		ttyunlock(_fname);
	}
	_fd = -1;
	_connected = FALSE;
	return(SUCCESS);
}


#ifndef	O_NONBLOCK
#define	O_NONBLOCK	O_NDELAY
#endif

//  ===================================================================
//  NAME:		Serial::Serial
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
Serial::Serial()
{
	Init();
}

//  ===================================================================
//  NAME:		Serial::Init
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
void Serial::Init()
{
	_nl = 13;
	_connected = FALSE;
	_xmodem.CRCFlag = FALSE;
	_xmodem.FirstSector = TRUE;
	_xmodem.BytesLeft = DEFBYTL;
	_xmodem.BlockLength = 0;
	_xmodem.Only1EOF = TRUE;
	EOFSeen = FALSE;
	reportstart = NULL;
	report      = NULL;
	reportstop  = NULL;
	Binary = TRUE;
	UserCanceled = FALSE;
	MaxErrors = ERRORMAX;
	TimeMult = TIME_MULT;
	ForceSave = FALSE;
	resumeDir = NULL;
	FailedXFers = 0;

	_zmodem.Rxtimeout = 3000;
	_zmodem.bytes_per_error=0;

	memset((void *) &reportstruct, 0, sizeof(ReportStruct));
}

//  ===================================================================
//  NAME:		Serial::Serial
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
Serial::Serial(const char *fname, long speed, int format, int flow)
{
	Init();

	Open(fname, speed, format, flow);
}


//  ===================================================================
//  NAME:		Serial::Open
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int Serial::Open(const char *fname, long speed, int format, int flow)
{
  	strcpy(_fname,fname);
	debug(D_FLOW, "Serial::Open() Opening %s\n", _fname);

	ttylock(_fname);

	_fd = open(_fname, O_RDWR | O_NONBLOCK | O_NOCTTY);
	if(_fd < 0)
	{
		ttyunlock(_fname);
		_connected = FALSE;
		debug(D_ERR, "Error: Serial::Open() failed to open %s, errno=%d\n",
		      _fname, errno);
		return(-1);
	}

	tcgetattr(_fd, &_restore);
	tcgetattr(_fd, &_current);

	_current.c_oflag = 0;
	_current.c_lflag = 0;
	_current.c_iflag = 0;
	_current.c_cflag = CLOCAL | CREAD | HUPCL;
	memset(&_current.c_cc, 0, sizeof(_current.c_cc));
	_current.c_cc[VMIN] = 0;

	set_speed(speed, &_current);
	set_format(format, &_current);
	set_flow(flow, &_current);
	tcsetattr(_fd, TCSANOW, &_current);
	fcntl(_fd, F_SETFL, O_RDWR);
	_connected = TRUE;
	ClearBuffers();
	return(0);
}

int Serial::Reconnect(int tries, time_t wait)
{

	debug(D_FLOW,"Serial::Reconnect: Entering...\n");
	debug(D_CHECK,"Serial::Reconnect: Attempting %d reconnects with %d seconds sleep\n",tries,wait);
	Close();
	sleep(1);
	for (int i = 0; i < tries; i++, sleep(wait)) {
		if (Open((const char *)_fname, _speed, _format, _flow) == 0) return(0);
		sleep(1);
		debug(D_ERR,"Serial::Reconnect: Attempt %2d: Unable to connect to %s\n",i+1, _fname);
	}
	debug(D_FLOW,"Serial::Reconnect: Exiting (failure)...\n");
	return(-1);
}

//  ===================================================================
//  NAME:		Serial::setFormat
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
void Serial::setFormat(int format)
{
	set_format(format, &_current);
	tcsetattr(_fd, TCSANOW, &_current);
}

//  ===================================================================
//  NAME:		Serial::setSpeed
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
void Serial::setSpeed(long speed)
{
	_speed = speed;
	set_speed(_speed, &_current);
	tcsetattr(_fd, TCSANOW, &_current);
}

//  ===================================================================
//  NAME:		Serial::getSpeed
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
long Serial::getSpeed()
{
	speed_t tmp = cfgetospeed((const termios *)&_current);

	if (B115200 == tmp) return(115200);
	if (B57600 == tmp) return(57600);
	if (B38400 == tmp) return(38400);
	if (B19200 == tmp) return(19200);
	if (B9600 == tmp) return(9600);
	if (B4800 == tmp) return(4800);
	if (B2400 == tmp) return(2400);
	if (B1200 == tmp) return(1200);
	if (B600 == tmp) return(600);
	if (B300 == tmp) return(300);
	return(-1);

}

//  ===================================================================
//  NAME:		Serial::setFlowControl
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
void Serial::setFlowControl(int flow)
{
	set_flow(flow, &_current);
	tcsetattr(_fd, TCSANOW, &_current);
}

//  ===================================================================
//  NAME:		Serial::isCharAvailable
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
// return time remaining in timer
int Serial::isCharAvailable(timeout_t timeout)
{
#if 1

	return ( isCharAvail(_fd,timeout*TimeMult)/TimeMult);
#else

	int	rc;
	struct timeval tv;
	fd_set rfds;
	struct itimerval itv, cur;

	timeout_t mytimeout = timeout * TimeMult;

	tv.tv_sec = 0;
	tv.tv_usec = 0;

	FD_ZERO(&rfds);
	FD_SET(_fd, &rfds);
	rc = select(_fd+1, &rfds, NULL, NULL, &tv);
	FD_ZERO(&rfds);
	if (rc > 0) return(timeout);
	tv.tv_sec = mytimeout / 1000;
	tv.tv_usec = (mytimeout % 1000) * 1000;

	signal(SIGALRM, SIG_IGN);
	itv.it_interval.tv_sec = 0;
	itv.it_interval.tv_usec = 0;
	itv.it_value.tv_sec = tv.tv_sec;
	itv.it_value.tv_usec = tv.tv_usec;
	setitimer(ITIMER_REAL, &itv, NULL);

	FD_ZERO(&rfds);
	FD_SET(_fd, &rfds);
	rc = select(_fd+1, &rfds, NULL, NULL, &tv);
	FD_ZERO(&rfds);

	itv.it_interval.tv_sec = 0;
	itv.it_interval.tv_usec = 0;
	itv.it_value.tv_sec = 0;
	itv.it_value.tv_usec = 0;
	setitimer(ITIMER_REAL, &itv, &cur);

	if (rc > 0) {
		return((int) (((cur.it_value.tv_sec * 1000) + (cur.it_value.tv_usec / 1000))/TimeMult));
	} else {
		return(0);
	}
#endif
}

//  ===================================================================
//  NAME:		Serial::Inkey
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int Serial::Inkey(timeout_t timeout)
{
	unsigned char buf;
	int rc;

	if (isCharAvailable(timeout)) {
		rc = _read(_fd, &buf, 1, 0);
		if (rc > 0) {
			debug(1000,"%X ",buf);
			return (int)buf;
		}
	}
	return (int)-1;
}

//  ===================================================================
//  NAME:		Serial::WaitFor
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int Serial::WaitFor(int timeout, const char **patterns, int numpatterns)
{
	int key;
	int timer;
	char *buffer;
	int pos;

	timer = (timeout < 1000) ? 1000 : timeout;

	pos = 1;
	buffer = (char *) new char[pos];
	do {
		timer = isCharAvailable(timer);
		if (timer > 0) {
			pos++;
			key = Inkey(1);
			buffer = (char *) realloc(buffer, pos);
			buffer[pos-2] = key;
			buffer[pos-1] = '\0';
			for (int i = 0; i < numpatterns; i++) {
				if (strstr(buffer,patterns[i]) != NULL) {
					delete buffer;
					return(i);
				}
			}
		}
	} while ((pos <= 1024) && (timer > 0));
	return(-1);
}

//  ===================================================================
//  NAME:		Serial::WaitFor
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int Serial::WaitFor(int timeout, unsigned char pattern)
{
	int key;
	int timer;
	unsigned char tmp;

	timer = (timeout < 1000) ? 1000 : timeout;

	do {
		timer = isCharAvailable(timer);
		if (timer > 0) {
			key = Inkey(1);
			tmp = (unsigned char) key;
		}
	} while ((tmp != pattern) && timer > 0);
	if (timer > 0) return(0);
	return(1);
}

//  ===================================================================
//  NAME:		Serial::Read
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int Serial::Read(void *buf, size_t max, timeout_t timeout)
{
	size_t len = 0;
	int timer;
	int rc = 0;
	unsigned char *mybuf;

	mybuf = (unsigned char *) buf;

	timer = timeout;
	while(len < max)
	{
		timer = isCharAvailable(timer);
		debug(D_SERINFO+5,"Serial::Read: isCharAvailable timer return is %d\n",timer);
		if (timer == 0) break;

//		rc = _read(_fd, mybuf, 1, 0);
		rc = _read(_fd, mybuf, max - len, 0);
		debug(D_SERINFO+5,"Serial::Read: Read %d chars from _read\n",rc);
		//error
		if (rc == -1) break;
		//end of file
		if (rc == 0) break;

		len += rc;
		mybuf += rc;
//		len++; mybuf++;
	}
	if (rc == -1) return(-1);
	*mybuf = 0;
	return(len);
}

//  ===================================================================
//  NAME:		Serial::Readline
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int Serial::Readline(char *buf, size_t max, timeout_t timeout)
{
	size_t len = 0;
	int timer;
	unsigned char mybuf;
	int rc;

	timer = timeout;
	while(len < max)
	{
		timer = isCharAvailable(timer);
		if (timer == 0) break;

		rc = _read(_fd, &mybuf, 1, 0);
		if (rc == -1) break;
		debug(1000,"%X ",mybuf);

		if ((int) mybuf == 10) {
			continue;
		}
		if((int)mybuf == (int)_nl) {
			buf[len] = '\n';
			len++;
			break;
		}
		buf[len] = mybuf;
		len++;
	}
	buf[len] = 0;
	return len;
}

ssize_t Serial::_read(int d, void *buf, size_t nbytes, int flags)
{
        return(read(d, buf, nbytes));
}

//  ===================================================================
//  NAME:		Serial::ClearRXBuffer
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
void Serial::ClearRXBuffer()
{
//	int rc;
	if (isConnected()) {
//		while ((rc = Inkey(100)) != -1);
		tcflush(_fd, TCIFLUSH);
	}
}

//  ===================================================================
//  NAME:		Serial::ClearTXBuffer
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
void Serial::ClearTXBuffer()
{
	if (isConnected()) tcflush(_fd, TCOFLUSH);
}

//  ===================================================================
//  NAME:		Serial::ClearBuffers
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
void Serial::ClearBuffers()
{
	ClearRXBuffer();
	ClearTXBuffer();
}


//  ===================================================================
//  NAME:       interrupt_transfer
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
//bool Serial::UserCanceled;
void interrupt_transfer(int sig)
{

	debug(50,"interrupt_transfer: Entering...\n");
	debug(25,"interrupt_transfer: setting User Canceled\n");
	Serial::UserCanceled = TRUE;
	debug(50,"interrupt_transfer: Exiting...\n");
}

//  ===================================================================
//  NAME:       setResumeDir
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================

void Serial::setResumeDir(const char *dir)
{
	debug(50,"Serial::setResumeDir: Entering...\n");
	debug(100,"Serial::setResumeDir: passed %s with length %d\n",dir,strlen(dir));
	if (strlen(dir) == 0) {
		resumeDir = NULL;
		return;
	}       
	if (dir == NULL) {
		resumeDir = NULL;
		return;
	}
	resumeDir = new char [strlen(dir) + 2];
	strcpy(resumeDir, dir);
	if (resumeDir[strlen(resumeDir)-1] != '/') strcat(resumeDir,"/");
	debug(50,"Serial::setResumeDir: Exiting...\n");
}

//  ===================================================================
//  NAME:       setMinBPS
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
void Serial::setMinBPS(int bps, int secs)
{
	reportstruct.minbps = bps;
	reportstruct.minbps_secs = secs;
	debug(150,"reportstruct.minbps = %d, reportstruct.minbps_secs = %d\n",reportstruct.minbps, reportstruct.minbps_secs);
}



// need to remove from zmodem and write here
//  ===================================================================
//  NAME:       setResumeFileName
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int Serial::setResumeFileName(const char *x, int y, int z)
{
	if (resumeFileName != NULL) {
		delete resumeFileName;
		resumeFileName = NULL;
	}
	resumeFileName = new char [strlen(resumeDir) + 128];
	return(sprintf(resumeFileName, RESUMEFILE, resumeDir, x, y, z));
}
//
//  ===================================================================
//  NAME:       setResumeFileName
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int Serial::writeResumeFile(deque<unsigned char> *File)
{
	ofstream pfd;
	unsigned char *myFile;
	int fileSize, i;
	struct stat sb;
	// copy vector of file to unsigned char to return
	fileSize = File->size();
	myFile = new unsigned char[fileSize];
	for (i = 0; i < fileSize; i++) {
		myFile[i] = (*File)[i];
	}

	try {

		if (fileSize == 0) throw "Serial::zmodemreceive: No data to save for resume\n";
		if (! (Binary && tryResume())) throw "Serial::zmodemreceive: Not allowed to resume\n";

		// RESUME: open file and write contents of File to it
		debug(100, "Trying to write resume file %s\n",resumeFileName);
		pfd.open(resumeFileName, ios::binary | ios::out | ios::trunc);
		if (!pfd.is_open()) throw "Serial::zmodemreceive: Cannot open file\n";

		pfd.write((const char *)myFile, fileSize);
		if (pfd.fail()) {
			pfd.close();
			throw "Serial::zmodemreceive: Could not write contents to file\n";
		}
		debug(100, "Successfully wrote resume file %s with %d bytes\n",resumeFileName,fileSize);
		pfd.close();
       	}
	catch (const char *pmessage) {
		debug(100,pmessage);
		if (stat(resumeFileName, &sb) == 0) unlink(resumeFileName);
		return(FAILURE);
	}
	return(SUCCESS);
}
//
//  ===================================================================
//  NAME:       readResumeFile
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int Serial::readResumeFile()
{
	ifstream pfd;
	char buffer[1024];
	int bytes_read;
	int totalbytes = 0;
	struct stat sb;
	debug(150,"Serial::readResumeFile: Entering\n");
	try {
// shouldn't be needed?
//		if (! allowResume) throw "Serial::readResumeFile: Not allowed to resume\n";

		// RESUME: stat file, if exists read in to buffer
		debug(100, "Serial::readResumeFile: Looking for resume file %s -- ",resumeFileName);
		if (stat(resumeFileName, &sb) != 0) throw "file does NOT exist\n";
		debug(100, "file exists\n");

		// RESUME: open file and read in 1024k blocks
		// RESUME: ignore final non 1024k block
		pfd.open(resumeFileName);
		if (!pfd.is_open()) throw "Serial::readResumeFile: Cannot open file\n";

		while (pfd.good()) {
			pfd.read((char *)buffer, 1024);
			bytes_read=pfd.gcount();

			debug(150,"Serial::readResumeFile: Read %d from file\n",bytes_read);
			if (pfd.eof()) break;
			if (pfd.fail()) throw "Error reading file\n";

			if (bytes_read != 1024) break;

			// add data to the zmodem transfer
			AddSectorToBuffer( buffer, bytes_read);
			totalbytes += bytes_read;
			debug(150, "Serial::readResumeFile: Added %d bytes from resume file %s\n",bytes_read, resumeFileName);
		}
		pfd.close();
	}
	catch (const char *pmessage) {
		debug(100,pmessage);
		return(0);
	}
	debug(150,"Serial::readResumeFile: Exiting (%d bytes read)\n",totalbytes);
	return totalbytes;
}

//  ===================================================================
//  NAME:       initReportStruct && setReports
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
void Serial::initReportStruct() 
{
	int a, b;
	a = reportstruct.minbps;
	b = reportstruct.minbps_secs;
	memset((void *) &reportstruct, 0, sizeof(ReportStruct));
	reportstruct.minbps      = a;
	reportstruct.minbps_secs = b;
}

void Serial::setReports(void (*a)(ReportStruct *), void (*b)(ReportStruct *), void (*c)(ReportStruct *))
{
	reportstart = a;
	report      = b;
	reportstop  = c;
}

