/* 
#    $Id: sharcrt.h,v 1.6 2016/12/01 19:26:56 gps Exp $
#
#    sharcrt.h -- Header file for real time streaming using threads
#
#    Copyright (C) 2000 US Geological Survey
#    Written by Keith F. Stark
#    
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _SHARCRT_H
#define _SHARCRT_H

#define SHARCRT

/* system includes */
#include <pthread.h>
#include <vector>

using namespace std;

#include "gpscommon.h"

#if USE_STREAM

#include "sharc.h"
#include "StreamType.h"

#define MAXTHREADS 16
#define MAXRECONNECT 10

#define LEAP_SECONDS 18

// different writer types
#define WRITE_UNKNOWN	0
#define WRITE_FILE	1
#define WRITE_NET	2
#define WRITE_PIPE	4
#define WRITE_EXEC	8
#define WRITE_NTRIP	16

#define DBENMAXTYPES 5

#if 0
#define DATATYPE_UNKNOWN 0
#define DATATYPE_DBEN	1
#define DATATYPE_BINEX	2
#define DATATYPE_RAW 	3
#define DATATYPE_RTCM 	4
#define DATATYPE_BINEX2	5
#define DATATYPE_RTCM22	6
#define DATATYPE_RTCM24	7
#define DATATYPE_RTCM30	8
#endif
enum {
DATATYPE_UNKNOWN,
DATATYPE_DBEN,
DATATYPE_BINEX,
DATATYPE_RAW,
DATATYPE_BINEX2,
DATATYPE_RTCM,
DATATYPE_RTCM22,
DATATYPE_RTCM24,
DATATYPE_RTCM30,
};

#define DBEN_PACKET -1
#define MBEN_PACKET -2
#define PBEN_PACKET -3
#define BINEX_PACKET -4

#define PACKET_FOUND 1
#define INCOMPLETE_PACKET 0

#ifdef RTCM_ENABLE
#define RTCM_ANTFREQ 10
#endif


typedef struct read_thread_info {
	int filedes;
	int streamType;
#ifdef RTCM_ENABLE
	bool translateRTCM;
	string antennaFile;
#endif
} read_thread_info_t;

typedef struct data_record {
// BINEX related values
	unsigned int record_len;
	unsigned int record_id;
	int reversible;
	int little_endian;

// Raw data record
	unsigned char *record;
	vector<unsigned char> vrecord;
#ifdef RTCM_ENABLE
	unsigned char *rtcm;
	vector<unsigned char> vrtcm;
	unsigned int rtcm_len;

	unsigned char *rtcm3;
	vector<unsigned char> vrtcm3;
	unsigned int rtcm3_len;
#endif
	long gpsMinutes;
	unsigned short gpsm_Seconds;
	bool timeknown;
	unsigned short handled;	// set to 0, writer increments when done
	StreamType *mystream;
} data_record_t;

typedef struct input_buffer {
	vector<unsigned char> inbuffer;
	vector<unsigned char> cur_packet;
	unsigned int inbuf_pos;
} input_buffer_t;

// Globals for thread creation
typedef struct writerinfo {
	pthread_t thread;
	int output_type;
	int streamType;
	unsigned long sample_rate;
	bool active;
	unsigned int writernum;
	int filedes[2];
	int stats;
	int leapsec;
// file stuff
	string dir_template;
	string file_template;
	int	seconds_in_file;
// pipe?
	bool pipe;
// network stuff
	string hostname;
	int port;
// network ntrip stuff
	string mount;
	string user;
	string password;
// exec stuff
	string program_name;
// translation stuff
#ifdef RTCM_ENABLE
	bool outputRTCM;
	int RTCMVersion;
#endif
} writerinfo_t;


// Structure
#define MAXCACHESIZE 100

int cache_init();
int cache_add(data_record_t *rec);
int cache_removehandled();
int cache_getcopy(unsigned int writernum, data_record_t *rec);
int sharcStream(SOps *pOps);

/* local function declarations */
void * serialThread(void *argin);
void * readDataThread(void *argin);
void * writeDataThread(void *argin);
void * writeNetworkThread(void *argin);
int open_net(string hostname, unsigned int port);
int ntrip_login(int fd, string mount, string password);
int ntrip_mount(Serial *pSerial, string mount, string user, string password);

void * shutdownThread(void *argin);
void createWriteThread(int streamType, int i);
void createReadThread(int streamType, void *argin);
void CloseStreamAndExit(Serial *pSerial, SOps *pOps);
void computeLatency(time_t rec_time_sec, unsigned short milli_sec, time_t *latency, int *numrecs, time_t *now);
bool checkSample(writerinfo_t *info, data_record_t *record);
int read_rawStream(int fd, void *buf, size_t max, unsigned int * record_len, timeout_t timeout);
int getSaveFile(void **fh, string *dir, string *filename, time_t rec_time, string dir_template, string file_template, bool BINEX);
void all_exit();
void last_breath(void *message);

//extern int numwriters;
//extern writerinfo_t *writerThreads[MAXTHREADS];

#endif

#endif
