/***************************************************************************
 *            socket.h
 *
 *  Wed Jun 39 13:7:13 2005
 *  Copyright  2005  Daniel Cartoon
 *  Email dcartoon@usgs.gov
 ****************************************************************************/
/* 	Modified by Daniel Cartoon, 2005
 * 
 *  Copyright (C) 1999-2002 Open Source Telecom Corporation.
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <iostream>
#include <iomanip>

#include <string>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/ioctl.h>
#include <sys/poll.h>
#include <fcntl.h>
#include <signal.h>
#include <stdlib.h>
#include <errno.h>
#include <stdio.h>
#include <ctype.h>
#include <sys/time.h>
#include <termios.h>

#include <socket.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/tcp.h>

#include <arpa/inet.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

#include "gpscommon.h"

#define MYTIMEOUT 5000

#define CCXX_USE_POLL 

Socket::Socket(){
	Init();
	_xmodem.Only1EOF = FALSE;
}

Socket::~Socket(){
	Close();
}

//  ===================================================================
//  NAME: determineIP
//
//  PURPOSE: Split port into IP Address and Port if possible
//
//  RETURNS: Port if completed
//			 -1 if not possible(dealing with non IP connection or mangled
//				address)
//				Sets (char *)address to IP Address
//  HISTORY: (HVO)Added for TCP/IP support
//
//  ===================================================================

int Socket::determineIP(string *address){
	string myPort = *address;
	int splitPoint = myPort.find_first_of(':');
	string temp;

	*address = (myPort.substr(0,splitPoint));
	temp = myPort.substr(splitPoint+1,myPort.length()-splitPoint);
	return(atoi(temp.c_str()));
}

/**
* Get the status of pending operations.  This can be used to
* examine if input or output is waiting, or if an error has
* occured on the descriptor.
*
* @return true if ready, false on timeout.
* @param pend ready check to perform.
* @param timeout in milliseconds, inf. if not specified.
*/

bool Socket::isPending(Pending pending, timeout_t timeout){
	int status;
#ifdef CCXX_USE_POLL
	struct pollfd pfd;

	pfd.fd = _fd;
	pfd.revents = 0;
	switch(pending)
	{
	case SOCKET_PENDING_INPUT:
			pfd.events = POLLIN;
			break;
	case SOCKET_PENDING_OUTPUT:
			pfd.events = POLLOUT;
			break;
	case SOCKET_PENDING_ERROR:
			pfd.events = POLLERR | POLLHUP;
			break;
	}

	status = 0;
	while(status < 1) {

		if (timeout == TIMEOUT_INF) {
			status = poll(&pfd, 1, -1);
		} else {
			status = poll(&pfd, 1, timeout);
		}
		if (status < 1) {
			// don't stop polling because of a simple
			// signal :)
			if(status == -1 && errno == EINTR) continue;
			return false;
		}
	}

	if(pfd.revents & pfd.events){
		return true;
	}
	return false;
#else
	struct timeval tv;
	struct timeval *tvp = &tv;
	fd_set grp;

	if(timeout == TIMEOUT_INF) {
		tvp = NULL;
	} else {
		tv.tv_usec = (timeout % 1000) * 1000;
		tv.tv_sec = timeout / 1000;
	}

	FD_ZERO(&grp);
	FD_SET(_fd, &grp);
	switch(pending)
	{
	case SOCKET_PENDING_INPUT:
		status = select(_fd + 1, &grp, NULL, NULL, tvp);
		break;
	case SOCKET_PENDING_OUTPUT:
		status = select(_fd + 1, NULL, &grp, NULL, tvp);
		break;
	case SOCKET_PENDING_ERROR:
		status = select(_fd + 1, NULL, NULL, &grp, tvp);
		break;
	}
	if(status < 1)
		return false;
	if(FD_ISSET(_fd, &grp))
		return true;
	return false;
#endif
}

//  ===================================================================
//  NAME: Open
//
//  PURPOSE: Open a socket using given IP address/port
//
//  RETURNS: 
//
//  HISTORY: (HVO)Added for TCP/IP support
//
//  ===================================================================

int Socket::Open(const char *fname, long speed, int format, int flow){

	int port;
	string address = fname;
	struct hostent *h;
	struct sockaddr_in host_addr;

	strcpy(_fname,fname);
	_speed = speed;
	_format = format;
	_flow = flow;

	debug(150,"Fname is: %s\n",fname);
	port = determineIP(&address);

	//(HVO)Need to add hostname resolution
	fname=address.c_str();
	debug(40,"IP: %s Port: %d\n",fname,port);

	if ((h=gethostbyname(fname)) == NULL) {  // get the host info
		debug(10,"Socket::Open: Error from gethostbyname\n");
		return(FAILURE);
	} 

	debug(50,"Socket::Open: IP is: %s\n",inet_ntoa(*((struct in_addr *)h->h_addr)));

	int yes=1;
	if ((_fd = socket(AF_INET,SOCK_STREAM, 0)) == -1){
		debug(10,"Socket::Open: Error from socket()\n");
		return(FAILURE);
	}
	if (setsockopt(_fd,SOL_SOCKET,SO_REUSEADDR,&yes,sizeof(int)) == -1) {
		debug(10,"Socket::Open: Error from setsockopt()\n");
		return(FAILURE);
	} 
/*
	int rcvsize = 50;
	if (setsockopt(_fd,SOL_SOCKET,SO_RCVBUF,&rcvsize,sizeof(int)) == -1) {
		debug(10,"Socket::Open: Error from setsockopt()\n");
		return(FAILURE);
	} 
	int sndsize = 0;
	if (setsockopt(_fd,SOL_SOCKET,SO_SNDBUF,&sndsize,sizeof(int)) == -1) {
		debug(10,"Socket::Open: Error from setsockopt()\n");
		return(FAILURE);
	} 
	if (setsockopt(_fd,IPPROTO_TCP,TCP_NODELAY,&yes,sizeof(int)) == -1) {
		debug(10,"Socket::Open: Error from setsockopt()\n");
		return(FAILURE);
	} 
*/

	host_addr.sin_family = AF_INET;
	host_addr.sin_port = htons(port);

	//host_addr.sin_addr.s_addr = inet_addr(address.c_str());
	host_addr.sin_addr = *((struct in_addr *)h->h_addr); 
	memset(&(host_addr.sin_zero),'\0',8);

	debug(50,"Socket::Open: Port: %d\n",port);

	if (connect(_fd, (struct sockaddr *)&host_addr,sizeof(struct sockaddr)) == -1) {
		debug(10,"Socket::Open: Error from connect()\n");
		Close();
		return(FAILURE);
	} 

	_connected = TRUE;
	ClearBuffers();
	return(SUCCESS);

}

//  ===================================================================
//  NAME: Read
//
//  PURPOSE: Read into *buf(of size max) from open connection, timer 
//					 is ignored at the moment
//
//  RETURNS: Number of bytes read
//
//  HISTORY: (HVO)Added for TCP/IP support
//
//  ===================================================================

ssize_t Socket::_read(int d, void *buf, size_t nbytes, int flags)
{

	if (! isConnected() ) {
		throw(SerialFail);
	}
	flags |= MSG_DONTWAIT;
	size_t i = recv(d, buf, nbytes, flags);
	debug(1000,"Return from recv = %d\n",i);
	if ((i <= 0) && (! is_recoverable(errno))) {
		debug(D_ERR,"Socket::_read: socket disconnected, closing local side, throwing SerialFail\n");
 		debug(D_ERR,"Socket::_read:errno=%d, %s\n",  errno, strerror(errno));
		throw(SerialFail);
	}
	return(i);
}

int Socket::ToggleDTR()
{
	return(0);
}

int Socket::Write(char *buf)
{
	return(Write(buf,strlen(buf)));	 
}

int Socket::Write(void *buf, size_t len)
{
	return(Write((unsigned char *)buf,len,MYTIMEOUT));
}

int Socket::Write(unsigned char *buf, size_t len)
{
	return(Write(buf,len,MYTIMEOUT));
}

int Socket::Write(unsigned char *buf, size_t len,timeout_t timeout) 
{
	if (len < 1)
		return (0);

	ssize_t nstat;
	unsigned char *Slide = buf;
	if (! isConnected() ) {
		throw(SerialFail);
	}

	while (true) {

		/* make sure all data is written before returning */
		if (timeout) {
			if (!isPending (pendingOutput, timeout)) {
				debug(100,"Socket::Write: Output error\n");
				throw(SerialFail);
			}
		}

		nstat =::send (_fd, Slide, _IOLEN64 len, 0);

		if ((nstat <= 0) && (! is_recoverable(errno))) {
			debug(D_ERR,"Socket::Write: Unrecoverable write error, closing local side\n");
			throw(SerialFail);
		}

		if (nstat == 0){
			debug(100,"Socket::Write Done writing\n");
			break;
		}
		len -= nstat;
		Slide += len;

	  	if (len <= 0)
			break;
	}

	return (nstat);

}
