/***************************************************************************
 *            socket.h
 *
 *  Wed Jun 29 13:7:13 2005
 *  Copyright  2005  Daniel Cartoon
 *  Email dcartoon@usgs.gov
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 33, Boston, MA 02111-1307, USA.
 */
 
#ifndef _SOCKET_H
#define _SOCKET_H

//#pragma implementation "commbase.h"

#include <iostream>
#include <iomanip>

#include <string>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/ioctl.h>
#include <sys/poll.h>
#include <fcntl.h>
#include <signal.h>
#include <stdlib.h>
#include <errno.h>
#include <stdio.h>
#include <ctype.h>
#include <sys/time.h>
#include <termios.h>
#include <ttylock.h>
#include <serial.h>

/**
 * A modification to the Common C++ Socket files to be interchangeable
 * with the Serial class
 * 
 * @author Daniel Cartoon <dcartoon@usgs.gov> modifying code by:
 * @author David Sugar <dyfet@ostel.com>
 * @short Socket access.
 */
 
#define TIMEOUT_INF ~((timeout_t) 0)
#if defined(WIN32) && !defined(__CYGWIN32__)
#include <winsock2.h>

typedef int socklen_t;
#else
#define INVALID_SOCKET	-1
typedef int SOCKET;
#endif

enum sockerror_t
{
	SOCKET_SUCCESS = 0,
	SOCKET_CREATE_FAILED,
	SOCKET_COPY_FAILED,
	SOCKET_INPUT_ERROR,
	SOCKET_INPUT_INTERRUPT,
	SOCKET_RESOURCE_FAILURE,
	SOCKET_OUTPUT_ERROR,
	SOCKET_OUTPUT_INTERRUPT,
	SOCKET_NOT_CONNECTED,
	SOCKET_CONNECT_REFUSED,
	SOCKET_CONNECT_REJECTED,
	SOCKET_CONNECT_TIMEOUT,
	SOCKET_CONNECT_FAILED,
	SOCKET_CONNECT_INVALID,
	SOCKET_CONNECT_BUSY,
	SOCKET_CONNECT_NOROUTE,
	SOCKET_BINDING_FAILED,
	SOCKET_BROADCAST_DENIED,
	SOCKET_ROUTING_DENIED,
	SOCKET_KEEPALIVE_DENIED,
	SOCKET_SERVICE_DENIED,
	SOCKET_SERVICE_UNAVAILABLE,
	SOCKET_MULTICAST_DISABLED,
	SOCKET_TIMEOUT_ERROR,
	SOCKET_NODELAY_ERROR,
	SOCKET_EXTENDED_ERROR
};

enum sockpend_t
{
	SOCKET_PENDING_INPUT,
	SOCKET_PENDING_OUTPUT,
	SOCKET_PENDING_ERROR
};

enum Pending
{
	pendingInput,
	pendingOutput,
	pendingError
};

#ifndef	_IOLEN64
#define	_IOLEN64
#endif

class Socket : public Serial {
	private:
		
	public:
	
		Socket();
		virtual ~Socket();

		int determineIP(string *address);
			
		bool isPending(Pending pending, timeout_t timeout = TIMEOUT_INF);
		
		int Open(const char *fname, long speed, int format, int flow);

		ssize_t _read(int d, void *buf, size_t nbytes, int flags);

		int ToggleDTR();

		int Write(char *buf);
		int Write(void *buf, size_t len);
		int Write(unsigned char *buf, size_t len);
		int Write(unsigned char *buf, size_t len,timeout_t timeout);


};

#endif
