/* 
#    $Id: utils.cpp,v 1.5 2018/02/23 04:33:16 gps Exp $
#
#    Copyright (C) 2000 US Geological Survey
#    Written by Keith F. Stark
#    
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <iostream>
#include <iomanip>
#include <fstream>
#include <vector>
#include <stdarg.h>
#include <string>
#include <math.h>
#include <signal.h>
#include <string.h>
#include <math.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <time.h>
#include <pthread.h>

#include "gpscommon.h"
#include "socket.h"
#ifdef USE_LEAPSECONDS_FILE
#include <qlib2.h>
#endif

extern int errno;
static pthread_mutex_t log_mutex = (pthread_mutex_t)PTHREAD_MUTEX_INITIALIZER;

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int stringtokenize(string input, string tokens, string output[], int count) {

	int numtokens = 0;
	size_t tmp;
	while ((tmp = input.find_first_of(tokens)) != string::npos) {
		if (numtokens >= count) return(numtokens);

		output[numtokens++] = input.substr(0,tmp);
		input.erase(0, input.find_first_not_of(tokens,tmp+1));
	}
	output[numtokens++] = input.substr(0); // last token
	return(numtokens);
}


//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int localmemcpy(void *out,void *in,int count, int byteorder) {
	if (byteorder != __BYTE_ORDER) {
		int i;
		char *str, *str2;

		str = (char *) in;
		str2 = (char *) out;
		for (i = 0;i < count; i++) {
			str2[i] = str[(count-1) - i];
		}
	} else {
		memcpy(out,in,count);
	}
	return(count);
}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int CopyType(unsigned int type, void *out, void *in, int count, int byteorder) {

	int i = 0;
	char *pIn = (char *)in;
	char *pOut = (char *)out;
	switch(type) {
		case RFI_CHAR:
			count *= sizeof(char);
			break;
		case RFI_USHORT:
			count *= sizeof(unsigned short);
			break;
		case RFI_SHORT:
			count *= sizeof(short);
			break;
		case RFI_UINT:
			count *= sizeof(unsigned int);
			break;
		case RFI_INT:
			count *= sizeof(int);
			break;
		case RFI_ULONG:
			count *= sizeof(unsigned long);
			break;
		case RFI_LONG:
			count *= sizeof(long);
			break;
		case RFI_FLOAT:
			count *= sizeof(float);
			break;
		case RFI_DOUBLE:
			count *= sizeof(double);
			break;
	}
	while (i < count) {
		pIn = (char *)in + i;
		pOut = (char *)out + i;
		switch(type) {
			case RFI_CHAR:
				i += localmemcpy(pOut, pIn, sizeof(char),byteorder);
				break;
			case RFI_USHORT:
				i += localmemcpy(pOut, pIn, sizeof(unsigned short), byteorder);
				break;
			case RFI_SHORT:
				i += localmemcpy(pOut, pIn, sizeof(short), byteorder);
				break;
			case RFI_UINT:
				i += localmemcpy(pOut, pIn, sizeof(unsigned int), byteorder);
				break;
			case RFI_INT:
				i += localmemcpy(pOut, pIn, sizeof(int), byteorder);
				break;
			case RFI_ULONG:
				i += localmemcpy(pOut, pIn, sizeof(unsigned long), byteorder);
				break;
			case RFI_LONG:
				i += localmemcpy(pOut, pIn, sizeof(long), byteorder);
				break;
			case RFI_FLOAT:
				i += localmemcpy(pOut, pIn, sizeof(float), byteorder);
				break;
			case RFI_DOUBLE:
				i += localmemcpy(pOut, pIn, sizeof(double), byteorder);
				break;
		}
	}
	return(i);
}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
double rnd(double in,int place)
{
	double thepow;
	double themod;

	thepow = pow(10,(double)place);
	themod = fmod(in,thepow);
	if (in < 0.0) thepow *= -1;
	if (fabs(themod) < (0.5 * fabs(thepow)))
		return(in - themod);
	else
		return(in - themod + thepow);
}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
extern Serial *pSerial;
int OpenConnection(SComm *pComm)
{
	string Send;
	int ret;

	debug(50,"OpenConnection: Entering...\n");
	debug(25,"OpenConnection: Opening port = %s, baud = %d\n",pComm->port.c_str(), pComm->baud);
	pComm->serial_format = SerialFormat(pComm);
	if (pComm->serial_format < 0) {
		debug(0,"The serial format is incorrect\n");
		return(FAILURE);
	}
	ret = pComm->pSerial->Open(pComm->port.c_str(), pComm->baud, pComm->serial_format, pComm->serial_flow);
//	if (errno == EACCES) {
//		debug(0,"You do not have permission to open %s\n",pComm->port.c_str());
//		return(FAILURE);
//	}
	if (!pComm->pSerial->isConnected()) {
		debug(1,"OpenConnection:Cannot connect to port %s at %d baud.\n",pComm->port.c_str(),pComm->baud);
		return(FAILURE);
	}

	if (pComm->isDC) {
		pComm->pSerial->ClearBuffers();
		addToSumFile("Communications: OK\n");
		debug(1,"OpenConnection:Direct connection achieved...\n");

	} else if (pComm->isMC || pComm->isFW) {
		debug(10,"OpenConnection:Clearing Serial buffer before doing anything\n");
		pComm->pSerial->ClearBuffers();
		while ((ret = pComm->pSerial->Inkey(100)) != -1) {
			debug(20,"%c",(char) ret);
		}
		pComm->pSerial->setFlowControl(SERIAL_FLOW_SOFT);
		if (pComm->modemInit.length() > 0) {
			if (pComm->modemInit[0] != ',') {
				Send = pComm->modemInit + "\r";
				pComm->pSerial->Write((void *) Send.c_str(), Send.length());
				debug(10,"Sending: %s\n",pComm->modemInit.c_str());
				debug(10,"OpenConnection:Waiting for return from initialization\n");
				sleep(3);
				while ((ret = pComm->pSerial->Inkey(100)) != -1) {
					debug(20,"%c",(char) ret);
				}
			}
		}

		if (pComm->isFW) {
			if (pComm->fwUsesRepeater) {
				if (pComm->fwRepeaterCBEntry == "") {
					pComm->fwRepeaterCBEntry = SetFWRepeaterPath( pSerial, 
													pComm->fwRepeater1, 
													pComm->fwRepeater2);
				}
				Send = "ATXC" + pComm->fwRepeaterCBEntry + "\r";
				debug(1,"OpenConnection:Using Repeater Path: %s\n",Send.c_str());
				pComm->pSerial->Write((void *) Send.c_str(), Send.length());
			} else {
				Send = "ATXCA\r";
				debug(1,"OpenConnection:Using Repeater Path: %s\n",Send.c_str());
				pComm->pSerial->Write((void *) Send.c_str(), Send.length());
			}
			Send = "ATD" + pComm->modemDial + "\r";
			debug(1,"OpenConnection:Connecting to FW Slave at: %s",Send.c_str());
		} else {
			Send = "ATDT" + pComm->modemDial + "\r";
			debug(1,"OpenConnection:Dialing Modem at: %s\n",Send.c_str());
		}
		while ((ret = pComm->pSerial->Inkey(100)) != -1) {
			debug(20,"%c",(char) ret);
		}
		pComm->pSerial->Write((void *) Send.c_str(), Send.length());
		sleep(2);
#define NUMRESPONSE 4
		const char *modemresponse[NUMRESPONSE] = {
			"connect",
			"CONNECT",
			"NO CARRIER",
			"BUSY"
		};

		int status = pSerial->WaitFor(120000,modemresponse,NUMRESPONSE);
		if ((status != 0) && (status != 1)) {
			if (status != -1) {
				debug(10,"OpenConnection: Found response %s\n",modemresponse[status]);
			} else {
				debug(10,"OpenConnection:: Timeout exceeded during connection attempt\n");
			}
			debug(1, "OpenConnection:Did not connect successfully\n");
			addToSumFile("Communications: Failed\n");
			pComm->pSerial->Close();
			return(FAILURE);
		}
		debug(10,"OpenConnection: Found response %s\n",modemresponse[status]);
		if (pComm->isFW && pComm->fwUsesRepeater) {
			debug(5,"OpenConnection:Found First Repeater... Waiting for full connection\n");
			sleep(40); //first modem returns connect
		}
		if (pComm->isMC) {
			debug(25,"OpenConnection: Waiting for full modem connection\n");
			sleep(4);
			pComm->pSerial->ClearBuffers();
		}
		debug(5,"OpenConnection: Reading Garbage from Receiver (if any)\n");
		char buffer[1025];
		pComm->pSerial->Read(buffer, 1024, 1000);
		if (DEBUGLEVEL > 10) cerr << buffer << endl;
		pComm->pSerial->ClearBuffers();
		debug(1,"OpenConnection:Connected to Receiver\n");
	}
	if (pComm->maxTransferErrors >= 0) {
		debug(50,"OpenConnection::Setting max number of transfer errors to %d \n",pComm->maxTransferErrors);
		pComm->pSerial->setMaxTransferErrors(pComm->maxTransferErrors);
	}
	if (pComm->timeoutMultiplier >= 0) {
		debug(50,"OpenConnection::Setting timeout multiplier to %d \n",pComm->timeoutMultiplier);
		pComm->pSerial->setTimeoutMultiplier(pComm->timeoutMultiplier);
	}
	debug(50,"OpenConnection: Exiting\n");
	return(SUCCESS);

}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
// Assume serial port is open
// Put FW into Setup mode
// Enter callbook
// Read callbook
// Check for existing entry
// Write new entry
// return entry

#define FWSKIPNUMBER "123-4567"
#define FWZERO "000-0000"
#define FWBOGUSNUMBER "555-8747"
#define FWDEFAULTLOCATION 0

string SetFWRepeaterPath(Serial *pSerial, string repeater1, string repeater2) {

	int ret;
	char buf[101];
	debug(50,"SetFWRepeaterPath: Entering...\n");

	FWCleanup(pSerial);
	if ((repeater1 == "") || (repeater1.length() != 7)) {
		debug(25,"SetFWRepeaterPath: Problem with first repeater\n");
		return("A");
	}
	if ((repeater2 != "") && (repeater1.length() != 7)) {
		debug(25,"SetFWRepeaterPath: Problem with second repeater\n");
		return("A");
	}
	pSerial->Write("ATXS\n");
/*
                                MAIN MENU
                         Version 5.51  7-23-98
                           Standard Hop Table
                        Modem Serial Number 900-4552

(0)   Set Operation Mode
(1)   Set Baud Rate
(2)   Edit Call Book
(3)   Edit Radio Transmission Characteristics
(4)   Show Radio Statistics
(5)   Edit MultiPoint Parameters
(6)   TDMA Menu
(Esc) Exit Setup

Enter Choice  
*/
	ReadlineUntil(pSerial, "Enter Choice");
	debug(30,"SetFWRepeaterPath: Found Enter Choice\n");

	pSerial->Write("2");
	ReadlineUntil(pSerial, "Repeater1");
	debug(30,"SetFWRepeaterPath: Found Header Line\n");

/*
                                MODEM CALL BOOK
                                Entry to Call is 02
Entry    Number     Repeater1    Repeater2
(0)     123-4567     903-0031
(1)     123-4567     903-0031     903-0201
(2)     123-4567     903-6361
(3)     123-4567     903-6361     903-6381
(4)     123-4567     903-6361     903-0200
(5)     000-0000
(6)     000-0000
(7)     000-0000
(8)     000-0000
(9)     000-0000
(C)     Change Entry to Use (0-9) or A(ALL)
(Esc)   Exit to Main Menu
Enter all zeros (000-0000) as your last number in list
6
Enter New Number 123-4567
Enter Repeater1 (ESC=no repeater used) 
123-4567
Enter Repeater2 (ESC=no repeater used) 
123-4567
*/
	string entrynum[10];
	string number[10];
	string rptr1[10];
	string rptr2[10];
	int entry = 0;
	while ((ret = pSerial->Readline(buf,100,500)) != 0) {
		char *tmp;
		string Line = buf;
		if ((int)buf[ret-1] == 10) {
			buf[ret-1] = 0;
			ret--;
		}
		if ((int)buf[ret-1] == 13) {
			buf[ret-1] = 0;
			ret--;
		}
		if (Line.find("Change") != string::npos) break;
		tmp = strtok(buf," ");
		(tmp != NULL) ? entrynum[entry] = tmp : entrynum[entry] = "";
		tmp = strtok(NULL," ");
		(tmp != NULL) ? number[entry] = tmp : number[entry] = "";
		tmp = strtok(NULL," ");
		(tmp != NULL) ? rptr1[entry] = tmp : rptr1[entry] = "";
		tmp = strtok(NULL," ");
		(tmp != NULL) ? rptr2[entry] = tmp : rptr2[entry] = "";
		size_t start = rptr1[entry].find("-");
		if (start != string::npos) rptr1[entry].erase(start,1);
		start = rptr2[entry].find("-");
		if (start != string::npos) rptr2[entry].erase(start,1);
		entry++;
	}
	ReadlineUntil(pSerial, "Enter all");

	string callbookentry = "A";
	for (int i = 0; i < entry; i++) {
		debug(50,"SetFWRepeaterPath: Searching callbook entry %d\n",i);
		if ((rptr1[i] == repeater1) && (rptr2[i] == repeater2)) {
			callbookentry = entrynum[i].substr(1,1);
			debug(50,"SetFWRepeaterPath: Found Match\n");
			break;
		}
	}

	if (callbookentry == "A") {
		debug(15,"SetFWRepeaterPath: Adding Callbook Entry\n");
		int addto = -1;
		for (int i = 0; i < entry; i++) {
			if (number[i] != FWZERO) continue;
			debug(35,"SetFWRepeaterPath: Found empty entry at %d\n",i);
			addto = i;
			break;
		}
		for (int i = 0; (i < entry) && (addto == -1); i++) {
			if (number[i] == FWSKIPNUMBER) {
				debug(35,"SetFWRepeaterPath: Found Protected Entry at %d\n",i);
				continue;
			}
			addto = i;
			break;
		}
		if (addto == -1) addto = FWDEFAULTLOCATION;
		debug(20,"SetFWRepeaterPath:Adding entry to callbook #%d\n",addto);
		callbookentry = entrynum[addto].substr(1,1);
		pSerial->Write((char *)callbookentry.c_str());
		ReadlineUntil(pSerial, "Enter New");
		pSerial->Write(FWBOGUSNUMBER);
		ReadlineUntil(pSerial, "Enter Repeater1");
		pSerial->Write((char *)repeater1.c_str());
		ReadlineUntil(pSerial, "Enter Repeater2");
		if (repeater2 != "") {
			pSerial->Write((char *)repeater2.c_str());
		} else {
			pSerial->PutChar((char) 27);
		}
		ReadlineUntil(pSerial, "Enter all");
	}

	FWCleanup(pSerial);
	debug(50,"SetFWRepeaterPath: Exiting\n");
	return(callbookentry);
}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
void FWCleanup(Serial *pSerial) {
	sleep(1);
	pSerial->ClearBuffers();
	pSerial->ClearBuffers();
	pSerial->PutChar((char) 27);
	pSerial->PutChar((char) 27);
	pSerial->PutChar((char) 27);
	pSerial->PutChar((char) 27);
	sleep(1);
	pSerial->ClearBuffers();
	pSerial->ClearBuffers();
}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
void ReadlineUntil(Serial *pSerial, string findme) {
	char buf[1024];
	int ret;
	while ((ret = pSerial->Readline(buf,1024,500)) != 0) {
		string Line = buf;
		if (Line.find(findme.c_str()) != string::npos) break;
	}

}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int SerialFormat(SComm *c) {

	switch (c->dataBits) {
		case 7: case 8:
			break;
		default:
			debug(5,"SerialFormat: dataBits is %d, should be 7 or 8\n",c->dataBits);
			return(-1);
	}

	switch (c->parity) {
		case 'E':
			return((c->dataBits == 7) ? SERIAL_FORMAT_7E1 : SERIAL_FORMAT_8E1);
		case 'O':
			return((c->dataBits == 7) ? SERIAL_FORMAT_7O1 : SERIAL_FORMAT_8O1);
		case 'N':
			return((c->dataBits == 7) ? SERIAL_FORMAT_7N1 : SERIAL_FORMAT_8N1);
		case 'S':
			return((c->dataBits == 7) ? SERIAL_FORMAT_7S2 : SERIAL_FORMAT_8S2);
		default:
			debug(5,"SerialFormat: parity is %c, should be E, N, O, or S\n",c->parity);
			return(-1);
	}
	return(-1);
}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
void InitCommStruct(SComm *c) {

//	c->pSerial = new Serial();

	c->port = "/dev/gps";
	c->baud = 38400L;
	c->ipHost = "";
	c->ipPort = NTRIP_PORT;

	c->parity = 'N';
	c->dataBits = 8;
	c->stopBits = 1;
	c->serial_format = SERIAL_FORMAT_8N1;
	c->serial_flow = SERIAL_FLOW_HARD;
	c->modemDial = "";
	c->modemInit = "AT";
	c->fwRepeaterCBEntry = "";
	c->fwRepeater1 = "";
	c->fwRepeater2 = "";
	c->resumeDir = "";
	c->forceSave = FALSE;
	c->minbps = 0;
	c->minbps_secs = 120;

	c->isDC = TRUE;
	c->isMC = FALSE;
	c->isFW = FALSE;
	c->isIP = FALSE;
	c->fwUsesRepeater = FALSE;
	c->maxTransferErrors = -1;
	c->timeoutMultiplier = -1;

	return;
}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
void InitWithArgs(SOps *pOps, SComm *pComm) {

	if (pComm->isIP) {
		pComm->pSerial = new Socket();
	} else {
		pComm->pSerial = new Serial();
	}
	if (pComm->resumeDir != "") {
		pComm->pSerial->setResumeDir(pComm->resumeDir.c_str());
	}
	if (pComm->forceSave) {
		pComm->pSerial->setForceSave(1);
	}
//	pSerial->setReports(TransferReportStart, TransferReport, TransferReportStop);
//	if (pComm->minbps > 0) {
//		pSerial->setReports(TransferReportStart, TransferReportMinBps, TransferReportStop);
//		pSerial->setMinBPS(pComm->minbps, pComm->minbps_secs);
//	}




	return;
}
//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
string SUMFILE;
void addToSumFile(const char *fmt, ...) {

	ofstream tmp;
	char buffer[256];
	va_list ap;
	va_start(ap, fmt);

	vsprintf(buffer, fmt, ap);
	if (SUMFILE.length() > 0) {
		int status = WriteFile(NULL, SUMFILE.c_str(), (unsigned char *) buffer, strlen(buffer), TRUE);
		if (status == FAILURE) {
			cerr << "Cannot write to sumfile" << endl;
		}
	} else {
		cout << buffer;
	}
}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
string DEBUGFILE;
string DEBUGOUT;
int DEBUGFD = STDERR_FILENO;
int DEBUGLEVEL = 5;
#undef KILLDEBUG
#ifdef KILLDEBUG
void debug(int level, const char *fmt, ...) {
}
#else
void debug(int level, const char *fmt, ...) {


	if (level > -100 ) {
		if (level > DEBUGLEVEL) return;
	} else {
		if (level != DEBUGLEVEL) return;
	}
	ofstream tmp;
	char buffer[1024], timestamp[256];
	va_list ap;
	va_start(ap, fmt);
	vsprintf(buffer, fmt, ap);
	time_t tt;
	time(&tt);
	char *utc_time = asctime(gmtime(&tt));
	utc_time[strlen(utc_time)-1] = '\0'; // replace newline with space
	sprintf(timestamp, "%s UTC ", utc_time);

	(void)pthread_mutex_lock(&log_mutex);

// DEBUGFD is either STDOUT or STDERR
	if (DEBUGFD >= 0) {
		write(DEBUGFD,DEBUGOUT.c_str(),(int)DEBUGOUT.length());
	        write(DEBUGFD, timestamp, (int)strlen(timestamp));
		write(DEBUGFD,buffer,(int)strlen(buffer));
		(void)pthread_mutex_unlock(&log_mutex);
		return;
	}

// DEBUGFILE was defined
	int status;
	status = WriteFile(NULL, DEBUGFILE.c_str(), (unsigned char *) DEBUGOUT.c_str(), DEBUGOUT.length(), TRUE);
	if (status == FAILURE) {
		cerr << "Cannot write to debugfile" << endl;
		DEBUGFILE.clear();
		(void)pthread_mutex_unlock(&log_mutex);
		return;
	}
	status = WriteFile(NULL, DEBUGFILE.c_str(), (unsigned char *)timestamp, strlen(timestamp), TRUE);
	if (status == FAILURE) {
		cerr << "Cannot write to debugfile" << endl;
		DEBUGFILE.clear();
		(void)pthread_mutex_unlock(&log_mutex);
		return;
	}
	status = WriteFile(NULL, DEBUGFILE.c_str(), (unsigned char *) buffer, strlen(buffer), TRUE);
	if (status == FAILURE) {
		cerr << "Cannot write to debugfile" << endl;
		DEBUGFILE.clear();
		(void)pthread_mutex_unlock(&log_mutex);
		return;
	}
}
#endif


//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
void TransferReportStart(ReportStruct *prs) {
	debug(150,"TransferReportStart: Entering...\n");
	if (prs->filename != NULL) {
		debug(1,"\nDownloading File: %s", prs->filename);
	} else {
		debug(1,"\nDownloading File: unknown  ");
	}
	if (prs->bytes_skipped >= 0) {
		debug(1,"\tResume Bytes: %d", prs->bytes_skipped);
	} else {
		debug(1,"\tResume Bytes: NONE");
	}
	if (prs->filesize >= 0) {
		debug(1,"\tSize: %d", prs->filesize);
	} else {
		debug(1,"\tSize: unknown");
	}
	debug(1,"\n");
//	debug(1,"\t\tElapsed Time\tBlocks\tErrors\tBytes Received\tPercent\n");
	debug(150,"TransferReportStart: Exiting\n");
}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
void TransferReport(ReportStruct *prs) {
	debug(150,"TransferReport: Entering...\n");
	int percent = 0;
	int estimated_time = 0;
	unsigned long cur_down = 0; // total downloaded during current run (ie remove skipped)
	unsigned int bps = 0;
	static int outchar = 0;

	outchar %= 4;

	char out[4] = {'|','/','-','\\'};

	percent = 0;
	estimated_time = 0;
	cur_down = prs->bytes_downloaded - prs->bytes_skipped;

	float kbdown = (float)prs->bytes_downloaded / 1024.0;

	if (prs->filesize > 0) {
		percent = (int) (((double)prs->bytes_downloaded/(double)prs->filesize) * 100.0);
	}
	if (prs->elapsed_time > 0) {
		bps = (int) ((double) (cur_down * 8) / (double) prs->elapsed_time);
		estimated_time = (int) (((double)(prs->filesize - prs->bytes_skipped) * 8.0) / (double)bps);
		//estimated_time = (int) ((100.0/ (double)percent) * (double) prs->elapsed_time);
	}
	if ((estimated_time > (999 * 60)) || (estimated_time < 0)) {
		estimated_time = 0;
	}
	string asterik = "";
	for (int i = 0; i < percent; i++) {
		if ((i % 2) == 0) asterik += "*";
	}
	debug(1,"%03d%%%c%-50s| %02d:%02d/%03d:%02d  %6dbps  %9.1fKB\r",
				(int) percent,
				out[outchar++],
				asterik.c_str(),
				prs->elapsed_time/60,
				prs->elapsed_time%60,
				((int) estimated_time)/60,
				((int) estimated_time)%60,
				bps, kbdown);

	prs->estimated_time = estimated_time;
	prs->bps = bps;
	prs->percent = percent;
	prs->fail = FALSE;
	debug(150,"TransferReport: Exiting\n");
}

void TransferReportMinBps(ReportStruct *prs) {
	debug(150,"TransferReportMinBps: Entering...\n");
	int now;
	TransferReport(prs);
	prs->fail = FALSE;
	now = time(NULL);
	if (0 == prs->minbps_elapsed) {
		prs->minbps_elapsed = now;
		return;
	}
	if (prs->bps > prs->minbps) {
		prs->minbps_elapsed = now;
		return;
	}
	debug(150,"now = %d, prs->minbps_elapsed = %d, minbps_secs = %d\n",now,prs->minbps_elapsed, prs->minbps_secs);
	if ((now - prs->minbps_elapsed) < prs->minbps_secs) {
		debug(25,"Below minimum baud rate for %d seconds\n",now-prs->minbps_elapsed);
		return;
	}

	debug(25,"Marking download as failed, %d bps for %d seconds\n",prs->bps,now-prs->minbps_elapsed);
	prs->fail = TRUE;
	debug(150,"TransferReportMinBps: Exiting\n");
	return;
}


//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
void TransferReportStop(ReportStruct *prs) {
	debug(150,"TransferReportStop: Entering...\n");
	prs->elapsed_time = 0;
	debug(1,"\nDownload Done\n");
	debug(150,"TransferReportStop: Exiting\n");
}

//  ===================================================================
//  NAME:       TestDirectory(char *, char * out);
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
#define MAXDIRLENGTH 256
int TestDirectory(const char * datadir, char * out) {

	struct stat buf;
	try 
	{
		errno = 0;
		if (stat(datadir, &buf) == -1) {
			switch(errno) {
				case ENOENT:
					throw string("A component of the path file_name does not exist, or the path is an empty string.\n");

				case ENOTDIR:
					throw string("A component of the path is not a directory.\n");

				case ELOOP:
					throw string("Too  many symbolic links encountered while travers ing the path.\n");

				case EFAULT:
					throw string("Bad address.\n");

				case EACCES:
					throw string("Permission denied.\n");

				case ENOMEM:
					throw string("Out of memory (i.e. kernel memory).\n");

				case ENAMETOOLONG:
					throw string("File name too long.\n");
			}
		}
		if (!S_ISDIR(buf.st_mode)) throw("Path is not a directory\n");
	}       
	catch (string pmessage)
	{
		if (out != NULL) strcpy(out, pmessage.c_str());
//		cerr << pmessage;
		return(FAILURE);
	}   

	return(SUCCESS);
}

/* taken from http://cfaj.freeshell.org/shell/bash-2.05b/examples/loadables/mkdir.c */
/* Make all the directories leading up to PATH, then create PATH.  Note that
   this changes the process's umask; make sure that all paths leading to a
   return reset it to ORIGINAL_UMASK */
int make_path (string path)
{
	struct stat sb;
	char *p;
	char npath[1024];

	int original_umask = umask (0);
	umask (original_umask);

	int nmode = (S_IRWXU | S_IRWXG | S_IRWXO) & ~original_umask;
	int parent_mode = nmode | (S_IWRITE|S_IEXEC);	/* u+wx */

	if (stat (path.c_str(), &sb) == 0) {
		if (S_ISDIR (sb.st_mode) == 0) {
			debug(D_ERR,"`%s': file exists but is not a directory", path.c_str());
			return 1;
		}
	
		if (chmod (path.c_str(), nmode))
		{
			debug(D_ERR,"%s: %s", path.c_str(), strerror (errno));
			return 1;
		}
		return 0;
	}

	strcpy(npath,path.c_str());
    
	/* Check whether or not we need to do anything with intermediate dirs. */

	/* Skip leading slashes. */
	p = npath;
	while (*p == '/')
		p++;

	while ((p = strchr (p, '/'))) {
		*p = '\0';
		if (stat (npath, &sb) != 0)
		{
			if (mkdir (npath, parent_mode))
			{
				debug(D_ERR,"cannot create directory `%s': %s", npath, strerror (errno));
				umask (original_umask);
				return 1;
			}
		} else if (S_ISDIR (sb.st_mode) == 0) {
			debug(D_ERR,"`%s': file exists but is not a directory", npath);
			umask (original_umask);
			return 1;
		}
		
		*p++ = '/';	/* restore slash */
		while (*p == '/')
			p++;
	}

  /* Create the final directory component. */
	if (stat (npath, &sb) && mkdir (npath, nmode))
	{
		debug(D_ERR,"cannot create directory `%s': %s", npath, strerror (errno));
		umask (original_umask);
		return 1;
	}

	umask (original_umask);
	return 0;
}



#if HAVE_LIBCURL

#include <curl/curl.h>
#include <curl/types.h>
#include <curl/easy.h>

size_t ReadFunction( void *ptr, size_t size, size_t nmemb, void *stream) {

	static unsigned char *current_offset = NULL;
	static size_t length = 0;

	cerr << "Entering Read Function\n";

	if (ptr == NULL) {
		current_offset = NULL;
		length = size;
		cerr << "Set length to " << size << endl;
		return(0);
	}

	cerr << "length = " << length << endl;
	if (current_offset == NULL) current_offset = (unsigned char *) stream;

	size_t maxcopy = size*nmemb;
	if (length < maxcopy) maxcopy = length;

	memcpy(ptr, current_offset, maxcopy);

	length -= maxcopy;

	cerr << "Exiting Read Function with " << maxcopy << "\n";

	return(maxcopy);

}

int WriteUrl(string datadir, const char * filename, unsigned char * contents, int length, int append = FALSE) {

	static int count = 0;
	CURL *curl;
	CURLcode res;

	string outfile;

	outfile = datadir + filename;

	char myerr[CURL_ERROR_SIZE];

	//FILE *uploadfile;

	if (count > 0) return(SUCCESS);
	//count++;
	//uploadfile = fopen("dumpit","w");
	//uploadfile = fopen("/etc/issue","r");

	curl = curl_easy_init();

	cerr << "2 - length is " << length << endl;
	ReadFunction(NULL, length, 0, NULL);
	if (curl) {
		curl_easy_setopt(curl, CURLOPT_URL, outfile.c_str());
		curl_easy_setopt(curl, CURLOPT_READFUNCTION, ReadFunction);
		curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, myerr);
		curl_easy_setopt(curl, CURLOPT_UPLOAD, 1);
		curl_easy_setopt(curl, CURLOPT_INFILE, contents);

		res = curl_easy_perform(curl);
		if (res != 0) cout << "myerr = " << myerr << endl;
		curl_easy_cleanup(curl);
	}
	//fclose(uploadfile);
	return(SUCCESS);
}

#endif
//  ===================================================================
//  NAME:       isValidUrl(char *, char *, unsigned char *);
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int isValidUrl(const char * datadir) {

	if (datadir == NULL) return(FAILURE);
	string test = datadir;

	if (test.find("ftp://") == 0) return(SUCCESS);
	return(FAILURE);
}

//  ===================================================================
//  NAME:       WriteFile(char *, char *, unsigned char *);
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
#define DEFAULTPATH "."
#ifdef __CYGWIN32__
  #define TMPDIR "c:\\temp\\"
  #define DIRSEP '\\'
#else
  #define TMPDIR "/tmp/"
  #define DIRSEP '/'
#endif
int WriteFile(const char * datadir, const char * filename, unsigned char * contents, int length, int append) {

	static string DataDir = "";
	static int count;
	static int url = FALSE;

	if ((DataDir == "") || (datadir != NULL)) {
//URL Check
		if (isValidUrl(datadir) == SUCCESS) {
#if HAVE_LIBCURL
			// cerr << "WriteFile: " << datadir << " is a valid URL\n";
			debug(D_CHECK,"WriteFile: \"%s\" is a valid URL\n", DataDir.c_str());
			DataDir = datadir;
			if (DataDir[DataDir.length()-1] != '/') DataDir += "/";
			url = TRUE;
#else
			// cerr << "This version of Sharc does not support URL's, ignoring request\n";
			debug(D_CHECK,"This version of Sharc does not support URL's, ignoring request\n");
#endif
		} else {
			// cerr << "WriteFile: ";
			debug(D_CHECK,"WriteFile: ");
			if (datadir == NULL)
				// cerr << "NULL String ";
				debug(D_CHECK,"NULL String ");
			else
				// cerr << datadir;
				debug(D_CHECK,"%s ",DataDir.c_str());
			// cerr << " is NOT a valid URL\n";
			debug(D_CHECK," is NOT a valid URL\n");
			url = FALSE;
		}

//DIR Check
		if (url == FALSE) {
			debug(D_CHECK,"WriteFile: Checking directory\n");
			if (TestDirectory(datadir, NULL) == SUCCESS) {
				DataDir = datadir;
			} else if (TestDirectory(DEFAULTPATH, NULL) == SUCCESS) {
				DataDir = DEFAULTPATH;
			} else if (TestDirectory(TMPDIR, NULL) == SUCCESS) {
				DataDir = TMPDIR;
			} else {
				cerr << "WriteFile: Cannot find any suitable directory to write files\n";
				return(FAILURE);
			}
			if (DataDir[DataDir.length()-1] != DIRSEP) DataDir += DIRSEP;
			// cerr << "WriteFile: Directory set to " << DataDir.c_str() << endl;
			debug(D_CHECK,"WriteFile: Directory set to \"%s\"\n", DataDir.c_str());
		}
	}

	// this is ok, we may have just wanted to set the datadirectory
	if ((filename == NULL) && (contents == NULL)) return(SUCCESS);

	// empty filename is bad, empty contents is ok.
	if (filename == NULL) return(FAILURE);

	string outputfile = filename;
	// if not in a given directory, add the directory path
	if (outputfile.find(DIRSEP) == string::npos) {
		outputfile = DataDir + string(filename);
	}

#if HAVE_LIBCURL
	if (url) {
		cerr << "1 - length is " << length << endl;
		return WriteUrl(DataDir, filename, contents, length, append);
	}
#endif

	try
	{
		ofstream pfd;
		if (append) {
			pfd.open(outputfile.c_str(), ios::binary | ios::out | ios::app);
		} else {
			pfd.open(outputfile.c_str(), ios::binary | ios::out | ios::trunc);
		}

		if (!pfd) {
			throw string("Could not open file ") + outputfile.c_str() + string("\n");
		}

		pfd.write((const char *)contents, length);
		if (pfd.fail()) {
			throw string("Could not write contents to file ") + outputfile.c_str() + string("\n");
		}
		pfd.close();
	}   
	catch (string pmessage)
	{
		count++;
		if (count > 1) {
			count = 0;
			return(FAILURE); 
		}
		cerr << "Could not open file " << outputfile << endl;
		if (DataDir == TMPDIR) return(FAILURE);
		cerr << "Trying to write in directory " << TMPDIR << endl;
		string savedir = DataDir;
		int status = WriteFile(TMPDIR, filename, contents, length, append);
		DataDir = savedir;
		if (status == FAILURE) return(FAILURE);
	}

	return(SUCCESS);
}


int
is_recoverable (int error)
{
#ifdef _WIN32
	if ((WSAGetLastError() == WSAEWOULDBLOCK) || (WSAGetLastError() == WSAEINTR) || (WSAGetLastError() == WSAEINPROGRESS))
		return 1;
#else
	if ((error == EAGAIN) || (error == EINTR) || (error == EINPROGRESS))
		return 1;
#endif

#ifdef SOLARIS
	if ((error == EWOULDBLOCK))
		return 1;
#endif

#ifdef LINUX
	if (error == EIO) /* Works around an error with gdb and linux */
		return 1;
#endif

	return 0;
}


//  ===================================================================
//  NAME:		isCharAvailable
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
// return time remaining in timer
int isCharAvail(int _fd, timeout_t timeout)
{
	int	rc;
	struct timeval tv;
	fd_set rfds;
	struct itimerval itv, cur;

	timeout_t mytimeout = timeout;

	tv.tv_sec = 0;
	tv.tv_usec = 0;

	FD_ZERO(&rfds);
	FD_SET(_fd, &rfds);
	rc = select(_fd+1, &rfds, NULL, NULL, &tv);
	FD_ZERO(&rfds);
	if (rc > 0) return(timeout);
	tv.tv_sec = mytimeout / 1000;
	tv.tv_usec = (mytimeout % 1000) * 1000;

	signal(SIGALRM, SIG_IGN);
	itv.it_interval.tv_sec = 0;
	itv.it_interval.tv_usec = 0;
	itv.it_value.tv_sec = tv.tv_sec;
	itv.it_value.tv_usec = tv.tv_usec;
	setitimer(ITIMER_REAL, &itv, NULL);

	FD_ZERO(&rfds);
	FD_SET(_fd, &rfds);
	rc = select(_fd+1, &rfds, NULL, NULL, &tv);
	FD_ZERO(&rfds);

	debug(1000,"rc = %d, sec = %d, usec = %d\n",rc, tv.tv_sec, tv.tv_usec);

	itv.it_interval.tv_sec = 0;
	itv.it_interval.tv_usec = 0;
	itv.it_value.tv_sec = 0;
	itv.it_value.tv_usec = 0;
	setitimer(ITIMER_REAL, &itv, &cur);


	if (rc > 0) {
		debug(1000,"isCharAvail returning sec = %d, usec = %d\n",rc, cur.it_value.tv_sec, cur.it_value.tv_usec);
		return((int) ((cur.it_value.tv_sec * 1000) + (cur.it_value.tv_usec / 1000)));
	} else {
		debug(D_SERINFO+5,"isCharAvail: returning 0\n");
		return(0);
	}
}

void reverseBytes(char * in, int count)
{
	vector<unsigned char> temp;

	for (int i = 0; i < count; i++) {
		temp[i] = in[i];
	}
	for (int i = 0; i < count; i++) {
		in[i] = temp[count-i];
	}
}

#ifdef USE_LEAPSECONDS_FILE
// use qlib2 and leapseconds file to find leapsecs since gps epoch
int leapsecs_since_gps_epoch() {
  INT_TIME *it, it2;
  double n0, n1, t0, t1, delta;
  int result;
  double epoch;

  // nominal and true secs at GPS Epoch
  it = parse_date((char*)"1980/01/06/00:00:00.0000");
  it2 = *it;
  n0 = int_to_nepoch(it2);
  t0 = int_to_tepoch(it2);

  // get nominal and true secs since UNIX Epoch
  epoch = (double)time(NULL);
  it2 = nepoch_to_int(epoch);
  n1 = int_to_nepoch(it2);
  t1 = int_to_tepoch(it2); 
  
  // leap secs since GPS Epoch
  delta = t1 - t0 - n1 + n0;
  result = (int)round(delta);

  // if LEAPSECONDS file not found use hardcoded value
  if(result == 0) {
    result = LEAP_SECONDS;
  }

  return result;
}
#endif
