// Copyright (C) 1999 Open Source Telecom Corporation.
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// 

#include <iostream>
#include <iomanip>
#include <fstream>
#include <string>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/ioctl.h>
#include <sys/poll.h>
#include <fcntl.h>
#include <signal.h>
#include <stdlib.h>
#include <errno.h>
#include <stdio.h>
#include <ctype.h>
#include <sys/time.h>

#include "gpscommon.h"
#include "xmodem.h"
#include "zglobal.h"

bool Serial::UserCanceled;

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
unsigned char * Serial::xmodemreceive(long *fileSize, timeout_t timer, const char *filename)
{

	int i, rc;
	unsigned char *myFile = NULL;

	if (UserCanceled) {
		debug(25,"Checksum error\n");
		return(NULL);
	}

	/* Fork. */
	tcgetattr(_fd, &_restore);
	tcgetattr(_fd, &_current);

	memset(&_current.c_cc, 0, sizeof(_current.c_cc));
	initReportStruct();

	set_flow(SERIAL_FLOW_XMODEM, &_current);
	tcsetattr(_fd, TCSANOW, &_current);

	reportstruct.filename = filename;
	reportstruct.filesize = *fileSize;
	struct sigaction old_signal;
	struct sigaction new_signal;
	new_signal.sa_handler = interrupt_transfer;
	sigaction(SIGINT, &new_signal, &old_signal);
	ClearBuffers();
	try {
		rc = GetXModem(timer*TIME_MULT);
	}
	catch (SerialException& se) {
		CancelXModem();
		++FailedXFers;
		*fileSize = 0;
		File.clear();
		throw;
	}
		
	sigaction(SIGINT, &old_signal, NULL);

	tcsetattr(_fd, TCSANOW, &_restore);
	tcgetattr(_fd, &_current);

	if (rc == ERROR) {
		CancelXModem();
		++FailedXFers;
		*fileSize = 0;
		File.clear();
		return NULL;
	}

	*fileSize = File.size();
	myFile = new unsigned char[*fileSize];
	for (i = 0; i < *fileSize; i++) {
		myFile[i] = File.front();
		File.pop_front();
	}
	File.clear();
	return(myFile);
}


/*
 * rz.c By Chuck Forsberg
 *  Copyright 1993 Omen Technology INC All Rights Reserved
 *  Part of the ZMODEM Developer's Collection
 *
 *  Alarm signal handling changed to work with 4.2 BSD 7-15-84 CAF 
 *
 *  HOWMANY may be tuned for best performance
 */

/*
 * Adapted from CMODEM13.C, written by
 * Jack M. Wierda and Roderick W. Hart
 */

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
/* should be in mode 1 */
int Serial::GetXModem(timeout_t timer)
{
	int sectnum, sectcurr;
	char sendchar;
	int cblklen;                        /* bytes to dump this block */
	char secbuf[1025];

//	time_t start, now;
//	start = time(NULL);

	_xmodem.Errors = 0;
	_xmodem.FirstSector = TRUE;
	EOFSeen = FALSE;
	sectnum  = 0; 
	sendchar = _xmodem.CRCFlag ? WANTCRC : NAK;

	reportstruct.start_time = time(NULL);
	if (reportstart != NULL) reportstart(&reportstruct);
	for (;;) {
//		now = time(NULL);
//		if ((now - timer) > start) return ERROR;
		if (_xmodem.Errors > ERRORMAX) {
			UserCanceled = TRUE;
			return ERROR;
		}
		if (UserCanceled) return ERROR;
		ClearBuffers();
		PutChar(sendchar);        /* send it now, we're ready! */
		_xmodem.MaxTime =  (sectnum & 0177) ? 50 : 130;

		reportstruct.blocks_downloaded = sectnum;
		reportstruct.bytes_downloaded = File.size();
		reportstruct.elapsed_time = time(NULL) - reportstruct.start_time;
		reportstruct.errors = _xmodem.Errors;
		if (report != NULL) report(&reportstruct);
		if (reportstruct.fail)
			throw(SerialRetryable);
		sectcurr = GetXModemSector(secbuf);
		debug(150,"sectcurr = %d\n",sectcurr);
		if (sectcurr == ((sectnum + 1) & 0377)) {
			sectnum++;
			if (_xmodem.BytesLeft > _xmodem.BlockLength)
				cblklen = _xmodem.BlockLength;
			else
				cblklen = _xmodem.BytesLeft;
			if (AddSectorToBuffer(secbuf, cblklen) == ERROR) return ERROR;
			if ((_xmodem.BytesLeft -= cblklen) < 0) _xmodem.BytesLeft  = 0;
			sendchar = ACK;
		} else if (sectcurr == (sectnum & 0377)) {
			/* zperr( "Received dup Sector"); */
			sendchar = ACK;
		} else if (sectcurr == WCEOT) {
			PutChar(ACK);
			return OK;
		} else if (sectcurr == ERROR) {
			return ERROR;
		} else {
			/* zperr( "Sync Error"); */
			return ERROR;
		}
	}
	if (reportstop != NULL) reportstop(&reportstruct);
}

/*
 * Wcgetsec fetches a Ward Christensen type sector.
 * Returns sector number encountered or ERROR if valid sector not received,
 * or CAN CAN received
 * or WCEOT if eot sector
 * time is timeout for first char, set to 4 seconds thereafter
 ***************** NO ACK IS SENT IF SECTOR IS RECEIVED OK **************
 *    (Caller must do that when he is good and ready to get next sector)
 */

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int Serial::GetXModemSector(char *rxbuf)
{
	int checksum, wcj, firstch;
	unsigned short oldcrc;
	int sectcurr;
	int errors;
	int seenEOF=FALSE;

	_xmodem.LastRX = 0;
	for (errors = 0; errors < RETRYMAX; errors++) {

		firstch = ReadXModemBuffer(_xmodem.MaxTime);
		if ((firstch == STX) || (firstch == SOH)) {

			if (firstch == STX) _xmodem.BlockLength = 1024;
			if (firstch == SOH) _xmodem.BlockLength = 128;

			sectcurr = ReadXModemBuffer(1);
			oldcrc = ReadXModemBuffer(1);

			if ((sectcurr + oldcrc) == 0377) {
				oldcrc = checksum = 0;
				/* read in the entire sector */
				for (wcj = 0; wcj < _xmodem.BlockLength; wcj++) {
					firstch = ReadXModemBuffer(1);
					if (firstch < 0) break;
					oldcrc = updcrc(firstch, oldcrc);
					rxbuf[wcj] = firstch;
					checksum += firstch;
				}
				if (wcj < _xmodem.BlockLength) {
					RestartSector(1);
					continue;
				}
				firstch = ReadXModemBuffer(1);
				if (firstch < 0) {
					RestartSector(1);
					continue;
				}
				if (_xmodem.CRCFlag) {
					oldcrc = updcrc(firstch, oldcrc);
					firstch = ReadXModemBuffer(1);
					if (firstch < 0) {
						RestartSector(1);
						continue;
					}
					oldcrc = updcrc(firstch, oldcrc);
					if (oldcrc & 0xFFFF) {
						/* zperr( "CRC"); */
					} else {
						_xmodem.FirstSector = FALSE;
						return sectcurr;
					}
				} else if (((checksum - firstch) & 0377) == 0) {
					_xmodem.FirstSector = FALSE;
					return sectcurr;
				} else {
					debug(1,"Checksum error\n");
				}
			} else {
				continue;
			}
		} else if ((firstch == EOT) && _xmodem.Only1EOF && seenEOF) {
			return WCEOT;
		} else if ((firstch == EOT)) {
#if 0
			return WCEOT;
#else
			if (!_xmodem.FirstSector) {
				return WCEOT;
			}
			seenEOF = TRUE;
			int i = (_xmodem.FirstSector) ? 0 : 1;
			RestartSector(i);
			continue;
#endif
		} else if (firstch == CAN) {
			if (_xmodem.LastRX == CAN) {
				return ERROR;
			} else {
				_xmodem.LastRX = CAN;
				continue;
			}
		} else if (firstch == TIMEOUT) {
			int i = (_xmodem.FirstSector) ? 0 : 1;
			RestartSector(i);
			continue;
		} else {
			debug(1,"Sector header error message\n");
			int i = (_xmodem.FirstSector) ? 0 : 1;
			RestartSector(i);
			continue;
			/* zperr( "Got 0%o sector header", firstch); */
		}
	}
	/* try to stop the bubble machine. */
	CancelXModem();
	return ERROR;
}


//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
void Serial::RestartSector(int istimeout)
{
	if (istimeout) {
		debug(1,"TIMEOUT\n");
	}
	_xmodem.LastRX = 0;
	while(ReadXModemBuffer(1) != TIMEOUT)
			;
	if (_xmodem.FirstSector) {
		PutChar(_xmodem.CRCFlag ? WANTCRC : NAK);
	} else {
		_xmodem.MaxTime = 100; PutChar(NAK);
	}
	_xmodem.Errors++;
}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
/*
 * This version of ReadXModemBuffer is reasoably well suited for
 * reading many characters.
 *  (except, currently, for the Regulus version!)
 *
 * timeout is in tenths of seconds
 */
int Serial::ReadXModemBuffer(int timeout)
{

	int i;

	i = Inkey((timeout >= 30) ? timeout*100 : 3000);
	if (i == -1) return TIMEOUT;
	return (i & 0377);
}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
/*
 * Putsec writes the n characters of buf to receive file fout.
 *  If not in binary mode, carriage returns, and all characters
 *  starting with CPMEOF are discarded.
 *  This routine is specific to Unix like systems!!!
 */
int Serial::AddSectorToBuffer(char *sector, int n)
{

	if (n == 0) return OK;
	char *p;

	debug(150,"AddSectorToBuffer: Entering...\n");
	if (Binary) {
		for (p = sector; --n >= 0; p++ ) {
			File.push_back(*p);
		}
	} else {
		if (EOFSeen) return OK;
		for (p = sector; --n >= 0; ++p ) {
			if (*p == '\r') continue;
			if (*p == CPMEOF) {
				EOFSeen = TRUE;
				return OK;
			}
			File.push_back(*p);
		}
	}
	debug(150,"AddSectorToBuffer: Exiting...\n");
	return OK;
}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
/* send cancel string to get the other end to shut up */
int Serial::CancelXModem()
{
	char buf[8192];
//	static char canistr[] = {
//	24,24,24,24,24,24,24,24,24,24,8,8,8,8,8,8,8,8,8,8,0
//	};
	static char canistr[] = {
	CAN,CAN,CAN,CAN,CAN,CAN,CAN,CAN,CAN,CAN,8,8,8,8,8,8,8,8,8,8,0
	};

	for (int i = 0; i <  3; i++) {
		debug(150,"Serial::CancelXModem: Send cancel string and sleep attempt %d\n",i);
		ClearBuffers();
		Write(canistr);
		sleep(2);
		Read(buf, 8192, 1000);
	}

	return(1);
}

