/*
  zm.c - zmodem protocol handling lowlevelstuff
  Copyright (C) until 1998 Chuck Forsberg (OMEN Technology Inc)
  Copyright (C) 1996, 1997 Uwe Ohse

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.

  originally written by Chuck Forsberg
*/
/* historical comment: -- uwe
 *   Z M . C
 *    ZMODEM protocol primitives
 *    05-09-88  Chuck Forsberg Omen Technology Inc
 *
 * Entry point Functions:
 *	zsbhdr(type, hdr) send binary header
 *	zshhdr(type, hdr) send hex header
 *	zgethdr(hdr, eflag) receive header - binary or hex
 *	zsdata(buf, len, frameend) send data
 *	zrdata(buf, len, bytes_received) receive data
 *	stohdr(pos) store position data in _zmodem.Txhdr
 *	long rclhdr(hdr) recover position offset from header
 */

#include <iostream>
#include <fstream>
#include <string.h>
#include <strings.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <map>
#include <stdlib.h>

#include "gpscommon.h"
#include "zglobal.h"
#include "xmodem.h"

static const char *frametypes[] = {
	"Carrier Lost",		/* -3 */
	"TIMEOUT",		/* -2 */
	"ERROR",		/* -1 */
#define FTOFFSET 3
	"ZRQINIT",
	"ZRINIT",
	"ZSINIT",
	"ZACK",
	"ZFILE",
	"ZSKIP",
	"ZNAK",
	"ZABORT",
	"ZFIN",
	"ZRPOS",
	"ZDATA",
	"ZEOF",
	"ZFERR",
	"ZCRC",
	"ZCHALLENGE",
	"ZCOMPL",
	"ZCAN",
	"ZFREECNT",
	"ZCOMMAND",
	"ZSTDERR",
	"xxxxx"
#define FRTYPES 22	/* Total number of frame types in this array */
			/*  not including psuedo negative entries */
};

#define badcrc "Bad CRC"
/* static char *badcrc = "Bad CRC"; */
extern int zmodem_requested;

#define sendline(c) PutChar((c) & 0377)
#define xsendline(c) PutChar(c)

/*
 * Read a character from the modem line with timeout.
 *  Eat parity, XON and XOFF characters.
 */
int Serial::noxrd7(void)
{
	register int c;

	for (;;) {
		if ((c = Inkey(_zmodem.Rxtimeout)) < 0)
			return c;
		switch (c &= 0177) {
		case XON:
		case XOFF:
			continue;
		default:
			if (Zctlesc && !(c & 0140))
				continue;
		case '\r':
		case '\n':
		case ZDLE:
			return c;
		}
	}
}

int Serial::zgeth1(void)
{
	register int c, n;

	if ((c = noxrd7()) < 0)
		return c;
	n = c - '0';
	if (n > 9)
		n -= ('a' - ':');
	if (n & ~0xF)
		return ERROR;
	if ((c = noxrd7()) < 0)
		return c;
	c -= '0';
	if (c > 9)
		c -= ('a' - ':');
	if (c & ~0xF)
		return ERROR;
	c += (n<<4);
	return c;
}

/* Decode two lower case hex digits into an 8 bit byte value */
int Serial::zgethex(void)
{
	register int c;

	c = zgeth1();
//	VPRINTF(9,("zgethex: %02X", c));
	return c;
}

/*
 * Read a byte, checking for ZMODEM escape encoding
 *  including CAN*5 which represents a quick abort
 */
int Serial::zdlread(void)
{
	int c;
	/* Quick check for non control characters */
	if ((c = Inkey(_zmodem.Rxtimeout)) & 0140)
		return c;
	return zdlread2(c);
}
/* no, i don't like gotos. -- uwe */
int Serial::zdlread2(int c)
{
//	goto jump_over; /* bad style */
//
//again:
//	/* Quick check for non control characters */
//	if ((c = Inkey(_zmodem.Rxtimeout)) & 0140)
//		return c;
//
//jump_over:

	for (;;) {
		try {
			debug(300,"zdlread2:c = %d\n",c);
			switch (c) {
				case ZDLE:
					break;
				case XON:
				case (XON|0200):
				case XOFF:
				case (XOFF|0200):
					throw "XON/XOFF";
				default:
					if (Zctlesc && !(c & 0140)) {
						throw "Default+Zctlesc";
					}
					throw c;
			}
		}
		catch (const char *pmessage) {
			/* Quick check for non control characters */
			debug(200,"zdlread2: %s\n", pmessage);
			if ((c = Inkey(_zmodem.Rxtimeout)) & 0140)
				return c;
			continue;
		}
		catch (int myc) {
			return myc;
		}
		debug(200,"zdlread2: Continue from ZDLE\n");
		break;
	}

	for (;;) {
		try {
			if ((c = Inkey(_zmodem.Rxtimeout)) < 0)
				throw c;
			if (c == CAN && ((c = Inkey(_zmodem.Rxtimeout)) < 0))
				throw c;
			if (c == CAN && ((c = Inkey(_zmodem.Rxtimeout)) < 0))
				throw c;
			if (c == CAN && ((c = Inkey(_zmodem.Rxtimeout)) < 0))
				throw c;
			switch (c) {
				case CAN:
					debug(200,"zdlread2: Got Cancel\n");
					throw GOTCAN;

				case ZCRCE:
				case ZCRCG:
				case ZCRCQ:
				case ZCRCW:
					debug(200,"zdlread2: Got ZCRC\n");
					throw (c | GOTOR);

				case ZRUB0:
					debug(200,"zdlread2: Got ZRUB0\n");
					throw 0177;

				case ZRUB1:
					debug(200,"zdlread2: Got ZRUB1\n");
					throw 0377;

				case XON:
				case (XON|0200):
				case XOFF:
				case (XOFF|0200):
					debug(200,"zdlread2: Got XON/XOFF\n");
					throw "XON/XOFF";

				default:
					if (Zctlesc && ! (c & 0140)) {
						throw "Default+Zctlesc";
					}
					if ((c & 0140) ==  0100)
						throw (c ^ 0100);
					break;
			}
		}
		catch (const char *pmessage) {
			debug(200,"zdlread2:(part2) %s\n", pmessage);
			continue;
		}
		catch (int myreturn) {
			return(myreturn);
		}
		break;
	}
	debug(150,"Bad escape sequence %d\n",c);
	return ERROR;
}



/*
 * Send character c with ZMODEM escape sequence encoding.
 *  Escape XON, XOFF. Escape CR following @ (Telenet net escape)
 */
void Serial::zsendline(int c)
{

	switch(_zmodem.zsendline_tab[(unsigned) (c&=0377)])
	{
	case 0: 
		xsendline(_zmodem.lastsent = c); 
		break;
	case 1:
		xsendline(ZDLE);
		c ^= 0100;
		xsendline(_zmodem.lastsent = c);
		break;
	case 2:
		if ((_zmodem.lastsent & 0177) != '@') {
			xsendline(_zmodem.lastsent = c);
		} else {
			xsendline(ZDLE);
			c ^= 0100;
			xsendline(_zmodem.lastsent = c);
		}
		break;
	}
}

void Serial::zsendline_s(const char *s, size_t count) 
{
	const char *end=s+count;
	while(s!=end) {
		int last_esc=0;
		const char *t=s;
		while (t!=end) {
			last_esc=_zmodem.zsendline_tab[(unsigned) ((*t) & 0377)];
			if (last_esc) 
				break;
			t++;
		}
		if (t!=s) {
			fwrite(s,(size_t)(t-s),1,stdout);
			_zmodem.lastsent=t[-1];
			s=t;
		}
		if (last_esc) {
			int c=*s;
			switch(last_esc) {
			case 0: 
				xsendline(_zmodem.lastsent = c); 
				break;
			case 1:
				xsendline(ZDLE);
				c ^= 0100;
				xsendline(_zmodem.lastsent = c);
				break;
			case 2:
				if ((_zmodem.lastsent & 0177) != '@') {
					xsendline(_zmodem.lastsent = c);
				} else {
					xsendline(ZDLE);
					c ^= 0100;
					xsendline(_zmodem.lastsent = c);
				}
				break;
			}
			s++;
		}
	}
}


/* Send ZMODEM binary header hdr of type type */
void Serial::zsbhdr(int type, char *hdr)
{
	register int n;
	register unsigned short crc;

	debug(150, "zsbhdr: %s %ld\n",frametypes[type+FTOFFSET],rclhdr(hdr));
	if (type == ZDATA)
		for (n = _zmodem.Znulls; --n >=0; )
			xsendline(0);

	xsendline(ZPAD); xsendline(ZDLE);

	_zmodem.Crc32t=_zmodem.Txfcs32;
	if (_zmodem.Crc32t)
		zsbh32(hdr, type);
	else {
		xsendline(ZBIN); zsendline(type); crc = updcrc(type, 0);

		for (n=4; --n >= 0; ++hdr) {
			zsendline(*hdr);
			crc = updcrc((0377& *hdr), crc);
		}
		crc = updcrc(0,updcrc(0,crc));
		zsendline(crc>>8);
		zsendline(crc);
	}
	if (type != ZDATA)
		flushmo();
}


/* Send ZMODEM binary header hdr of type type */
void Serial::zsbh32(char *hdr, int type)
{
	register int n;
	register unsigned long crc;

	xsendline(ZBIN32);  zsendline(type);
	crc = 0xFFFFFFFFL; crc = UPDC32(type, crc);

	for (n=4; --n >= 0; ++hdr) {
		crc = UPDC32((0377 & *hdr), crc);
		zsendline(*hdr);
	}
	crc = ~crc;
	for (n=4; --n >= 0;) {
		zsendline((int)crc);
		crc >>= 8;
	}
}

/* Send ZMODEM HEX header hdr of type type */
void Serial::zshhdr(int type, char *hdr)
{
	register int n;
	register unsigned short crc;
	char s[30];
	size_t len;

	debug(150, "zshhdr: Entering...\n");
	debug(150, "zshhdr: %s %ld\n",frametypes[type+FTOFFSET],rclhdr(hdr));
	s[0]=ZPAD;
	s[1]=ZPAD;
	s[2]=ZDLE;
	s[3]=ZHEX;
	zputhex(type & 0x7f ,s+4);
	len=6;
	_zmodem.Crc32t = 0;

	crc = updcrc((type & 0x7f), 0);
	for (n=4; --n >= 0; ++hdr) {
		zputhex(*hdr,s+len); 
		len += 2;
		crc = updcrc((0377 & *hdr), crc);
	}
	crc = updcrc(0,updcrc(0,crc));
	zputhex(crc>>8,s+len); 
	zputhex(crc,s+len+2);
	len+=4;

	/* Make it printable on remote machine */
	s[len++]=015;
	s[len++]=0212;
	/*
	 * Uncork the remote in case a fake XOFF has stopped data flow
	 */
	if (type != ZFIN && type != ZACK)
	{
		s[len++]=021;
	}
	flushmo();
	Write(s,len);
	debug(150,"zshhdr: Exiting...\n");
}

/*
 * Send binary array buf of length length, with ending ZDLE sequence frameend
 */
static const char *Zendnames[] = { "ZCRCE", "ZCRCG", "ZCRCQ", "ZCRCW"};
void Serial::zsdata(const char *buf, size_t length, int frameend)
{
	register unsigned short crc;

	debug(150,"zsdata: Entering...\n");
	debug(150,"zsdata: %ld %s\n",length,Zendnames[(frameend-ZCRCE)&3]);
	crc = 0;
	do {
		zsendline(*buf); crc = updcrc((0377 & *buf), crc);
		buf++;
	} while (--length>0);
	xsendline(ZDLE); xsendline(frameend);
	crc = updcrc(frameend, crc);

	crc = updcrc(0,updcrc(0,crc));
	zsendline(crc>>8); zsendline(crc);
	if (frameend == ZCRCW) {
		xsendline(XON);  flushmo();
	}
}

void Serial::zsda32(const char *buf, size_t length, int frameend)
{
	int c;
	unsigned long crc;
	int i;
//	VPRINTF(3,("zsdat32: %d %s", length, Zendnames[(frameend-ZCRCE)&3]));

	crc = 0xFFFFFFFFL;
	zsendline_s(buf,length);
	for (; length; length--) {
		c = *buf & 0377;
		crc = UPDC32(c, crc);
		buf++;
	}
	xsendline(ZDLE); xsendline(frameend);
	crc = UPDC32(frameend, crc);

	crc = ~crc;
	for (i=4; --i >= 0;) {
		c=(int) crc;
		if (c & 0140)
			xsendline(_zmodem.lastsent = c);
		else
			zsendline(c);
		crc >>= 8;
	}
	if (frameend == ZCRCW) {
		xsendline(XON);  flushmo();
	}
}

#if __GNUC__ < 2 || (__GNUC__ == 2 && __GNUC_MINOR__ <= 4)
#  undef DEBUG_BLOCKSIZE
#endif

#ifdef DEBUG_BLOCKSIZE
struct debug_blocksize {
	int size;
	long count;
};
struct debug_blocksize blocksizes[]={
	{32,0},
	{64,0},
	{128,0},
	{256,0},
	{512,0},
	{1024,0},
	{2048,0},
	{4096,0},
	{8192,0},
	{0,0}
};
void Serial::count_blk(int size)
{
	int i;
	for (i=0;blocksizes[i].size;i++) {
		if (blocksizes[i].size==size) {
			blocksizes[i].count++;
			return;
		}
	}
	blocksizes[i].count++;
}

static void printout_blocksizes(void) __attribute__((__destructor__));
void Serial::printout_blocksizes(void) 
{
	int i;
	for (i=0;blocksizes[i].size;i++) {
		if (blocksizes[i].count) {
			lsyslog(LOG_DEBUG,"%4d byte: %ld blocks\n",
				   blocksizes[i].size,blocksizes[i].count);
		}
	}
	if (blocksizes[i].count) {
		lsyslog(LOG_DEBUG,"unk. byte: %ld blocks",
			   blocksizes[i].count);
	}
}
#define COUNT_BLK(x) count_blk(x)
#else
#define COUNT_BLK(x)
#endif

/*
 * Receive array buf of max length with ending ZDLE sequence
 *  and CRC.  Returns the ending character or error code.
 *  NB: On errors may store length+1 bytes!
 */
int Serial::zrdata(char *buf, int length, size_t *bytes_received)
{
	register int c;
	register unsigned short crc;
	register char *end;
	register int d;

	debug(150, "Serial::zrdata: Entering...\n");
	*bytes_received=0;
	if (_zmodem.Rxframeind == ZBIN32)
		return zrdat32(buf, length, bytes_received);

	crc = 0;  end = buf + length;
	while (buf <= end) {
		if ((c = zdlread()) & ~0377) {
crcfoo:
			switch (c) {
			case GOTCRCE:
			case GOTCRCG:
			case GOTCRCQ:
			case GOTCRCW:
				{ 
					d = c;
					c &= 0377;
					crc = updcrc(c, crc);
					if ((c = zdlread()) & ~0377)
						goto crcfoo;
					crc = updcrc(c, crc);
					if ((c = zdlread()) & ~0377)
						goto crcfoo;
					crc = updcrc(c, crc);
					if (crc & 0xFFFF) {
						debug(150, "1- %s\n",badcrc);
						return ERROR;
					}
					*bytes_received = length - (end - buf);
					COUNT_BLK(*bytes_received);
					debug(150,"zrdata: %ld %s\n",(*bytes_received),Zendnames[(d-GOTCRCE)&3]);
					return d;
				}
			case GOTCAN:
				debug(150,"Sender Canceled\n");
				return ZCAN;
			case TIMEOUT:
				debug(150,"TIMEOUT\n");
				return c;
			default:
				debug(150,"Bad data subpacket\n");
				return c;
			}
		}
		*buf++ = c;
		crc = updcrc(c, crc);
	}
	debug(150,"Data subpacket too long\n");
	debug(150,"Serial::zrdata: Exiting...\n");
	return ERROR;
}

int Serial::zrdat32(char *buf, int length, size_t *bytes_received)
{
	register int c;
	register unsigned long crc;
	register char *end;
	register int d;

	crc = 0xFFFFFFFFL;  end = buf + length;
	while (buf <= end) {
		if ((c = zdlread()) & ~0377) {
crcfoo:
			switch (c) {
			case GOTCRCE:
			case GOTCRCG:
			case GOTCRCQ:
			case GOTCRCW:
				d = c;
				c &= 0377;
				crc = UPDC32(c, crc);
				if ((c = zdlread()) & ~0377)
					goto crcfoo;
				crc = UPDC32(c, crc);
				if ((c = zdlread()) & ~0377)
					goto crcfoo;
				crc = UPDC32(c, crc);
				if ((c = zdlread()) & ~0377)
					goto crcfoo;
				crc = UPDC32(c, crc);
				if ((c = zdlread()) & ~0377)
					goto crcfoo;
				crc = UPDC32(c, crc);
				if (crc != 0xDEBB20E3) {
					debug(150, "2- %s\n",badcrc);
					debug(150,"zrdat32: %ld %s\n",length - (end - buf), Zendnames[(d-GOTCRCE)&3]);
					return ERROR;
				}
				*bytes_received = length - (end - buf);
				COUNT_BLK(*bytes_received);
				debug(150,"zrdat32: %ld %s\n",*bytes_received, Zendnames[(d-GOTCRCE)&3]);
				return d;
			case GOTCAN:
				debug(150,"Sender Canceled");
				return ZCAN;
			case TIMEOUT:
				debug(150,"TIMEOUT");
				return c;
			default:
				debug(150,"Bad data subpacket");
				return c;
			}
		}
		debug(200,"%d ",length - (end - buf));
		*buf++ = c;
		crc = UPDC32(c, crc);
	}
	debug(150,"Data subpacket too long");
	return ERROR;
}

/*
 * Read a ZMODEM header to hdr, either binary or hex.
 *  eflag controls local display of non zmodem characters:
 *	0:  no display
 *	1:  display printing characters only
 *	2:  display all non ZMODEM characters
 *  On success, set Zmodem to 1, set Rxpos and return type of header.
 *   Otherwise return negative on error.
 *   Return ERROR instantly if ZCRCW sequence, for fast error recovery.
 */
#define Zrwindow 1400
#define Baudrate 24000
//#define Zrwindow 4000
//#define Baudrate 1000
int Serial::zgethdr(char *hdr, int eflag, size_t *Rxpos)
{
	register int c, cancount;
	unsigned int max_garbage; /* Max bytes before start of frame */
	size_t rxpos=0; /* keep gcc happy */

	debug(150,"zgethdr:: Entering...\n");

	max_garbage = Zrwindow + Baudrate;
	_zmodem.Rxframeind = _zmodem.Rxtype = 0;

startover:
	cancount = 5;
again:
	/* Return immediate ERROR if ZCRCW sequence seen */
	c = Inkey(_zmodem.Rxtimeout);
	switch (c) {
	case RCDO:
	case TIMEOUT:
	case -1: // timeout from Inkey
		goto fifi;
	case CAN:
gotcan:
		if (--cancount <= 0) {
			c = ZCAN; goto fifi;
		}
		c = Inkey(1);
		switch (c) {
		case TIMEOUT:
		case -1: // timeout from Inkey
			goto again;
		case ZCRCW:
			c = ERROR;
		/* **** FALL THRU TO **** */
		case RCDO:
			goto fifi;
		default:
			break;
		case CAN:
			if (--cancount <= 0) {
				c = ZCAN; goto fifi;
			}
			goto again;
		}
	/* **** FALL THRU TO **** */
	default:
agn2:
		if ( --max_garbage == 0) {
			debug(150,"Serial::zgethdr: Garbage count exceeded\n");
			return(ERROR);
		}
		if ((max_garbage % 100) == 0) {
			debug(150,"Serial::zgethdr: Read another 100 Garbage Characters\n");
		}
		goto startover;
	case ZPAD|0200:		/* This is what we want. */
	case ZPAD:		/* This is what we want. */
		break;
	}
	cancount = 5;
splat:
	switch (c = noxrd7()) {
	case ZPAD:
		goto splat;
	case RCDO:
	case TIMEOUT:
		goto fifi;
	default:
		goto agn2;
	case ZDLE:		/* This is what we want. */
		break;
	}

	switch (c = noxrd7()) {
	case RCDO:
	case TIMEOUT:
		goto fifi;
	case ZBIN:
		_zmodem.Rxframeind = ZBIN;  _zmodem.Crc32 = FALSE;
		c =  zrbhdr(hdr);
		break;
	case ZBIN32:
		_zmodem.Crc32 = _zmodem.Rxframeind = ZBIN32;
		c =  zrbhdr32(hdr);
		break;
	case ZHEX:
		_zmodem.Rxframeind = ZHEX;  _zmodem.Crc32 = FALSE;
		c =  zrhhdr(hdr);
		break;
	case CAN:
		goto gotcan;
	default:
		goto agn2;
	}
	rxpos = hdr[ZP3] & 0377;
	rxpos = (rxpos<<8) + (hdr[ZP2] & 0377);
	rxpos = (rxpos<<8) + (hdr[ZP1] & 0377);
	rxpos = (rxpos<<8) + (hdr[ZP0] & 0377);
fifi:
	switch (c) {
	case GOTCAN:
		c = ZCAN;
	/* **** FALL THRU TO **** */
	case ZNAK:
	case ZCAN:
	case ERROR:
	case TIMEOUT:
	case RCDO:
		debug(150,"Got %s\n",frametypes[c+FTOFFSET]);
	/* **** FALL THRU TO **** */
	default:
		;
		if (c >= -3 && c <= FRTYPES)
			debug(150,"zgethdr: %s %ld\n",frametypes[c+FTOFFSET], (unsigned long) rxpos);
		else
			debug(150,"zgethdr: %d %ld\n",c, (unsigned long) rxpos);
	}
	if (Rxpos)
		*Rxpos=rxpos;
	debug(150,"zgethdr:: Exiting...\n");
	return c;
}

/* Receive a binary style header (type and position) */
int Serial::zrbhdr(char *hdr)
{
	register int c, n;
	register unsigned short crc;

	if ((c = zdlread()) & ~0377)
		return c;
	_zmodem.Rxtype = c;
	crc = updcrc(c, 0);

	for (n=4; --n >= 0; ++hdr) {
		if ((c = zdlread()) & ~0377)
			return c;
		crc = updcrc(c, crc);
		*hdr = c;
	}
	if ((c = zdlread()) & ~0377)
		return c;
	crc = updcrc(c, crc);
	if ((c = zdlread()) & ~0377)
		return c;
	crc = updcrc(c, crc);
	if (crc & 0xFFFF) {
		debug(150, "3- %s\n",badcrc);
		return ERROR;
	}
	_zmodem.protocol = ZM_ZMODEM;
	zmodem_requested=TRUE;
	return _zmodem.Rxtype;
}

/* Receive a binary style header (type and position) with 32 bit FCS */
int Serial::zrbhdr32(char *hdr)
{
	register int c, n;
	register unsigned long crc;

	if ((c = zdlread()) & ~0377)
		return c;
	_zmodem.Rxtype = c;
	crc = 0xFFFFFFFFL; crc = UPDC32(c, crc);
	debug(200,"zrbhdr32 c=%X  crc=%lX\n", c, crc);

	for (n=4; --n >= 0; ++hdr) {
		if ((c = zdlread()) & ~0377)
			return c;
		crc = UPDC32(c, crc);
		*hdr = c;
		debug(200,"zrbhdr32 c=%X  crc=%lX\n", c, crc);
	}
	for (n=4; --n >= 0;) {
		if ((c = zdlread()) & ~0377)
			return c;
		crc = UPDC32(c, crc);
		debug(150,"zrbhdr32 c=%X  crc=%lX\n", c, crc);
	}
	if (crc != 0xDEBB20E3) {
		debug(150, "4- %s\n",badcrc);
		return ERROR;
	}
	_zmodem.protocol = ZM_ZMODEM;
	zmodem_requested=TRUE;
	return _zmodem.Rxtype;
}


/* Receive a hex style header (type and position) */
int Serial::zrhhdr(char *hdr)
{
	register int c;
	register unsigned short crc;
	register int n;

	if ((c = zgethex()) < 0)
		return c;
	_zmodem.Rxtype = c;
	crc = updcrc(c, 0);

	for (n=4; --n >= 0; ++hdr) {
		if ((c = zgethex()) < 0)
			return c;
		crc = updcrc(c, crc);
		*hdr = c;
	}
	if ((c = zgethex()) < 0)
		return c;
	crc = updcrc(c, crc);
	if ((c = zgethex()) < 0)
		return c;
	crc = updcrc(c, crc);
	if (crc & 0xFFFF) {
		debug(150, "5- %s\n",badcrc);
		return ERROR;
	}
	switch ( c = Inkey(1)) {
	case 0215:
		/* **** FALL THRU TO **** */
	case 015:
	 	/* Throw away possible cr/lf */
		Inkey(1);
		break;
	}
	_zmodem.protocol = ZM_ZMODEM;
	zmodem_requested=TRUE;
	return _zmodem.Rxtype;
}

/* Write a byte as two hex digits */
void Serial::zputhex(int c, char *pos)
{
	static char	digits[]	= "0123456789abcdef";

	debug(150,"zputhex: Entering...\n");

	debug(150,"zputhex: %02X\n", c);
	pos[0]=digits[(c&0xF0)>>4];
	pos[1]=digits[c&0x0F];
	debug(150,"zputhex: Exiting...\n");
}

void Serial::zsendline_init(void)
{
	int i;
	for (i=0;i<256;i++) {
		if (i & 0140)
			_zmodem.zsendline_tab[i]=0;
		else {
			switch(i)
			{
			case ZDLE:
			case XOFF: /* ^Q */
			case XON: /* ^S */
			case (XOFF | 0200):
			case (XON | 0200):
				_zmodem.zsendline_tab[i]=1;
				break;
			case 020: /* ^P */
			case 0220:
				if (_zmodem.turbo_escape)
					_zmodem.zsendline_tab[i]=0;
				else
					_zmodem.zsendline_tab[i]=1;
				break;
			case 015:
			case 0215:
				if (Zctlesc)
					_zmodem.zsendline_tab[i]=1;
				else if (!_zmodem.turbo_escape)
					_zmodem.zsendline_tab[i]=2;
				else 
					_zmodem.zsendline_tab[i]=0;
				break;
			default:
				if (Zctlesc)
					_zmodem.zsendline_tab[i]=1;
				else
					_zmodem.zsendline_tab[i]=0;
			}
		}
	}
}



/* Store pos in _zmodem.Txhdr */
void Serial::stohdr(size_t pos)
{
	debug(150,"Serial::stohdr: Entering...\n");
	long lpos=(long) pos;
	_zmodem.Txhdr[ZP0] = lpos;
	_zmodem.Txhdr[ZP1] = lpos>>8;
	_zmodem.Txhdr[ZP2] = lpos>>16;
	_zmodem.Txhdr[ZP3] = lpos>>24;
	debug(150,"Serial::stohdr: Exiting...\n");
}

/* Recover a long integer from a header */
long Serial::rclhdr(char *hdr)
{
	long l;

	l = (hdr[ZP3] & 0377);
	l = (l << 8) | (hdr[ZP2] & 0377);
	l = (l << 8) | (hdr[ZP1] & 0377);
	l = (l << 8) | (hdr[ZP0] & 0377);
	return l;
}

/* End of zm.c */
