/*
  lrz - receive files with x/y/zmodem
  Copyright (C) until 1988 Chuck Forsberg (Omen Technology INC)
  Copyright (C) 1994 Matt Porter, Michael D. Black
  Copyright (C) 1996, 1997 Uwe Ohse

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.

  originally written by Chuck Forsberg
*/

#include <iostream>
#include <iomanip>
#include <fstream>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <ctype.h>
#include <errno.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "gpscommon.h"
#include "zglobal.h"


#define MAX_BLOCK 8192

//bool Serial::UserCanceled;

/*
 * Max value for HOWMANY is 255 if NFGVMIN is not defined.
 *   A larger value reduces system overhead but may evoke kernel bugs.
 *   133 corresponds to an XMODEM/CRC sector
 */
#ifndef HOWMANY
#ifdef NFGVMIN
#define HOWMANY MAX_BLOCK
#else
#define HOWMANY 255
#endif
#endif

int Readnum = HOWMANY;	/* Number of bytes to ask for in read() from modem */
int skip_if_not_found;

int Rxbinary=FALSE;	/* receive all files in bin mode */
int Rxascii=FALSE;	/* receive files in ascii (translate) mode */
int try_resume=TRUE;
int junk_path=FALSE;
int zmodem_requested=FALSE;

char secbuf[MAX_BLOCK + 1];

char zconv;		/* ZMODEM file conversion request */
char zmanag;		/* ZMODEM file management request */
char ztrans;		/* ZMODEM file transport request */
int Zctlesc;		/* Encode control characters */
int Zrwindow = 1400;	/* RX window size (controls garbage count) */

int tryzhdrtype=ZRINIT;	/* Header type to send corresponding to Last rx close */
time_t stop_time;

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
unsigned char * Serial::zmodemreceive(long *fileSize, timeout_t timer)
{
	int i, rc;
	unsigned char *myFile = NULL;
	struct stat sb;
 
	debug(150,"zmodemreceive:: Entering...\n");
	tcgetattr(_fd, &_restore);
	tcgetattr(_fd, &_current);

	memset(&_current.c_cc, 0, sizeof(_current.c_cc));

	set_flow(SERIAL_FLOW_XMODEM, &_current);
	tcsetattr(_fd, TCSANOW, &_current);
//	strcpy(_zmodem.zi[0].resname,"");

	// actual call to get a file
        try {
		rc = wcreceive();
		if (rc == ERROR) throw SerialFail;
        }
        catch (SerialException& se) {
		writeResumeFile(&File);
                ++FailedXFers;
                *fileSize = 0;
                File.clear();
		if (se.isRetryable()) throw;
		return(NULL);
        }


	// restore port attributes to original settings
	tcsetattr(_fd, TCSANOW, &_restore);
	tcgetattr(_fd, &_current);

	// copy vector of file to unsigned char to return
	if (ForceSave) writeResumeFile(&File);

	*fileSize = File.size();
	myFile = new unsigned char[*fileSize];
	for (i = 0; i < *fileSize; i++) {
		myFile[i] = File.front();
		File.pop_front();
	}
	File.clear();

#if 0
	// write file to resume file if user selected ForceSave
	// occurs or user wants all files written
	if ( ForceSave || (rc == ERROR)) {
		try {
			ofstream pfd;

			if (*fileSize == 0) throw "Serial::zmodemreceive: No data to save for resume\n";
			if (! (Binary && zconv==ZCRESUM)) throw "Serial::zmodemreceive: Not allowed to resume\n";

			// RESUME: open file and write contents of File to it
			debug(100, "Trying to write resume file %s\n",resumeFileName);
			pfd.open(resumeFileName, ios::binary | ios::out | ios::trunc);
			if (!pfd.is_open()) throw "Serial::zmodemreceive: Cannot open file\n";

			pfd.write((const char *)myFile, *fileSize);
			if (pfd.fail()) {
				pfd.close();
				throw "Serial::zmodemreceive: Could not write contents to file\n";
			}
			debug(100, "Successfully wrote resume file %s with %d bytes\n",resumeFileName,*fileSize);
			pfd.close();
		}
		catch (const char *pmessage) {
			debug(100,pmessage);
			if (stat(resumeFileName, &sb) == 0) unlink(resumeFileName);
		}
	}

	if (rc == ERROR) {
		++FailedXFers;
		*fileSize = 0;
		delete myFile;
		return NULL;
	}
#endif

// RESUME:: delete resume file if it exists
	if (! ForceSave) 
		if (stat(resumeFileName, &sb) == 0)
			if (unlink(resumeFileName) == 0)
				debug(100, "Successfully unlinked resume file %s\n",resumeFileName);

	debug(150,"zmodemreceive:: Exiting...\n");
	return(myFile);

}   


//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
int Serial::wcreceive()
{
	int c;
	int rc;

	c = ERROR;
	rc = ERROR;
	debug(150,"wcreceive:: Entering...\n");
	if (UserCanceled) {
		debug(25,"wcreceive:: Downloads canceled by user\n");
		return(ERROR);
	}
	_zmodem.CRCFlag=1;
	try
	{
		c = tryz();
		if ((c == 0) || (c == ERROR)) throw "BAD return from tryz";

		struct sigaction old_signal;
		struct sigaction new_signal;
		new_signal.sa_handler = interrupt_transfer;
		sigaction(SIGINT, &new_signal, &old_signal);
		c = rzfiles();
		sigaction(SIGINT, &old_signal, NULL);

		if (c) throw "BAD return from rzfiles";
		rc = OK;
	}
	catch (const char *pmessage)
	{
		if (c != 0) CancelXModem();
		debug(1,"%s\n",pmessage);
		rc = ERROR;
	}

	debug(150,"wcreceive:: Exiting...\n");
	return(rc);
}


//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
/*
 * Process incoming file information header
 */
int Serial::procheader(char *name, int index)
{
	char *p;
	static char *name_static=NULL;
	char *nameend;
	bool allowResume = FALSE;

	debug(150,"Serial::procheader: Entering...\n");
	if (name_static)
		delete name_static;
	if (junk_path) {
		p=strrchr(name,'/');
		if (p) {
			p++;
			if (!*p) {
				return ERROR;
			}
			name=p;
		}
	}
	name_static= new char [strlen(name)+1];
/*
	if (!name_static)
		error(1,0,_("out of memory"));
*/
	strcpy(name_static,name);
	_zmodem.zi[index].fname=name_static;

	/* set default parameters and overrides */
	Binary = (!Rxascii) || Rxbinary;

	/*
	 *  Process ZMODEM remote file management requests
	 */
	if (!Rxbinary && zconv == ZCNL)	/* Remote ASCII override */
		Binary = 0;
	if (zconv == ZCBIN)	/* Remote Binary override */
		Binary = TRUE;
	if (Binary && zconv == ZCBIN && tryResume()) {
		zconv=ZCRESUM;
		allowResume=TRUE;
	}

	_zmodem.zi[index].bytes_total = DEFBYTL;
	_zmodem.zi[index].mode = 0; 
	_zmodem.zi[index].eof_seen = 0; 
	_zmodem.zi[index].modtime = 0;

	nameend = name + 1 + strlen(name);
	if (*nameend) {	/* file coming from Unix or DOS system */
		long modtime;
		long bytes_total;
		int mode;
		sscanf(nameend, "%ld%lo%o", &bytes_total, &modtime, &mode);
		_zmodem.zi[index].modtime=modtime;
		_zmodem.zi[index].bytes_total=bytes_total;
		_zmodem.zi[index].mode=mode;
		if (_zmodem.zi[index].mode & UNIXFILE)
			++Binary;
	}

	if (!*nameend) {		/* File coming from CP/M system */
		for (p=name_static; *p; ++p)		/* change / to _ */
			if ( *p == '/')
				*p = '_';

		if ( *--p == '.')		/* zap trailing period */
			*p = 0;
	}

	debug(150,"name = %s\n",name);

	_zmodem.zi[index].bytes_skipped = 0;
	if (allowResume) {
		debug(150,"Calling setResumeFileName\n");
		setResumeFileName(_zmodem.zi[index].fname,
				_zmodem.zi[index].bytes_total,
				(int) _zmodem.zi[index].modtime);
		_zmodem.zi[index].bytes_skipped = readResumeFile();
	} else {
		debug(100,"Serial::procheader: Not allowed to resume\n");
	}
	debug(10,"Serial::procheader: Skipping %d bytes read from resume file\n", _zmodem.zi[index].bytes_skipped);
	_zmodem.zi[index].bytes_received=_zmodem.zi[index].bytes_skipped;

	debug(150,"Serial::procheader: Exiting...\n");
	return OK;
}


//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
/*
 * Initialize for Zmodem receive attempt, try to activate Zmodem sender
 *  Handles ZSINIT frame
 *  Return ZFILE if Zmodem filename received, -1 on error,
 *   ZCOMPL if transaction finished,  else 0
 */
int Serial::tryz(void)
{
	register int c, n;
#ifdef UNUSED
	register int cmdzack1flg;
#endif
	int rc = 0;
	int zrqinits_received=0;
	size_t bytes_in_block=0;

	debug(150,"Serial::tryz: Entering...\n");
	int n_start = zmodem_requested ? 15 : 5;
	int loop_started = FALSE;
	for (n = n_start; ((n + zrqinits_received) >=0) && (zrqinits_received < 10); --n ) {
		try
		{
			if (! loop_started) throw 0;
			for (;;) {
				switch (zgethdr(_zmodem.Rxhdr, 0, NULL)) {
					/* getting one ZRQINIT is totally ok. Normally a ZFILE follows 
		 			* (and might be in our buffer, so don't purge it). But if we
		 			* get more ZRQINITs than the sender has started up before us
		 			* and sent ZRQINITs while waiting. 
		 			*/
					case ZRQINIT:
						zrqinits_received++;
						throw 0;
					case ZEOF:
					case TIMEOUT:
					default:
						throw 1;;

					case ZFILE:
						zconv = _zmodem.Rxhdr[ZF0];
						// resume with sz -r is
						// impossible (at least with unix sz)
 						// if this is not set
						if (!zconv) zconv=ZCBIN;

						if (_zmodem.Rxhdr[ZF1] & ZF1_ZMSKNOLOC) {
							_zmodem.Rxhdr[ZF1] &= ~(ZF1_ZMSKNOLOC);
							skip_if_not_found=TRUE;
						}
						zmanag = _zmodem.Rxhdr[ZF1];
						ztrans = _zmodem.Rxhdr[ZF2];
						tryzhdrtype = ZRINIT;
						c = zrdata(secbuf, MAX_BLOCK,&bytes_in_block);
						// io_mode(0,3); //commented out since serial port should be OK
						if (c == GOTCRCW) return ZFILE;
						zshhdr(ZNAK, _zmodem.Txhdr);
						continue;

					case ZSINIT:
						/* this once was:
			 			* Zctlesc = TESCCTL & _zmodem.Rxhdr[ZF0];
			 			* trouble: if rz get --escape flag:
			 			* - it sends TESCCTL to sz, 
			 			*   get a ZSINIT _without_ TESCCTL (yeah - sender didn't know), 
			 			*   overwrites Zctlesc flag ...
			 			* - sender receives TESCCTL and uses "|=..."
			 			* so: sz escapes, but rz doesn't unescape ... not good.
			 			*/
						Zctlesc |= TESCCTL & _zmodem.Rxhdr[ZF0];
						if (zrdata(_zmodem.Attn, ZATTNLEN,&bytes_in_block) == GOTCRCW) {
							stohdr(1L);
							zshhdr(ZACK, _zmodem.Txhdr);
							continue;
						}
						zshhdr(ZNAK, _zmodem.Txhdr);
						continue;

					case ZFREECNT:
						stohdr(0L);
						zshhdr(ZACK, _zmodem.Txhdr);
						continue;

					case ZCOMMAND:
#ifdef UNUSED
						cmdzack1flg = _zmodem.Rxhdr[ZF0];
#endif

						if (zrdata(secbuf, MAX_BLOCK,&bytes_in_block) == GOTCRCW) {
							return ZCOMPL;
						}
						zshhdr(ZNAK, _zmodem.Txhdr);
						continue;

					case ZCOMPL:
						continue;

					case ZFIN:
						ackbibi();
						return ZCOMPL;

					case ZRINIT:
					case ZCAN:
						return ERROR;

				}
			}
		}
		catch (int type)
		{
			/* Set buffer length (0) and capability flags */
			stohdr(0L);

#ifdef CANBREAK
			_zmodem.Txhdr[ZF0] = CANFC32|CANFDX|CANOVIO|CANBRK;
#else
			_zmodem.Txhdr[ZF0] = CANFC32|CANFDX|CANOVIO;
#endif

			if (Zctlesc) _zmodem.Txhdr[ZF0] |= TESCCTL; /* TESCCTL == ESCCTL */

			zshhdr(tryzhdrtype, _zmodem.Txhdr);

			debug(150, "tryzhdrtype = %d\n",tryzhdrtype);
			if (tryzhdrtype == ZSKIP)	/* Don't skip too far */
				tryzhdrtype = ZRINIT;	/* CAF 8-21-87 */
			loop_started = TRUE;
			if (type == 0)  n++;
			continue;
		}
	}

	debug(150,"Serial::tryz: Exiting...\n");
	return rc;
}


//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
/*
 * Receive 1 or more files with ZMODEM protocol
 */
int Serial::rzfiles()
{
	register int c;

	for (int index = 0; index < MAXFILES; index++) {
		if (UserCanceled) return ERROR;
		c = rzfile(index);
		if (reportstop != NULL) reportstop(&reportstruct);
		switch (c) {
			case ZEOF:
				/* FALL THROUGH */
			case ZSKIP:
				switch (tryz()) {
				case ZCOMPL:
					return OK;
				default:
					return ERROR;
				case ZFILE:
					break;
				}
				continue;
			default:
				return c;
			case ERROR:
				return ERROR;
		}
	}
	return ERROR;
}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
/*
 * Receive a file with ZMODEM protocol
 *  Assumes file name frame is in secbuf
 */
int Serial::rzfile(int index)
{
	register int c, n;
	size_t bytes_in_block=0;

	debug(150,"rzfile: Entering...\n");

	_zmodem.zi[index].eof_seen=FALSE;

	n = MaxErrors;

	if (procheader(secbuf,index) == ERROR) {
		return (tryzhdrtype = ZSKIP);
	}

	reportstruct.start_time    = time(NULL);
	reportstruct.filesize      = _zmodem.zi[index].bytes_total;
	reportstruct.filename      = _zmodem.zi[index].fname;
	reportstruct.bytes_skipped = _zmodem.zi[index].bytes_skipped;
	if (reportstart != NULL) reportstart(&reportstruct);
	for (;;) {
		if (UserCanceled) return ERROR;
		stohdr(_zmodem.zi[index].bytes_received);
		zshhdr(ZRPOS, _zmodem.Txhdr);
nxthdr:
		if (UserCanceled) return ERROR;
		c = zgethdr(_zmodem.Rxhdr, 0, NULL);
		switch (c) {
		default:
			return ERROR;
		case ZNAK:
		case TIMEOUT:
			if ( --n < 0) {
				return ERROR;
			}
		case ZFILE:
			zrdata(secbuf, MAX_BLOCK,&bytes_in_block);
			continue;
		case ZEOF:
			if (rclhdr(_zmodem.Rxhdr) != (long) _zmodem.zi[index].bytes_received) {
				/*
				 * Ignore eof if it's at wrong place - force
				 *  a timeout because the eof might have gone
				 *  out before we sent our zrpos.
				 */
				_zmodem.Errors = 0;
				goto nxthdr;
			}
			return c;
		case ERROR:	/* Too much garbage in header search error */
			--n;
			if ( n < 0) return ERROR;
			zmputs(_zmodem.Attn);
			continue;
		case ZSKIP:
			return c;
		case ZDATA:
			if (rclhdr(_zmodem.Rxhdr) != (long) _zmodem.zi[index].bytes_received) {
				if ( --n < 0) return ERROR;
				zmputs(_zmodem.Attn);  continue;
			}
moredata:
			if (UserCanceled) return ERROR;
			//reportstruct.blocks_downloaded = sectnum;
			reportstruct.bytes_downloaded = _zmodem.zi[index].bytes_received;
			reportstruct.elapsed_time = time(NULL) - reportstruct.start_time;
			reportstruct.errors = _zmodem.Errors;
			if (report != NULL) report(&reportstruct);
			if (reportstruct.fail)
				throw(SerialRetryable);
			switch (c = zrdata(secbuf, MAX_BLOCK,&bytes_in_block))
			{
			case ZCAN:
				return ERROR;
			case ERROR:	/* CRC error */
				--n;
				if ( n < 0) return ERROR;
				zmputs(_zmodem.Attn);
				continue;
			case TIMEOUT:
				--n;
				if ( n < 0) return ERROR;
				continue;
			case GOTCRCW:
				n = MaxErrors;
				AddSectorToBuffer( secbuf, bytes_in_block);
				_zmodem.zi[index].bytes_received += bytes_in_block;
				stohdr(_zmodem.zi[index].bytes_received);
				zshhdr(ZACK | 0x80, _zmodem.Txhdr);
				goto nxthdr;
			case GOTCRCQ:
				n = MaxErrors;
				AddSectorToBuffer( secbuf, bytes_in_block);
				_zmodem.zi[index].bytes_received += bytes_in_block;
				stohdr(_zmodem.zi[index].bytes_received);
				zshhdr(ZACK, _zmodem.Txhdr);
				goto moredata;
			case GOTCRCG:
				n = MaxErrors;
				AddSectorToBuffer( secbuf, bytes_in_block);
				_zmodem.zi[index].bytes_received += bytes_in_block;
				goto moredata;
			case GOTCRCE:
				n = MaxErrors;
				AddSectorToBuffer( secbuf, bytes_in_block);
				_zmodem.zi[index].bytes_received += bytes_in_block;
				goto nxthdr;
			}
		}
	}
	if (reportstop != NULL) reportstop(&reportstruct);
	debug(150, "rzfile: Exiting...\n");
}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
/*
 * Send a string to the modem, processing for \336 (sleep 1 sec)
 *   and \335 (break signal)
 */
void Serial::zmputs(const char *s)
{
	const char *p;

	debug(150, "zmputs: Entering...\n");
	while (s && *s)
	{
		p=strpbrk(s,"\335\336");
		if (!p)
		{
			Write((char *) s,strlen(s));
			return;
		}
		if (p!=s)
		{
			Write((char *) s,(size_t) (p-s));
			s=p;
		}
		if (*p=='\336')
			sleep(1);
		else
			tcsendbreak(_fd,0);
		p++;
	}
	debug(150, "zmputs: Exiting...\n");
}

//  ===================================================================
//  NAME:
//
//  PURPOSE:
//
//  RETURNS:
//
//  HISTORY:
//
//  ===================================================================
/*
 * Ack a ZFIN packet, let byegones be byegones
 */
void Serial::ackbibi(void)
{
	int n,d;

	debug(150, "ackbibi: Entering...\n");
	Readnum = 1;
	stohdr(0L);
	for (n=3; --n>=0; ) {
		ClearRXBuffer();
		zshhdr(ZFIN, _zmodem.Txhdr);
		int c = Inkey(1000);
		debug(200, "ackbibi:c = %d\n",c);
		switch (c) {
		case 'O':
			d = Inkey(1000);
			debug(150, "ackbibi:d = %d\n",d);
		//	Inkey(1000);	/* Discard 2nd 'O' */
			return;
		case RCDO:
			return;
		case TIMEOUT:
		default:
			break;
		}
	}
	debug(150, "ackbibi: Exiting...\n");
}

/* End of lrz.c */
